// Direct3DRM.cpp: Implementierung der Klasse CDirect3DRM.
//
//////////////////////////////////////////////////////////////////////

#include "Direct3DRM.h"
#include "Direct3DRMObjects.h"
#include "DirectDraw.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CDirect3DRM::CDirect3DRM()
{
	// ... die Objekte von Anfang an leeren
	lpD3DRM = NULL;
	pDirectDraw = NULL;

	// ... und unabhngiges Objekt erzeugen
	CreateDirect3DRM();
}

CDirect3DRM::CDirect3DRM(CDirectDraw* pDD)
{
	// ... beide Objekte initialisieren
	pDirectDraw = pDD;
	lpD3DRM = NULL;

	// ... und ein Objekt erzeugen
	CreateDirect3DRM();
}

CDirect3DRM::CDirect3DRM(CDirectDraw* pDD, DXDLL_LPDIRECT3DRM lpD3D)
{
	// ... beide Objekte initialisieren
	pDirectDraw = pDD;
	lpD3DRM = NULL;

	// ... aktuelles Objekt setzen
	SetDirect3DObject(lpD3D);
}

CDirect3DRM::~CDirect3DRM()
{
	// ... lschen
	DestroyDirect3DRM();
}

int CDirect3DRM::CreateDirect3DRM()
{
	LPDIRECT3DRM lpD3DRMTmp = NULL;
	HRESULT hres = D3D_OK;

	// altes Objekt lschen
	DestroyDirect3DRM();

	// ... erstmal temporr eine Direct3DRM-Schnittstelle erzeugen
	if (SUCCEEDED(hres = Direct3DRMCreate(&lpD3DRMTmp)))
		// jetzt per Query Schnittstelle Direct3DRM2 holen
		hres = lpD3DRMTmp->QueryInterface(DXDLL_GUID_DIRECT3DRM, (void**)&lpD3DRM);

	// ist altes Interface belegt?
	if (lpD3DRMTmp)
		// altes Interface mu freigegeben werden!
		lpD3DRMTmp->Release();

	// Fehlerstelle zurckliefern
	return hres;
}

void CDirect3DRM::DestroyDirect3DRM()
{
	// existiert dieses Objekt?
	if (lpD3DRM)
	{
		// ... und dieses Objekt zerstren
		lpD3DRM->Release();
		lpD3DRM = NULL;
	}
}

void CDirect3DRM::SetDirect3DObject(DXDLL_LPDIRECT3DRM lpD3D)
{
	// erst altes zerstren
	DestroyDirect3DRM();

	// ... und neues einfach kopieren
	lpD3DRM = lpD3D;
}


CDirect3DRMDevice* CDirect3DRM::CreateDeviceFromSurface(DWORD dwFlags,
														CDirectDrawSurface* pDDS)
{
	CDirect3DRMDevice* pDev = NULL;
	DXDLL_LPDIRECT3DRMDEVICE lpDev = NULL;
	LPDIRECTDRAWSURFACE3 lpSurface;

	// existiert D3D-Objekt bzw. DD-Objekt?
	if (lpD3DRM && pDirectDraw && pDDS)
	{
		// um Surface als Rendertarget zu verwenden wird ein
		// DirectDrawSurface3 bentigt, da DirectDrawSurface4
		// nicht kompatibel zum DirectDrawSurface ist!
		if (FAILED(pDDS->lpSurface->QueryInterface(IID_IDirectDrawSurface3,
				(LPVOID*)&lpSurface)))
			// bei Fehler eben raus
			return NULL;

		// ... dann versuchen zu erstellen
		if (SUCCEEDED(lpD3DRM->CreateDeviceFromSurface(NULL, 
				(LPDIRECTDRAW)pDirectDraw->lpDD, 
				(LPDIRECTDRAWSURFACE)lpSurface, dwFlags, &lpDev)))
		{
			// ... und erstelltes Device mit neuer Klasse attachen
			pDev = new CDirect3DRMDevice(this, lpDev);
			lpSurface->Release();
			// diesen Pointer auf Klasse zurckliefern
			return pDev;
		}
		else
			// Surface freigeben
			lpSurface->Release();
	}

	// ... bei Fehler eben nix gemacht!
	return NULL;
}

CDirect3DRMDevice* CDirect3DRM::CreateDeviceFromSurface(LPGUID lpGUID,
														DWORD dwFlags,
														CDirectDrawSurface* pDDS)
{
	CDirect3DRMDevice* pDev = NULL;
	DXDLL_LPDIRECT3DRMDEVICE lpDev = NULL;
	LPDIRECTDRAWSURFACE3 lpSurface;

	// existiert D3D-Objekt bzw. DD-Objekt?
	if (lpD3DRM && pDirectDraw && pDDS)
	{
		// um Surface als Rendertarget zu verwenden wird ein
		// DirectDrawSurface3 bentigt, da DirectDrawSurface4
		// nicht kompatibel zum DirectDrawSurface ist!
		if (FAILED(pDDS->lpSurface->QueryInterface(IID_IDirectDrawSurface3,
				(LPVOID*)&lpSurface)))
			// bei Fehler eben raus
			return NULL;

		// ... dann versuchen zu erstellen
		if (SUCCEEDED(lpD3DRM->CreateDeviceFromSurface(lpGUID, 
				(LPDIRECTDRAW)pDirectDraw->lpDD,
				(LPDIRECTDRAWSURFACE)lpSurface, dwFlags, &lpDev)))
		{
			// ... und erstelltes Device mit neuer Klasse attachen
			pDev = new CDirect3DRMDevice(this, lpDev);
			lpSurface->Release();
			// diesen Pointer auf Klasse zurckliefern
			return pDev;
		}
		else
			// Surface freigeben
			lpSurface->Release();
	}

	// ... bei Fehler eben nix gemacht!
	return NULL;
}

CDirect3DRMDevice* CDirect3DRM::CreateDeviceFromClipper(CDirectDrawClipper* pClipper,
														int iWidth, int iHeight)
{
	CDirect3DRMDevice* pDev = NULL;
	DXDLL_LPDIRECT3DRMDEVICE lpDev = NULL;

	// existiert D3D-Objekt bzw. DD-Objekt?
	if (lpD3DRM && pClipper && pClipper->lpDDClipper)
	{
		// ... dann versuchen zu erstellen
		if (SUCCEEDED(lpD3DRM->CreateDeviceFromClipper( 
				pClipper->lpDDClipper, NULL, iWidth, iHeight, &lpDev)))
		{
			// ... und erstelltes Device mit neuer Klasse attachen
			pDev = new CDirect3DRMDevice(this, lpDev);
			// diesen Pointer auf Klasse zurckliefern
			return pDev;
		}
	}

	// ... bei Fehler eben nix gemacht!
	return NULL;
}

CDirect3DRMDevice* CDirect3DRM::CreateDeviceFromClipper(LPGUID lpGUID,
														CDirectDrawClipper* pClipper,
														int iWidth, int iHeight)
{
	CDirect3DRMDevice* pDev = NULL;
	DXDLL_LPDIRECT3DRMDEVICE lpDev = NULL;

	// existiert D3D-Objekt bzw. DD-Objekt?
	if (lpD3DRM && pClipper)
	{
		// ... dann versuchen zu erstellen
		if (SUCCEEDED(lpD3DRM->CreateDeviceFromClipper( 
				pClipper->lpDDClipper, lpGUID, iWidth, iHeight, &lpDev)))
		{
			// ... und erstelltes Device mit neuer Klasse attachen
			pDev = new CDirect3DRMDevice(this, lpDev);
			// diesen Pointer auf Klasse zurckliefern
			return pDev;
		}
	}

	// ... bei Fehler eben nix gemacht!
	return NULL;
}

CDirect3DRMFrame* CDirect3DRM::CreateScene()
{
	CDirect3DRMFrame* pFrame = NULL;
	DXDLL_LPDIRECT3DRMFRAME lpFrame = NULL;

	// existiert D3D-Objekt?
	if (lpD3DRM)
	{
		// ... dann versuchen zu erstellen
		if (SUCCEEDED(lpD3DRM->CreateFrame(NULL, &lpFrame)))
		{
			// ... und erstellte Scene mit neuer Klasse attachen
			pFrame = new CDirect3DRMFrame(this, lpFrame);
			// diesen Pointer auf Klasse zurckliefern
			return pFrame;
		}
	}

	// ... bei Fehler eben nix gemacht!
	return NULL;
}

CDirect3DRMFrame* CDirect3DRM::CreateFrame(CDirect3DRMFrame* pParentFrame)
{
	CDirect3DRMFrame* pFrame = NULL;
	DXDLL_LPDIRECT3DRMFRAME lpFrame = NULL;

	// existiert D3D-Objekt?
	if (lpD3DRM && pParentFrame)
	{
		// ... dann versuchen zu erstellen
		if (SUCCEEDED(lpD3DRM->CreateFrame(pParentFrame->lpD3DRMFrame, 
				&lpFrame)))
		{
			// ... und erstellte Scene mit neuer Klasse attachen
			pFrame = new CDirect3DRMFrame(this, lpFrame);
			// diesen Pointer auf Klasse zurckliefern
			return pFrame;
		}
	}

	// ... bei Fehler eben nix gemacht!
	return NULL;
}

CDirect3DRMViewport* CDirect3DRM::CreateViewport(CDirect3DRMDevice *pDevice,
												 CDirect3DRMFrame *pCamera)
{
	HRESULT hres = 0;
	char test[256];

	// alles gltige Werte
	if (pDevice && pCamera && lpD3DRM)
	{
		// auch der Frame / Device gltig belegt?
		if (pDevice->lpD3DRMDevice && pCamera->lpD3DRMFrame)
		{
			DXDLL_LPDIRECT3DRMVIEWPORT lpView;
			CDirect3DRMViewport* pView;
			int iWidth = pDevice->GetWidth();
			int iHeight = pDevice->GetHeight();

			// dann Viewport erzeugen
			if (SUCCEEDED(hres = lpD3DRM->CreateViewport(pDevice->lpD3DRMDevice,
								pCamera->lpD3DRMFrame, 0, 0, iWidth, iHeight,
								&lpView)))
			{
				// ... noch eine Viewportklasse erzeugen
				pView = new CDirect3DRMViewport(pDevice, lpView);
				// ... und diese zurckliefern
				return pView;
			}
			else
			{
				wsprintf(test, "%lu \n", hres);
				WriteLOGLine(test);
			}
		}
	}

	return NULL;
}

CDirect3DRMViewport* CDirect3DRM::CreateViewport(CDirect3DRMDevice *pDevice,
												 CDirect3DRMFrame *pCamera,
												 int ix, int iy, int iWidth,
												 int iHeight)
{
	// alles gltige Werte
	if (pDevice && pCamera && lpD3DRM)
	{
		// auch der Frame / Device gltig belegt?
		if (pDevice->lpD3DRMDevice && pCamera->lpD3DRMFrame)
		{
			DXDLL_LPDIRECT3DRMVIEWPORT lpView;
			CDirect3DRMViewport* pView;

			// dann Viewport erzeugen
			if (SUCCEEDED(lpD3DRM->CreateViewport(pDevice->lpD3DRMDevice,
								pCamera->lpD3DRMFrame, ix, iy, iWidth, iHeight,
								&lpView)))
			{
				// ... noch eine Viewportklasse erzeugen
				pView = new CDirect3DRMViewport(pDevice, lpView);
				// ... und diese zurckliefern
				return pView;
			}
		}
	}

	return NULL;
}

CDirect3DRMFace* CDirect3DRM::CreateFace()
{
	// existiert die Direct3DRM-Schnittstelle?
	if (lpD3DRM)
	{
		DXDLL_LPDIRECT3DRMFACE lpFace;
		CDirect3DRMFace* pFace;

		// ... dann ein Face erzeugen
		if (SUCCEEDED(lpD3DRM->CreateFace(&lpFace)))
		{
			// ... jetzt noch die Klasse erzeugen
			pFace = new CDirect3DRMFace(lpFace);
			// und Ergebnis zurckliefern
			return pFace;
		}
	}

	return NULL;
}

CDirect3DRMMeshBuilder* CDirect3DRM::CreateMeshBuilder()
{
	// existiert die Direct3DRM-Schnittstelle?
	if (lpD3DRM)
	{
		DXDLL_LPDIRECT3DRMMESHBUILDER lpMeshBuilder;
		CDirect3DRMMeshBuilder* pMeshBuilder;

		// ... dann ein Face erzeugen
		if (SUCCEEDED(lpD3DRM->CreateMeshBuilder(&lpMeshBuilder)))
		{
			// ... jetzt noch die Klasse erzeugen
			pMeshBuilder = new CDirect3DRMMeshBuilder(lpMeshBuilder);
			// und Ergebnis zurckliefern
			return pMeshBuilder;
		}
	}

	return NULL;
}

CDirect3DRMMaterial* CDirect3DRM::CreateMaterial(D3DVALUE d3dPower)
{
	// existiert die Direct3DRM-Schnittstelle?
	if (lpD3DRM)
	{
		DXDLL_LPDIRECT3DRMMATERIAL lpMaterial;
		CDirect3DRMMaterial* pMaterial;

		// ... dann ein Material erzeugen
		if (SUCCEEDED(lpD3DRM->CreateMaterial(d3dPower, &lpMaterial)))
		{
			// ... jetzt noch die Klasse erzeugen
			pMaterial = new CDirect3DRMMaterial(lpMaterial);
			// und Ergebnis zurckliefern
			return pMaterial;
		}
	}

	return NULL;
}

CDirect3DRMLight* CDirect3DRM::CreateLight(D3DRMLIGHTTYPE d3drmltLightType,
										   D3DCOLOR cColor)
{
	// existiert die Direct3DRM-Schnittstelle?
	if (lpD3DRM)
	{
		LPDIRECT3DRMLIGHT lpLight;
		CDirect3DRMLight* pLight;

		// ... dann ein Licht erzeugen
		if (SUCCEEDED(lpD3DRM->CreateLight(d3drmltLightType,
				cColor, &lpLight)))
		{
			// ... jetzt noch die Klasse erzeugen
			pLight = new CDirect3DRMLight(lpLight);
			// und Ergebnis zurckliefern
			return pLight;
		}
	}

	return NULL;
}

CDirect3DRMLight* CDirect3DRM::CreateLightRGB(D3DRMLIGHTTYPE d3drmltLightType,
											  D3DVALUE d3dRed, D3DVALUE d3dGreen,
											  D3DVALUE d3dBlue)
{
	// existiert die Direct3DRM-Schnittstelle?
	if (lpD3DRM)
	{
		LPDIRECT3DRMLIGHT lpLight;
		CDirect3DRMLight* pLight;

		// ... dann ein Licht erzeugen
		if (SUCCEEDED(lpD3DRM->CreateLightRGB(d3drmltLightType,
				d3dRed, d3dGreen, d3dBlue, &lpLight)))
		{
			// ... jetzt noch die Klasse erzeugen
			pLight = new CDirect3DRMLight(lpLight);
			// und Ergebnis zurckliefern
			return pLight;
		}
	}

	return NULL;
}

int CDirect3DRM::Tick(D3DVALUE d3dvalTick)
{
	// ... und Befehl auslsen
	return lpD3DRM->Tick(d3dvalTick);
}

int CDirect3DRM::AddSearchPath(LPCSTR lpPath)
{
	// existiert Objekt eigentlich?
	if (lpD3DRM)
		// ... dann Pfad hinzufgen
		return lpD3DRM->AddSearchPath(lpPath);

	// ... ansonsten nur Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRM::SetSearchPath(LPCSTR lpPath)
{
	// existiert Objekt eigentlich?
	if (lpD3DRM)
		// ... dann Pfad neu setzen
		return lpD3DRM->SetSearchPath(lpPath);

	// ... ansonsten nur Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRM::GetSearchPath(DWORD* lpdwSize, LPSTR lpszPath)
{
	// existiert Objekt eigentlich?
	if (lpD3DRM)
		// ... dann Pfad ermitteln
		return lpD3DRM->GetSearchPath(lpdwSize, lpszPath);

	// ... ansonsten nur Fehler
	return DXDLL_INTERNALERROR;
}

CDirect3DRMTexture* CDirect3DRM::CreateTexture(LPD3DRMIMAGE lpImage)
{
	// existiert Objekt eigentlich?
	if (lpD3DRM)
	{
		DXDLL_LPDIRECT3DRMTEXTURE lpTexture;
		CDirect3DRMTexture* pTexture;

		// ... erzeugen
		if (SUCCEEDED(lpD3DRM->CreateTexture(lpImage, &lpTexture)))
		{
			// ... noch dazugehrige Klasse erzeugen
			pTexture = new CDirect3DRMTexture(lpTexture);
			// diese zurckliefern
			return pTexture;
		}
	}

	// ... ansonsten nur Fehler
	return NULL;
}

CDirect3DRMTexture* CDirect3DRM::CreateTextureFromSurface(CDirectDrawSurface* pDDS)
{
	// existiert Objekt eigentlich?
	if (lpD3DRM && pDDS)
	{
		if (pDDS->lpSurface)
		{
			DXDLL_LPDIRECT3DRMTEXTURE lpTexture;
			CDirect3DRMTexture* pTexture;

			// ... erzeugen
			if (SUCCEEDED(lpD3DRM->CreateTextureFromSurface(
					(LPDIRECTDRAWSURFACE)pDDS->lpSurface, &lpTexture)))
			{
				// ... noch dazugehrige Klasse erzeugen
				pTexture = new CDirect3DRMTexture(lpTexture);
				// diese zurckliefern
				return pTexture;
			}
		}
	}

	// ... ansonsten nur Fehler
	return NULL;
}

CDirect3DRMTexture* CDirect3DRM::LoadTexture(const char * lpFileName)
{
	// existiert Objekt eigentlich?
	if (lpD3DRM)
	{
		DXDLL_LPDIRECT3DRMTEXTURE lpTexture;
		CDirect3DRMTexture* pTexture;

		// ... erzeugen
		if (SUCCEEDED(lpD3DRM->LoadTexture(lpFileName, &lpTexture)))
		{
			// ... noch dazugehrige Klasse erzeugen
			pTexture = new CDirect3DRMTexture(lpTexture);
			// diese zurckliefern
			return pTexture;
		}
	}

	// ... ansonsten nur Fehler
	return NULL;
}

CDirect3DRMTexture* CDirect3DRM::LoadTextureFromResource(HMODULE hModule,
														 LPCTSTR strName,
														 LPCTSTR strType)
{
	// existiert Objekt eigentlich?
	if (lpD3DRM)
	{
		DXDLL_LPDIRECT3DRMTEXTURE lpTexture;
		CDirect3DRMTexture* pTexture;

		// ... erzeugen
		if (SUCCEEDED(lpD3DRM->LoadTextureFromResource(hModule, strName,
				strType, &lpTexture)))
		{
			// ... noch dazugehrige Klasse erzeugen
			pTexture = new CDirect3DRMTexture(lpTexture);
			// diese zurckliefern
			return pTexture;
		}
	}

	// ... ansonsten nur Fehler
	return NULL;
}

int CDirect3DRM::SetDefaultTextureColors(DWORD dwColors)
{
	// existiert Objekt eigentlich?
	if (lpD3DRM)
		// ... dann standardmige Farbenanzahl angeben
		return lpD3DRM->SetDefaultTextureColors(dwColors);

	// ... ansonsten nur Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRM::SetDefaultTextureShades(DWORD dwShades)
{
	// existiert Objekt eigentlich?
	if (lpD3DRM)
		// ... dann standardmige Schattenanzahl angeben
		return lpD3DRM->SetDefaultTextureShades(dwShades);

	// ... ansonsten nur Fehler
	return DXDLL_INTERNALERROR;
}

CDirect3DRMWrap* CDirect3DRM::CreateWrap(D3DRMWRAPTYPE type, 
										 CDirect3DRMFrame* pRef,
										 D3DVALUE ox, D3DVALUE oy,
										 D3DVALUE oz, D3DVALUE dx,
										 D3DVALUE dy, D3DVALUE dz,
										 D3DVALUE ux, D3DVALUE uy,
										 D3DVALUE uz, D3DVALUE ou,
										 D3DVALUE ov, D3DVALUE su,
										 D3DVALUE sv)
{
	// existiert Objekt eigentlich?
	if (lpD3DRM && pRef)
	{
		if (pRef->lpD3DRMFrame)
		{
			LPDIRECT3DRMWRAP lpWrap;
			CDirect3DRMWrap* pWrap;

			// ... erzeugen
			if (SUCCEEDED(lpD3DRM->CreateWrap(type, pRef->lpD3DRMFrame,
					ox, oy, oz, dx, dy, dz, ux, uy, uz, ou, ov, su, sv,
					&lpWrap)))
			{
				// ... noch dazugehrige Klasse erzeugen
				pWrap = new CDirect3DRMWrap(lpWrap);
				// diese zurckliefern
				return pWrap;
			}
		}
	}

	// ... ansonsten nur Fehler
	return NULL;
}

