// Direct3DRM.h: Schnittstelle fr alle Klassen von Direct3DRM.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DIRECT3DRM_H__D95ED900_C590_11D2_AD5A_D7D8FA41524B__INCLUDED_)
#define AFX_DIRECT3DRM_H__D95ED900_C590_11D2_AD5A_D7D8FA41524B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// alle mglichen Defines
#include "DXDefines.h"
// alle mglichen Tools
#include "Tools.h"
// Direct3D-Verwaltung
#include "d3d.h"
#include "d3dcaps.h"
#include "d3drm.h"
#include "d3drmdef.h"
#include "d3drmobj.h"
#include "d3drmwin.h"
#include "d3dtypes.h"


// -------------------------------------------
// Forwarddeklarationen der einzelnen Klassen
// -------------------------------------------
class CDirect3DRM;
class CDirect3DRMDevice;
class CDirect3DRMFrame;
class CDirect3DRMViewport;

// nur, um ordentlich arbeiten zu knnen
class CDirectDraw;
class CDirectDrawSurface;
class CDirectDrawClipper;
class CDirect3DRMMeshBuilder;
class CDirect3DRMViewport;
class CDirect3DRMFace;
class CDirect3DRMMaterial;
class CDirect3DRMLight;
class CDirect3DRMWrap;
class CDirect3DRMTexture;


// ----------------------------------------------------
// richtige komplette Definionen der einzelnen Klassen
// ----------------------------------------------------
class __declspec(dllexport) CDirect3DRM
{
public:
	// erzeugt Klasse und verbindet diese mit einer unabhngigen
	// Direct3DRM-Schnittstelle
	CDirect3DRM();
	// erzeugt Klasse und verbindet diese mit Direct3DRM-Schnittstelle
	CDirect3DRM(CDirectDraw* pDD);
	// erzeugt Klasse und verbindet diese mit angegebener
	// Direct3DRM-Schnittstelle
	CDirect3DRM(CDirectDraw* pDD, DXDLL_LPDIRECT3DRM lpD3D);
	virtual ~CDirect3DRM();

	// erzeugt ein Direct3D-Objekt
	int CreateDirect3DRM();
	// lscht dieses Direct3D-Objekt
	void DestroyDirect3DRM();
	// Attached ein vorhandenes D3D-Objekt zur Klasse
	void SetDirect3DObject(DXDLL_LPDIRECT3DRM lpD3D);

	// erzeugt ein Device aus Surface und attached es mit Klasse
	CDirect3DRMDevice* CreateDeviceFromSurface(DWORD dwFlags,
		CDirectDrawSurface* pDDS);
	// erzeugt ein Device aus beliebiger GUID und Surface
	CDirect3DRMDevice* CreateDeviceFromSurface(LPGUID lpGUID,
		DWORD dwFlags, CDirectDrawSurface* pDDS);

	// erzeugt ein Device aus Clipper und attached es mit Klasse
	CDirect3DRMDevice* CreateDeviceFromClipper(CDirectDrawClipper* 
		pClipper, int iWidth, int iHeight);
	// erzeugt ein Device aus beliebiger GUID und Clipper
	CDirect3DRMDevice* CreateDeviceFromClipper(LPGUID lpGUID,
		CDirectDrawClipper* pClipper, int iWidth, int iHeight);

	// erzeugt eine neue Szene
	CDirect3DRMFrame* CreateScene();
	// erzeugt ein neues Frame fr beliebige Szene
	CDirect3DRMFrame* CreateFrame(CDirect3DRMFrame* pParentFrame);

	// erzeugt ein Viewport, welches an Device gebunden ist und
	// eine Blickrichtung mit der Kamera hat
	// ACHTUNG:
	// Viewport hat genau dieselbe Gre, wie Device
	CDirect3DRMViewport* CreateViewport(CDirect3DRMDevice* pDevice, 
		CDirect3DRMFrame* pCamera);
	// erzeugt ein Viewport, welches an Device gebunden ist und
	// eine Blickrichtung mit der Kamera hat
	CDirect3DRMViewport* CreateViewport(CDirect3DRMDevice *pDevice,
		CDirect3DRMFrame *pCamera, int ix, int iy, int iWidth,
		int iHeight);

	// erzeugt ein Face (= Polygonenzug)
	CDirect3DRMFace* CreateFace();

	// erzeugt einen MeshBuilder
	CDirect3DRMMeshBuilder* CreateMeshBuilder();

	// erzeugt ein Material
	CDirect3DRMMaterial* CreateMaterial(D3DVALUE d3dPower);

	// erzeugt ein Licht
	CDirect3DRMLight* CreateLight(D3DRMLIGHTTYPE d3drmltLightType,
		D3DCOLOR cColor);
	// erzeugt ein Licht via RGB-Werte
	CDirect3DRMLight* CreateLightRGB(D3DRMLIGHTTYPE d3drmltLightType,
		D3DVALUE d3dRed, D3DVALUE d3dGreen, D3DVALUE d3dBlue);

	// wartet eine angegebene Zahl Systemtakte und rendert nachher
	// ALLE vorhandenen Frames komplett neu
	// ACHTUNG:
	// Keine Sicherheitschecks!
	int Tick(D3DVALUE d3dvalTick);

	// fgt einen weiteren Suchpfad zu allen Pfaden hinzu
	int AddSearchPath(LPCSTR lpPath);
	// setzt Suchpfad komplett neu
	int SetSearchPath(LPCSTR lpPath);
	// ermittelt kompletten aktuellen Suchpfad
	int GetSearchPath(DWORD* lpdwSize, LPSTR lpszPath);

	// erzeugt eine Texture mit den angegebenen Attributen
	CDirect3DRMTexture* CreateTexture(LPD3DRMIMAGE lpImage);
	// erzeugt eine Texture aus dem angegebenen Surface
	CDirect3DRMTexture* CreateTextureFromSurface(CDirectDrawSurface* pDDS);
	// ldt eine Texture von Platte
	CDirect3DRMTexture* LoadTexture(const char * lpFileName);
	// erzeugt eine Texture aus einer Resource
	CDirect3DRMTexture* LoadTextureFromResource(HMODULE hModule,
		LPCTSTR strName, LPCTSTR strType);
	// setzt die standardmige Anzahl von Farben der Texture
	int SetDefaultTextureColors(DWORD dwColors);
	// setzt die standardmige Anzahl von Schatten der Texture
	int SetDefaultTextureShades(DWORD dwShades);

	// erzeugt ein Wrap-Objekt, um Texturekoordinaten richtig
	// berechnen zu knnen
	CDirect3DRMWrap* CreateWrap(D3DRMWRAPTYPE type, CDirect3DRMFrame* pRef,
		D3DVALUE ox, D3DVALUE oy, D3DVALUE oz, D3DVALUE dx, D3DVALUE dy,
		D3DVALUE dz, D3DVALUE ux, D3DVALUE uy, D3DVALUE uz, D3DVALUE ou,
		D3DVALUE ov, D3DVALUE su, D3DVALUE sv);

	// ... und hier das Direct3DRM-Objekt
	DXDLL_LPDIRECT3DRM lpD3DRM;
	CDirectDraw* pDirectDraw;
};

class __declspec(dllexport) CDirect3DRMDevice  
{
public:
	CDirect3DRMDevice();
	CDirect3DRMDevice(CDirect3DRM* pD3DRM);
	CDirect3DRMDevice(CDirect3DRM* pD3DRM,
		DXDLL_LPDIRECT3DRMDEVICE lpDev);
	virtual ~CDirect3DRMDevice();

	// lscht dieses D3D-Device
	void DestroyDirect3DDevice();

	// updatet aktuelles Bild
	// ACHTUNG:
	// keine Sicherheitschecks!
	int Update();

	// liefert Breite des angelegten Devices
	int GetWidth();
	// liefert Hhe des angelegten Devices
	int GetHeight();

	// setzt die Anzahl der Hintergrundbuffer
	int SetBufferCount(DWORD dwBufferSize);
	// ermittelt die Anzahl der Hintergrundbuffer
	int GetBufferCount();

	// ermittelt das aktuelle Farbenmodell
	int GetColorModel(D3DCOLORMODEL* lpD3DModel);

	// setzt das Dithering-Flag
	int SetDither(BOOL bDither);
	// ermittelt das Dithering-Flag
	int GetDither(BOOL* bDither);

	// ermittelt die Anzahl bereits gezeichneter Dreiecke
	// seit der Device erzeugt wurde
	int GetTrianglesDrawn();

	// ermittelt die aktuellen Wireframe-Optionen
	int GetWireframeOptions();

	// setzt die Render-Qualitt des Devices
	int SetQuality(D3DRMRENDERQUALITY d3dQuality);
	// ermittelt die Render-Qualitt des Devices
	int GetQuality(D3DRMRENDERQUALITY* d3dQuality);

	// setzt die maximale Anzahl Schatten im RAMP-Farbmodell
	int SetShades(DWORD dwShades);
	// ermittelt die maximale Anzahl Schatten im RAMP-Farbmodell
	int GetShades(DWORD* dwShades);

	// setzt die Texture-Render-Qualitt des Devices
	int SetTextureQuality(D3DRMTEXTUREQUALITY d3dTexQuality);
	// ermittelt die Texture-Render-Qualitt des Devices
	int GetTextureQuality(D3DRMTEXTUREQUALITY* d3dTexQuality);

	// setzt einige Renderoptionen
	int SetRenderMode(DWORD dwFlags);
	// ermittelt einige Renderoptionen
	int GetRenderMode(DWORD* dwFlags);

	// erzeugt eine Rckruffunktion, die anzeigt, da
	// der Device geupdatet wird
	int AddUpdateCallback(DXDLL_D3DRMUPDATECALLBACK d3drmUpdateProc,
		LPVOID arg);
	// lscht eine Rckruffunktion, die anzeigt, da
	// der Device geupdatet wird
	int DeleteUpdateCallback(DXDLL_D3DRMUPDATECALLBACK d3drmUpdateProc,
		LPVOID arg);

	// noch das Device zu Klasse
	DXDLL_LPDIRECT3DRMDEVICE lpD3DRMDevice;
	CDirect3DRM* pDirect3DRM;
};

class __declspec(dllexport) CDirect3DRMFrame
{
public:
	CDirect3DRMFrame();
	CDirect3DRMFrame(CDirect3DRM* pD3DRM);
	CDirect3DRMFrame(CDirect3DRM* pD3DRM, 
		DXDLL_LPDIRECT3DRMFRAME lpFrame);
	~CDirect3DRMFrame();

	// zerstrt kompletten Frame wieder
	void DestroyDirect3DRMFrame();

	// bewegt Objekte innerhalb Frame
	int Move(D3DVALUE d3dval);

	// setzt Ausrichtungsvektoren der Achsen
	int SetAxes(D3DVALUE dx, D3DVALUE dy, D3DVALUE dz, D3DVALUE ux,
		D3DVALUE uy, D3DVALUE uz);
	// ermittelt Ausrichtungsvektoren der Achsen
	int GetAxes(D3DVECTOR* dir, D3DVECTOR* up);

	// gibt an, ob Achsen aus Parent bernommen werden sollen
	int SetInheritAxes(BOOL inherit_from_parent);
	// ermittelt, ob Achsen aus Parent bernommen werden sollen
	int GetInheritAxes(BOOL* bInherit);

	// setzt die Hintergrundfarbe des Frames
	int SetSceneBackground(D3DCOLOR rcColor);
	// ermittelt Hintergrundfarbe des Frames
	int GetSceneBackground(D3DCOLOR* d3dColor);
	// setzt den Hintergrundsurface des Frames
	int SetSceneBackgroundDepth(CDirectDrawSurface* pImage);
	// ermittelt Hintergrundsurface des Frames
	int GetSceneBackgroundDepth(LPDIRECTDRAWSURFACE lpDDSurface);
	// setzt die Hintergrundtexture des Frames
	int SetSceneBackgroundImage(CDirect3DRMTexture* pTexture);
	// setzt Hintergrundfarbe des Frames den RGB-Werten entsprechend
	int SetSceneBackgroundRGB(D3DVALUE rvRed, D3DVALUE rvGreen,
		D3DVALUE rvBlue);

	// setzt uerste Koordinaten des Frames
	int SetBox(D3DRMBOX* lpD3DRMBox);
	// ermittelt die uersten Koordinaten des Frames
	int GetBox(D3DRMBOX* lpD3DRMBox);
	// setzt/lscht das Flag, um angegebene Boxwerte in Rendering
	// mit einzubeziehen
	int SetBoxEnable(BOOL bEnableFlag);
	// ermittelt das entsprechende Flag
	int GetBoxEnable(BOOL* bBoxEnable);
	// ermittelt uerste Koordinaten aller untergeordneten Frames 
	int GetHierarchyBox(D3DRMBOX* lpD3DRMBox);

	// setzt Farbe des Frames
	int SetColor(D3DCOLOR rcColor);
	// setzt Farbe des Frames per RGB-Werte
	int SetColorRGB(D3DVALUE rvRed, D3DVALUE rvGreen, D3DVALUE rvBlue);
	// ermittelt Farbe des Frames
	int GetColor(D3DCOLOR* d3dColor);

	// setzt die Farbe fr Fogging
	int SetSceneFogColor(D3DCOLOR rcColor);
	// ermittelt die Farbe fr Fogging
	int GetSceneFogColor(D3DCOLOR* d3dColor);
	// setzt Flag, ob Fog fr Frame aktiviert ist
	int SetSceneFogEnable(BOOL bEnable);
	// ermittelt, ob Fog fr Frame aktiviert ist
	int GetSceneFogEnable(BOOL* bFogEnable);
	// setzt den aktuellen Foggingmodus
	int SetSceneFogMode(D3DRMFOGMODE rfMode);
	// ermittelt den aktuellen Foggingmodus
	int GetSceneFogMode(D3DRMFOGMODE* lpFogMode);
	// setzt die aktuellen Fogging-Parameter dieser Szene
	int SetSceneFogParams(D3DVALUE rvStart, D3DVALUE rvEnd, 
		D3DVALUE rvDensity);
	// ermittelt die aktuellen Fogging-Parameter dieser Szene
	int GetSceneFogParams(D3DVALUE* lprvStart, D3DVALUE* lprvEnd,
		D3DVALUE* lprvDensity);

	// fgt ein Frame in dieses ein
	int AddChild(CDirect3DRMFrame* pD3DRMFrameChild);	
	// lscht ein Frame aus diesem
	int DeleteChild(CDirect3DRMFrame* pD3DRMFrameChild);	
	
	// fgt Lichtquelle zu Frame hinzu
	int AddLight(CDirect3DRMLight* pD3DRMLight);
	// lscht Lichtquelle aus Frame
	int DeleteLight(CDirect3DRMLight* pD3DRMLight);

	// ldt einen kompletten Frame von Platte
	int Load(LPVOID lpvObjSource, LPVOID lpvObjID, 
		DXDLL_D3DRMLOADOPTIONS d3drmLOFlags, 
		DXDLL_D3DRMLOADTEXTURECALLBACK d3drmLoadTextureProc,
		LPVOID lpArgLTP);

	// setzt eine Materialdefinition fr Frame
	int SetMaterial(CDirect3DRMMaterial* pMaterial);

	// setzt den Materialienmodus des Frames
	int SetMaterialMode(D3DRMMATERIALMODE rmmMode);
	// ermittelt den Materialienmodus des Frames
	int GetMaterialMode(D3DRMMATERIALMODE* lpMaterial);

	// fgt eine Rckruffunktion bei Bewegung hinzu
	int AddMoveCallback(DXDLL_D3DRMFRAMEMOVECALLBACK d3drmFMC,
		VOID* lpArg, DWORD dwFlags);
	// lscht eine Rckruffunktion
	int DeleteMoveCallback(DXDLL_D3DRMFRAMEMOVECALLBACK d3drmFMC,
		VOID* lpArg);

	// fgt eine Rotierung zum Frame hinzu
	int AddRotation(D3DRMCOMBINETYPE rctCombine, D3DVALUE rvX,
		D3DVALUE rvY, D3DVALUE rvZ, D3DVALUE rvTheta);
	// fgt eine Skalierung zum Frame hinzu
	int AddScale(D3DRMCOMBINETYPE rctCombine, D3DVALUE rvX,
		D3DVALUE rvY, D3DVALUE rvZ);
	// fgt eine Translation zum Frame hinzu
	int AddTranslation(D3DRMCOMBINETYPE rctCombine, D3DVALUE rvX,
		D3DVALUE rvY, D3DVALUE rvZ);

	// setzt aktuelle Orientation relativ zum Referenzframe
	int SetOrientation(CDirect3DRMFrame* pRef, D3DVALUE rvDx,
		D3DVALUE rvDy, D3DVALUE rvDz, D3DVALUE rvUx, D3DVALUE rvUy,
		D3DVALUE rvUz);
	// ermittelt aktuelle Orientation relativ zum Referenzframe
	int GetOrientation(CDirect3DRMFrame* pRef, LPD3DVECTOR lprvDir,
		LPD3DVECTOR lprvUp);

	// positioniert dieses Frame relativ zu angegebenen Frame
	int SetPosition(CDirect3DRMFrame* pRef, D3DVALUE d3dX, 
		D3DVALUE d3dY, D3DVALUE d3dZ);
	// ermittelt aktuelle Position relativ zum Referenzframe
	int GetPosition(CDirect3DRMFrame* pRef, LPD3DVECTOR lprvPos);

	// setzt aktuelle Rotierung relativ zum Referenzframe
	int SetRotation(CDirect3DRMFrame* pRef, D3DVALUE rvX, D3DVALUE rvY,
		D3DVALUE rvZ, D3DVALUE rvTheta);
	// ermittelt aktuelle Rotierung relativ zum Referenzframe
	int GetRotation(CDirect3DRMFrame* pRef, LPD3DVECTOR lprvAxis,
		LPD3DVALUE lprvTheta);

	// setzt aktuelle Velocity relativ zum Referenzframe
	int SetVelocity(CDirect3DRMFrame* pRef, D3DVALUE rvX, D3DVALUE rvY,
		D3DVALUE rvZ, BOOL fRotVel);
	// ermittelt aktuelle Velocity relativ zum Referenzframe
	int GetVelocity(CDirect3DRMFrame* pRef, LPD3DVECTOR lprvVel,
		BOOL fRotVel);

	int SetQuaternion(CDirect3DRMFrame* pRef, D3DRMQUATERNION *quat);

	// stellt den Frame einem Target gegenber relativ zum Referenzframe
	// und blockiert Rotierung ber angegebene Flags
	int LookAt(CDirect3DRMFrame* pTarget, CDirect3DRMFrame* pRef,
		D3DRMFRAMECONSTRAINT rfcConstraint);

	// setzt den zu verwendenden Sortmode
	int SetSortMode(D3DRMSORTMODE d3drmSM);
	// ermittelt aktuellen Sortmode
	int GetSortMode(D3DRMSORTMODE* lpSortMode);

	// setzt den zu verwendenden ZBuffer-Mode
	int SetZbufferMode(D3DRMZBUFFERMODE d3drmZBM);
	// ermittelt aktuellen ZBuffer-Mode
	int GetZbufferMode(D3DRMZBUFFERMODE* lpZBufMode);

	// setzt die Texture dieses Frames
	int SetTexture(CDirect3DRMTexture* pD3DRMTexture);

	// transformiert den Frame mittels einer 4D-Matrix
	int AddTransform(D3DRMCOMBINETYPE rctCombine, D3DRMMATRIX4D rmMatrix);
	// ermittelt aktuelle Transformierung
	int GetTransform(D3DRMMATRIX4D rmMatrix);
	// ermittelt aktuelle Transformierung bezogen auf angegebene Szene
	int GetTransform(CDirect3DRMFrame* pFrame, D3DRMMATRIX4D rmMatrix);
	// kehrt die aktuelle Transformierung um
	int InverseTransform(D3DVECTOR* lprvDst, D3DVECTOR* lprvSrc);
	// transformiert Weltkoordinaten
	int Transform(D3DVECTOR* lpd3dVDst, D3DVECTOR* lpd3dVSrc);

	// fgt ein visuelles Objekt dem Frame hinzu
	// Konvertierung: <Objekt>->GetVisual();
	// ACHTUNG:
	// Objekt kann nach hinzufgen zum Frame
	// wieder freigegeben werden, falls keine
	// Manipulation der Rotation, Translation,
	// ... gewnscht wird
	int AddVisual(LPDIRECT3DRMVISUAL lpD3DRMVisual);
	// lscht ein visuelles Objekt aus dem Frame
	// Konvertierung: <Objekt>->GetVisual();
	int DeleteVisual(LPDIRECT3DRMVISUAL lpD3DRMVisual);

	CDirect3DRM* pDirect3DRM;
	DXDLL_LPDIRECT3DRMFRAME lpD3DRMFrame;
};

class __declspec(dllexport) CDirect3DRMViewport  
{
public:
	// alle Werte werden auf NULL gesetzt
	CDirect3DRMViewport();
	// der Device wird vorbelegt
	CDirect3DRMViewport(CDirect3DRMDevice* pDev);
	// Device und das Viewport werden vorbelegt
	// ACHTUNG:
	// wird im Normalfall kaum bentigt
	CDirect3DRMViewport(CDirect3DRMDevice* pDev,
		DXDLL_LPDIRECT3DRMVIEWPORT lpViewport);
	// zerstrt Klasse
	virtual ~CDirect3DRMViewport();

	// zerstrt aktuelles Viewport-Interface
	void DestroyDirect3DRMViewport();

	// setzt das Cameraframe des Viewports
	int SetCamera(CDirect3DRMFrame* pCam);
	// ermittelt aktuelle Camera
	CDirect3DRMFrame* GetCamera();

	// setzt die  Position der back-clipping-plane
	// entlang der z-Achse
	int SetBack(D3DVALUE d3dval);
	// setzt die  Position der front-clipping-plane
	// entlang der z-Achse
	int SetFront(D3DVALUE d3dval);
	// setzt die Auflsung der front-clipping-plane entlang
	// der z-Achse
	int SetPlane(D3DVALUE d3dLeft, D3DVALUE d3dRight,
		D3DVALUE d3dBottom, D3DVALUE d3dTop);

	// ermittelt die  Position der back-clipping-plane
	// entlang der z-Achse
	int GetBack(D3DVALUE* d3dval);
	// ermittelt die  Position der front-clipping-plane
	// entlang der z-Achse
	int GetFront(D3DVALUE* d3dval);
	// ermittelt die Auflsung der front-clipping-plane
	// entlang der z-Achse
	int GetPlane(D3DVALUE* d3dLeft, D3DVALUE* d3dRight,
		D3DVALUE* d3dBottom, D3DVALUE* d3dTop);

	// ermittelt die Breite des Viewports auf Bildschirm
	int GetWidth();
	// ermittelt die Hhe des Viewports auf Bildschirm
	int GetHeight();

	// setzt Viewing Frostum erneut
	int SetField(D3DVALUE d3dval);
	// ermittelt Viewing Frostum
	int GetField(D3DVALUE* d3dval);

	// lscht das Viewport mit der aktuellen Hintergrundfarbe
	// ACHTUNG:
	// Es werden keine Sicherheitstests durchgefhrt!!
	int Clear();
	// lscht das Viewport mit der aktuellen Hintergrundfarbe
	// unter Verwendung der Flags
	// ACHTUNG:
	// Es werden keine Sicherheitstests durchgefhrt!!
	int Clear(DWORD dwFlags);

	// konfiguriert das Viewport auf dem Device neu
	int Configure(LONG lx, LONG ly, DWORD dwWidth, DWORD dwHeight);
	// gibt den Bereich an, der nach Aufruf von Update
	// neu geupdatet werden soll
	int ForceUpdate(DWORD dwx1, DWORD dwy1, DWORD dwx2, DWORD dwy2);

	// ermittelt Device, welcher an Klasse gebunden ist
	CDirect3DRMDevice* GetDevice();

	// Rendering des angegebenen Frames zum Viewport
	int Render(CDirect3DRMFrame* pFrame);

	// liefert die x-Startposition des Viewports auf Device
	int GetX();
	// liefert die y-Startposition des Viewports auf Device
	int GetY();

	// setzt den Projetionstyp
	int SetProjection(D3DRMPROJECTIONTYPE d3dproj);
	// ermittelt aktuellen Projektionstyp
	int GetProjection(D3DRMPROJECTIONTYPE* d3dproj);

	// Pointer auf Viewport-Interface
	DXDLL_LPDIRECT3DRMVIEWPORT lpD3DRMViewport;
	// Pointer auf dazugehriges Device
	CDirect3DRMDevice* pDirect3DRMDev;
	// Pointer auf Camera
	CDirect3DRMFrame* pCamera;
};

#endif // !defined(AFX_DIRECT3DRM_H__D95ED900_C590_11D2_AD5A_D7D8FA41524B__INCLUDED_)
