// Direct3DRMMeshBuilder.cpp: Implementierung der Klasse CDirect3DRMMeshBuilder.
//
//////////////////////////////////////////////////////////////////////

#include "Direct3DRMObjects.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CDirect3DRMMeshBuilder::CDirect3DRMMeshBuilder()
{
	// alles resetten
	lpD3DRMMeshBuilder = NULL;
}

CDirect3DRMMeshBuilder::CDirect3DRMMeshBuilder(DXDLL_LPDIRECT3DRMMESHBUILDER lpD3DMB)
{
	// alles bernehmen
	lpD3DRMMeshBuilder = lpD3DMB;
}

CDirect3DRMMeshBuilder::~CDirect3DRMMeshBuilder()
{
	// alles wieder lschen
	DestroyDirect3DMeshBuilder();
}

void CDirect3DRMMeshBuilder::DestroyDirect3DMeshBuilder()
{
	// existiert ein Objekt?
	if (lpD3DRMMeshBuilder)
	{
		// ... dann auslschen
		lpD3DRMMeshBuilder->Release();
		lpD3DRMMeshBuilder = NULL;
	}
}

LPDIRECT3DRMVISUAL CDirect3DRMMeshBuilder::GetVisual()
{
	// und das ganze zurckliefern
	return (LPDIRECT3DRMVISUAL)lpD3DRMMeshBuilder;
}

LPDIRECT3DRMOBJECT CDirect3DRMMeshBuilder::GetObject()
{
	// und das ganze zurckliefern
	return (LPDIRECT3DRMOBJECT)lpD3DRMMeshBuilder;
}

int CDirect3DRMMeshBuilder::SetColor(D3DCOLOR d3dCol)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... dann Wert setzen
		return lpD3DRMMeshBuilder->SetColor(d3dCol);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::SetColorRGB(D3DVALUE d3dRed, D3DVALUE d3dGreen, 
										D3DVALUE d3dBlue)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... dann Wert setzen
		return lpD3DRMMeshBuilder->SetColorRGB(d3dRed, d3dGreen, d3dBlue);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::SetColorSource(D3DRMCOLORSOURCE source)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... dann Wert setzen
		return lpD3DRMMeshBuilder->SetColorSource(source);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::GetColorSource(D3DRMCOLORSOURCE* source)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
	{
		// alles mgliche ermitteln
		*source = lpD3DRMMeshBuilder->GetColorSource();
		// ... kein Fehler aufgetreten
		return DXDLL_OK;
	}

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::GetBox(D3DRMBOX* lpBox)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// alles mgliche ermitteln und raus
		return lpD3DRMMeshBuilder->GetBox(lpBox);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::AddFace(CDirect3DRMFace* pFace)
{
	// existiert MeshBuilder-Objekt bzw. Face?
	if (lpD3DRMMeshBuilder && pFace)
		// ... Face hinzufgen und raus
		return lpD3DRMMeshBuilder->AddFace(pFace->lpD3DRMFace);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::GetFaceCount()
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... Faceanzahl ermitteln und raus
		return lpD3DRMMeshBuilder->GetFaceCount();

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::AddMeshBuilder(CDirect3DRMMeshBuilder* pMeshBuilder,
										   DWORD dwFlags)
{
	// existiert MeshBuilder-Objekt bzw. MeshBuilder?
	if (lpD3DRMMeshBuilder && pMeshBuilder)
		// ... MeshBuilder hinzufgen und raus
		return lpD3DRMMeshBuilder->AddMeshBuilder(pMeshBuilder->lpD3DRMMeshBuilder,
			dwFlags);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::ReserveSpace(DWORD vertexCount, 
										 DWORD normalCount,
										 DWORD faceCount)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... dann Speicher reservieren
		return lpD3DRMMeshBuilder->ReserveSpace(vertexCount,
			normalCount, faceCount);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::Scale(D3DVALUE sx, D3DVALUE sy, D3DVALUE sz)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... dann Skalierung auslsen
		return lpD3DRMMeshBuilder->Scale(sx, sy, sz);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::Translate(D3DVALUE tx, D3DVALUE ty, D3DVALUE tz)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... dann Translation ausfhren
		return lpD3DRMMeshBuilder->Translate(tx, ty, tz);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::SetMaterial(CDirect3DRMMaterial* pMaterial)
{
	// existiert MeshBuilder-Objekt bzw. Material?
	if (lpD3DRMMeshBuilder && pMaterial)
		// ... Material hinzufgen und raus
		return lpD3DRMMeshBuilder->SetMaterial(pMaterial->lpD3DRMMaterial);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::AddNormal(D3DVALUE x, D3DVALUE y, D3DVALUE z)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... Normalenvektor hinzufgen
		return lpD3DRMMeshBuilder->AddNormal(x, y, z);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::GenerateNormals(D3DVALUE dvCreaseAngle, 
											DWORD dwFlags)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... Normalenvektoren generieren
		return lpD3DRMMeshBuilder->GenerateNormals(dvCreaseAngle,
			dwFlags);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::SetNormal(DWORD index, D3DVALUE x, D3DVALUE y,
									  D3DVALUE z)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... Normalenvektor setzen
		return lpD3DRMMeshBuilder->SetNormal(index, x, y, z);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::SetPerspective(BOOL bPerspective)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... dann Perspektive setzen
		return lpD3DRMMeshBuilder->SetPerspective(bPerspective);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::GetPerspective(BOOL* bPerspective)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
	{
		// alles mgliche ermitteln
		*bPerspective = lpD3DRMMeshBuilder->GetPerspective();
		// ... kein Fehler aufgetreten
		return DXDLL_OK;
	}

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::SetQuality(D3DRMRENDERQUALITY lpQuality)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... dann Qualitt setzen
		return lpD3DRMMeshBuilder->SetQuality(lpQuality);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::GetQuality(D3DRMRENDERQUALITY* lpQuality)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
	{
		// alles mgliche ermitteln
		*lpQuality = lpD3DRMMeshBuilder->GetQuality();
		// ... kein Fehler aufgetreten
		return DXDLL_OK;
	}

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::SetTextureCoordinates(DWORD index, D3DVALUE u,
												  D3DVALUE v)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... dann Wert setzen
		return lpD3DRMMeshBuilder->SetTextureCoordinates(index, u, v);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::GetTextureCoordinates(DWORD index, D3DVALUE *lpU,
												  D3DVALUE *lpV)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// alles mgliche ermitteln und raus
		return lpD3DRMMeshBuilder->GetTextureCoordinates(index, lpU, lpV);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::SetTextureTopology(BOOL cylU, BOOL cylV)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... dann Wert setzen
		return lpD3DRMMeshBuilder->SetTextureTopology(cylU, cylV);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::SetTexture(CDirect3DRMTexture* pTexture)
{
	// existiert MeshBuilder-Objekt bzw. Texture?
	if (lpD3DRMMeshBuilder && pTexture)
		// ... Texture hinzufgen und raus
		return lpD3DRMMeshBuilder->SetTexture(pTexture->lpD3DRMTexture);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::AddVertex(D3DVALUE x, D3DVALUE y, D3DVALUE z)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... dann Vertex hinzufgen
		return lpD3DRMMeshBuilder->AddVertex(x, y, z);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::SetVertexColorRGB(DWORD index, D3DVALUE red,
											  D3DVALUE green, D3DVALUE blue)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... dann Farbe setzen
		return lpD3DRMMeshBuilder->SetVertexColorRGB(index, red, 
			green, blue);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::SetVertexColor(DWORD index, D3DCOLOR color)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... dann Farbe setzen
		return lpD3DRMMeshBuilder->SetVertexColor(index, color);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::GetVertexColor(DWORD index, D3DCOLOR* d3dColor)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
	{
		// alles mgliche ermitteln
		*d3dColor = lpD3DRMMeshBuilder->GetVertexColor(index);
		// ... kein Fehler aufgetreten
		return DXDLL_OK;
	}

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::GetVertexCount()
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... dann Anzahl Vertexes ermitteln
		return lpD3DRMMeshBuilder->GetVertexCount();

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::GetVertices(DWORD dwFirst, DWORD* lpdwCount,
										D3DVECTOR *lpdvVector)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... dann alles ermitteln
		return lpD3DRMMeshBuilder->GetVertices(dwFirst, lpdwCount,
			lpdvVector);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::SetVertex(DWORD index, D3DVALUE x, D3DVALUE y,
									  D3DVALUE z)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... dann Vertexposition setzen
		return lpD3DRMMeshBuilder->SetVertex(index, x, y, z);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::Load(void* lpvObjSource, void* lpvObjID, 
								 DXDLL_D3DRMLOADOPTIONS d3drmLOFlags, 
								 DXDLL_D3DRMLOADTEXTURECALLBACK
								 d3drmLoadTextureProc, void* lpvArg)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... dann einfach mal versuchen zu laden
		return lpD3DRMMeshBuilder->Load(lpvObjSource, lpvObjID, d3drmLOFlags,
			d3drmLoadTextureProc, lpvArg);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMMeshBuilder::Save(const char * lpFilename, D3DRMXOFFORMAT 
								 d3drmXOFFormat, D3DRMSAVEOPTIONS d3drmSOContents)
{
	// existiert MeshBuilder-Objekt?
	if (lpD3DRMMeshBuilder)
		// ... dann einfach mal versuchen zu laden
		return lpD3DRMMeshBuilder->Save(lpFilename, d3drmXOFFormat,
			d3drmSOContents);

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

