// Direct3DRMObjects.h: Schnittstelle fr die Klassen, die der Objekt-
//						erstellung und -verwaltung dienen.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DIRECT3DRMOBJECTS_H__3AFBAAE0_C590_11D2_AD5A_D7D8FA41524B__INCLUDED_)
#define AFX_DIRECT3DRMOBJECTS_H__3AFBAAE0_C590_11D2_AD5A_D7D8FA41524B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// alle mglichen Defines
#include "DXDefines.h"
// alle mglichen Tools
#include "Tools.h"
// Direct3D-Verwaltung
#include "d3d.h"
#include "d3dcaps.h"
#include "d3drm.h"
#include "d3drmdef.h"
#include "d3drmobj.h"
#include "d3drmwin.h"
#include "d3dtypes.h"


// -------------------------------------------
// Forwarddeklarationen der einzelnen Klassen
// -------------------------------------------
class CDirect3DRMFace;
class CDirect3DRMMeshBuilder;

class CDirect3DRMMaterial;
class CDirect3DRMTexture;

class CDirect3DRMLight;

// wird nur fr Wrap bentigt
class CDirect3DRMFrame;


// ----------------------------------------------------
// richtige komplette Definionen der einzelnen Klassen
// ----------------------------------------------------
class __declspec(dllexport) CDirect3DRMFace
{
public:
	CDirect3DRMFace();
	CDirect3DRMFace(DXDLL_LPDIRECT3DRMFACE lpFace);
	virtual ~CDirect3DRMFace();

	// zerstrt komplettes Polygon
	void DestroyDirect3DFace();

	// setzt Farbe des aktuellen Faces
	int SetColor(D3DCOLOR d3dcol);
	// setzt Farbe mittels RGB-Werte des aktuellen Faces
	int SetColorRGB(D3DVALUE d3dRed, D3DVALUE d3dGreen,
		D3DVALUE d3dBlue);
	// ermittelt aktuelle Farbe des Faces
	int GetColor(D3DCOLOR* d3dcol);

	// setzt Materialeigenschaften des Faces
	int SetMaterial(CDirect3DRMMaterial* pMaterial);
	// ermittelt aktuelle Materialeigenschaften des Faces
	CDirect3DRMMaterial* GetMaterial();

	// setzt Texture des Faces
	int SetTexture(CDirect3DRMTexture* pTexture);
	// ermittelt aktuelle Texture des Faces
	CDirect3DRMTexture* GetTexture();

	// setzt die Texturekoordinaten fr einen bestimmten
	// Vertex im Face
	int SetTextureCoordinates(DWORD vertex, D3DVALUE u,
		D3DVALUE v);
	// ermittelt die Texturekoordinaten fr einen bestimmten
	// Vertex im Face
	int GetTextureCoordinates(DWORD index, D3DVALUE* lpU,
		D3DVALUE* lpV);

	// setzt die Texturetopology des Faces
	int SetTextureTopology(BOOL cylU, BOOL cylV);
	// ermittelt die Texturetopology des Faces
	int GetTextureTopology(BOOL* cylU, BOOL* cylV);

	// ermittelt die Nummer des Vertexes aus dem Face
	// bezogen auf ein Mesh (Texturebezogen)
	int GetTextureCoordinateIndex(DWORD which, int* iMesh);

	// fgt einen Vertex zu aktuellen Face hinzu
	int AddVertex(D3DVALUE d3dx, D3DVALUE d3dy, D3DVALUE d3dz);
	// fgt einen Vertex mit dem Normalenvektor aus Mesh
	// zu aktuellen Face hinzu
	// ACHTUNG:
	// Vertex und Normalenvektor mu bereits in MeshBuilder-
	// Objekt existieren!
	int AddVertexAndNormalIndexed(DWORD vertex, DWORD normal);
	// holt die Vertexposition und den Normalenvektor eines
	// Vertexes des Faces
	int GetVertex(DWORD index, D3DVECTOR* lpPosition,
		D3DVECTOR* lpNormal);
	// holt die Anzahl Vertexes im Face
	// ACHTUNG:
	// Negative Rckgabewerte deuten auf Fehler hin!
	int GetVertexCount();
	// ermittelt die Nummer des Vertexes aus dem Face
	// bezogen auf ein Mesh
	// ACHTUNG:
	// Negative Rckgabewerte deuten auf Fehler hin!
	int GetVertexIndex(DWORD which);
	// holt alle Vertexes aus dem Face
	int GetVertices(DWORD *lpdwVertexCount, D3DVECTOR *lpPosition,
		D3DVECTOR *lpNormal);

	// holt den Normalenvektor des Faces
	int GetNormal(D3DVECTOR* lpNormal);

	// gibt die Ur-Schnittstelle aller D3D-Objekte bezogen
	// auf dieses Objekt zurck
	LPDIRECT3DRMOBJECT GetObject();

	// das eigentliche Polygon
	DXDLL_LPDIRECT3DRMFACE lpD3DRMFace;
	// ... hier ist das Material gespeichert
	CDirect3DRMMaterial* pDirect3DRMMaterial;
	// ... hier ist die Texture gespeichert
	CDirect3DRMTexture* pDirect3DRMTexture;
};

class __declspec(dllexport) CDirect3DRMMeshBuilder
{
public:
	CDirect3DRMMeshBuilder();
	CDirect3DRMMeshBuilder(DXDLL_LPDIRECT3DRMMESHBUILDER lpD3DMB);
	virtual ~CDirect3DRMMeshBuilder();

	// zerstrt das MeshBuilder-Objekt komplett
	void DestroyDirect3DMeshBuilder();

	// ermittelt einen Zeiger auf die Direct3DRMVisual-
	// Schnittstelle
	LPDIRECT3DRMVISUAL GetVisual();
	// ermittelt einen Zeiger auf die IDirect3DRMObject-
	// Schnittstelle
	LPDIRECT3DRMOBJECT GetObject();

	// setzt alle Faces und Vertexes zu einer festgelegten Farbe
	int SetColor(D3DCOLOR d3dCol);
	// setzt alle Faces und Vertexes zu einer festgelegten Farbe
	int SetColorRGB(D3DVALUE d3dRed, D3DVALUE d3dGreen, D3DVALUE d3dBlue);
	// setzt die Farbquelle des Builders
	int SetColorSource(D3DRMCOLORSOURCE source);
	// ermittelt die aktuelle Farbquelle des Builders
	int GetColorSource(D3DRMCOLORSOURCE* source);

	// ermittelt die Koordinaten, welche Builder eingrenzt
	int GetBox(D3DRMBOX* lpBox);

	// fgt einen Face zu Builder hinzu
	int AddFace(CDirect3DRMFace* pFace);
	// ermittelt die Anzahl der Faces in Builder
	int GetFaceCount();

	// fgt ein MeshBuilder-Objekt diesem Objekt hinzu
	int AddMeshBuilder(CDirect3DRMMeshBuilder* pMeshBuilder,
		DWORD dwFlags);

	// reserviert Speicher fr dieses Objekt
	int ReserveSpace(DWORD vertexCount, DWORD normalCount,
		DWORD faceCount);

	// Skaliert Builder-Objekt mit angegebenen Faktor
	int Scale(D3DVALUE sx, D3DVALUE sy, D3DVALUE sz);
	// bewegt Builder-Objekt um angegebene Einheiten
	int Translate(D3DVALUE tx, D3DVALUE ty, D3DVALUE tz);

	// setzt das Material fr gesamtes Builder-Objekt
	int SetMaterial(CDirect3DRMMaterial* pMaterial);

	// fgt einen Normalenvektor zu Builder hinzu
	int AddNormal(D3DVALUE x, D3DVALUE y, D3DVALUE z);
	// generiert fr jeden Vertex im Builder einen Normalenvektor
	int GenerateNormals(D3DVALUE dvCreaseAngle, DWORD dwFlags);
	// setzt den Normalenvektor fr den angegebenen Vertex
	int SetNormal(DWORD index, D3DVALUE x, D3DVALUE y, D3DVALUE z);

	// setzt die Perspektivenkorrektur an/aus
	int SetPerspective(BOOL bPerspective);
	// ermittelt, ob Perspektivenkorrektur an/aus
	int GetPerspective(BOOL* bPerspective);

	// setzt die Renderqualitt fr MeshBuilder
	int SetQuality(D3DRMRENDERQUALITY lpQuality);
	// ermittelt die Renderqualitt fr MeshBuilder
	int GetQuality(D3DRMRENDERQUALITY* lpQuality);

	// setzt die Texturekoordinaten fr einen Vertex
	int SetTextureCoordinates(DWORD index, D3DVALUE u, D3DVALUE v);
	// ermittelt die Texturekoordinaten fr einen Vertex
	int GetTextureCoordinates(DWORD index, D3DVALUE *lpU, D3DVALUE *lpV);
	// setzt die Texture topology des Builde-Objekts
	int SetTextureTopology(BOOL cylU, BOOL cylV);
	// setzt die Texture fr alle Faces des Builder-Objekts
	int SetTexture(CDirect3DRMTexture* pTexture);

	// fgt einen Vertex zum Builder-Objekt hinzu
	int AddVertex(D3DVALUE x, D3DVALUE y, D3DVALUE z);
	// setzt die Farbe des angegebenen Vertexes via RGB-Angaben
	int SetVertexColorRGB(DWORD index, D3DVALUE red, D3DVALUE green,
		D3DVALUE blue);
	// setzt die Farbe des angegebenen Vertexes
	int SetVertexColor(DWORD index, D3DCOLOR color);
	// ermittelt die Vertex-Farbe eines Vertexes
	int GetVertexColor(DWORD index, D3DCOLOR* d3dColor);
	// ermittelt die Anzahl Vertexes im Builder-Objekt
	int GetVertexCount();
	// holt alle Vertexes des Builder-Objekts
	int GetVertices(DWORD dwFirst, DWORD* lpdwCount, D3DVECTOR *lpdvVector);
	// setzt die Koordinaten des angegebenen Vertexes
	int SetVertex(DWORD index, D3DVALUE x, D3DVALUE y, D3DVALUE z);

	// ldt ein MeshBuilder-Objekt
	int Load(void* lpvObjSource, void* lpvObjID, 
		DXDLL_D3DRMLOADOPTIONS d3drmLOFlags, DXDLL_D3DRMLOADTEXTURECALLBACK 
		d3drmLoadTextureProc, void* lpvArg);
	// speichert MeshBuilder-Objekt auf Platte ab
	int Save(const char * lpFilename, D3DRMXOFFORMAT d3drmXOFFormat,
		D3DRMSAVEOPTIONS d3drmSOContents);

	// ... die Schnittstelle, um Faces, Meshes, ... zu verwalten
	DXDLL_LPDIRECT3DRMMESHBUILDER lpD3DRMMeshBuilder;
};

class __declspec(dllexport) CDirect3DRMMaterial
{
public:
	CDirect3DRMMaterial();
	CDirect3DRMMaterial(DXDLL_LPDIRECT3DRMMATERIAL lpMaterial);
	virtual ~CDirect3DRMMaterial();

	// Material komplett zerstren
	void DestroyDirect3DMaterial();

	// setzt die Emissionseigenschaft des Lichtes des Materials
	int SetEmissive(D3DVALUE d3dr, D3DVALUE d3dg, D3DVALUE d3db);
	// ermittelt die Emissionseigenschaft des Lichtes des Materials
	int GetEmissive(D3DVALUE* d3dr, D3DVALUE* d3dg, D3DVALUE* d3db);

	// setzt Farbe fr specular highlights
	int SetSpecular(D3DVALUE d3dr, D3DVALUE d3dg, D3DVALUE d3db);
	// ermittelt aktuelle Farbe fr specular highlights
	int GetSpecular(D3DVALUE* d3dr, D3DVALUE* d3dg, D3DVALUE* d3db);

	// setzt die Ausprgung des specular highlights
	int SetPower(D3DVALUE d3dPower);
	// ermittelt die Ausprgung des specular highlights
	int GetPower(D3DVALUE* d3dPower);

	// setzt Ambientwert des Materials
	int SetAmbient(D3DVALUE lpdvRed, D3DVALUE lpdvGreen,
		D3DVALUE lpdvBlue);
	// ermittelt Ambientwert des Materials
	int GetAmbient(D3DVALUE* lpdvRed, D3DVALUE* lpdvGreen,
		D3DVALUE* lpdvBlue);

	// ... die eigentliche Materialdefinition
	DXDLL_LPDIRECT3DRMMATERIAL lpD3DRMMaterial;
};

class __declspec(dllexport) CDirect3DRMTexture
{
public:
	CDirect3DRMTexture();
	CDirect3DRMTexture(DXDLL_LPDIRECT3DRMTEXTURE lpTexture);
	virtual ~CDirect3DRMTexture();

	// Texture komplett zerstren
	void DestroyDirect3DTexture();

	// setzt die Anzahl der Farben, die maximal beim
	// Rendern der Texture verwendet werden soll
	int SetColors(DWORD dwColors);
	// ermittelt die Anzahl der Farben, die maximal beim
	// Rendern der Texture verwendet werden soll
	int GetColors(DWORD* dwColors);

	// setzt die Koordinaten des decals
	int SetDecalOrigin(LONG lx, LONG ly);
	// ermittelt die Koordinaten des decals
	int GetDecalOrigin(LONG* lx, LONG* ly);

	// setzt den Skalierungsfaktor des decals
	int SetDecalScale(DWORD dwScale);
	// ermittelt den Skalierungsfaktor des decals
	int GetDecalScale(DWORD* dwScale);

	// setzt die Gre des decals
	int SetDecalSize(D3DVALUE d3dWidth, D3DVALUE d3dHeight);
	// ermittelt die Gre des decals
	int GetDecalSize(D3DVALUE* d3dWidth, D3DVALUE* d3dHeight);

	// setzt oder lscht Transparenz des decals
	int SetDecalTransparency(BOOL bTransp);
	// ermittelt Transparenz des decals
	int GetDecalTransparency(BOOL* bTransp);

	// setzt die Farbe, welche der Transparenz dienen soll
	int SetDecalTransparentColor(D3DCOLOR d3dColor);
	// ermittelt die Farbe, welche der Transparenz dient
	int GetDecalTransparentColor(D3DCOLOR* d3dColor);

	// ermittelt Eigenschaften der Texture
	int GetImage(D3DRMIMAGE* lpImage);

	// generiert eine MIP Map der Texture
	int GenerateMIPMap();

	// wurden Vernderungen an Texture vorgenommen
	int Changed(DWORD dwFlags, DWORD dwcRects, LPRECT pRects);

	// setzt maximale Anzahl Schattierungen pro Farbe in Texture
	int SetShades(DWORD dwMaxShades);
	// ermittelt maximale Anzahl Schattierungen pro Farbe in Texture
	int GetShades(DWORD* dwMaxShades);

	// ermittelt eine Schnittstelle der Texture auf Direct3DRMVisual
	LPDIRECT3DRMVISUAL GetVisual();

	// Interface auf die Texture
	DXDLL_LPDIRECT3DRMTEXTURE lpD3DRMTexture;
};

class __declspec(dllexport) CDirect3DRMLight  
{
public:
	CDirect3DRMLight();
	CDirect3DRMLight(LPDIRECT3DRMLIGHT lpLight);
	virtual ~CDirect3DRMLight();

	// zerstrt die Schnittstelle auf das Licht
	void DestroyDirect3DLight();

	// setzt den konstanten Verminderungsfaktor des Lichtes
	int SetConstantAttenuation(D3DVALUE d3dAttenuation);
	// ermittelt den konstanten Verminderungsfaktor des Lichtes
	int GetConstantAttenuation(D3DVALUE* d3dAttenuation);

	// setzt den linearen Verminderungsfaktor des Lichtes
	int SetLinearAttenuation(D3DVALUE d3dAttenuation);
	// ermittelt den konstanten Verminderungsfaktor des Lichtes
	int GetLinearAttenuation(D3DVALUE* d3dAttenuation);

	// setzt den quadratischen Verminderungsfaktor des Lichtes
	int SetQuadraticAttenuation(D3DVALUE d3dAttenuation);
	// ermittelt den quadratischen Verminderungsfaktor des Lichtes
	int GetQuadraticAttenuation(D3DVALUE* d3dAttenuation);

	// setzt die Lichtfarbe
	int SetColor(D3DCOLOR d3dColor);
	// setzt die Lichtfarbe per RGB-Werte
	int SetColorRGB(D3DVALUE d3dRed, D3DVALUE d3dGreen, D3DVALUE d3dBlue);
	// ermittelt die aktuelle Lichtfarbe
	int GetColor(D3DCOLOR* d3dColor);

	// setzt den Lichttypen
	int SetType(D3DRMLIGHTTYPE LightType);
	// ermittelt den Lichttypen
	int GetType(D3DRMLIGHTTYPE* lpLightType);

	// setzt den Bereich des Lichtes
	int SetRange(D3DVALUE d3dRange);
	// ermittelt den Bereich des Lichtes
	int GetRange(D3DVALUE* d3dRange);

	// setzt den Winkel der Umbra
	int SetUmbra(D3DVALUE d3dUmbra);
	// ermittelt den Winkel der Umbra
	int GetUmbra(D3DVALUE* d3dUmbra);
	// setzt den Winkel der Penumbra
	int SetPenumbra(D3DVALUE d3dPenumbra);
	// ermittelt den Winkel der Penumbra
	int GetPenumbra(D3DVALUE* d3dPenumbra);

	// die Schnittstelle auf das Light
	DXDLL_LPDIRECT3DRMLIGHT lpD3DRMLight;
};

class __declspec(dllexport) CDirect3DRMWrap
{
public:
	CDirect3DRMWrap();
	CDirect3DRMWrap(LPDIRECT3DRMWRAP lpWrap);
	virtual ~CDirect3DRMWrap();

	// zerstrt das komplette Objekt
	void DestroyDirect3DWrap();

	// Einstellungen im Wrap bernehmen und neu berechnen, etc.
	// ACHTUNG:
	// Das Objekt kann ber folgenden Weg ermittelt werden
	// <Objekt>->GetObject()
	int Apply(LPDIRECT3DRMOBJECT lpD3DRMObject);
	// Einstellungen im Wrap zu Objekten bernehmen und neu 
	// berechnen, etc.
	// ACHTUNG:
	// Das Objekt kann ber folgenden Weg ermittelt werden
	// <Objekt>->GetObject()
	int ApplyRelative(CDirect3DRMFrame* pFrame,
		LPDIRECT3DRMOBJECT lpD3DRMObject);

	// die Schnittstelle auf das Wrap-Objekt
	DXDLL_LPDIRECT3DRMWRAP lpD3DRMWrap;
};

#endif // !defined(AFX_DIRECT3DRMOBJECTS_H__3AFBAAE0_C590_11D2_AD5A_D7D8FA41524B__INCLUDED_)
