// Direct3DRMViewport.cpp: Implementierung der Klasse CDirect3DRMViewport.
//
//////////////////////////////////////////////////////////////////////

#include "Direct3DRM.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CDirect3DRMViewport::CDirect3DRMViewport()
{
	// alles leeren
	lpD3DRMViewport = NULL;
	pDirect3DRMDev = NULL;
	pCamera = NULL;
}

CDirect3DRMViewport::CDirect3DRMViewport(CDirect3DRMDevice* pDev)
{
	// alles leeren
	lpD3DRMViewport = NULL;
	pCamera = NULL;
	// ... ansonsten bertragen
	pDirect3DRMDev = pDev;
}

CDirect3DRMViewport::CDirect3DRMViewport(CDirect3DRMDevice* pDev,
										 DXDLL_LPDIRECT3DRMVIEWPORT lpViewport)
{
	// ... den Rest leeren
	pCamera = NULL;
	// ... ansonsten bertragen
	lpD3DRMViewport = lpViewport;
	pDirect3DRMDev = pDev;
}

CDirect3DRMViewport::~CDirect3DRMViewport()
{
	// noch alles auslschen
	DestroyDirect3DRMViewport();
}

void CDirect3DRMViewport::DestroyDirect3DRMViewport()
{
	// ist Viewport belegt?
	if (lpD3DRMViewport)
	{
		// ... auslschen
		lpD3DRMViewport->Release();
		lpD3DRMViewport = NULL;
	}
}

int CDirect3DRMViewport::SetCamera(CDirect3DRMFrame* pCam)
{
	// ist Kamera gltig?
	if (pCam && lpD3DRMViewport)
		// ist auch Frame in Camera gltig?
		if (pCam->lpD3DRMFrame)
		{
			HRESULT hres;

			// ... dann diese setzen
			if (FAILED(hres = lpD3DRMViewport->SetCamera(pCam->lpD3DRMFrame)))
				// Fehlermeldung zurckliefern
				return hres;

			// Pointer auf Camera speichern
			pCamera = pCam;
			// und alles ok
			return hres;
		}

	// ansonsten eben Fehler!
	return DXDLL_INTERNALERROR;
}

CDirect3DRMFrame* CDirect3DRMViewport::GetCamera()
{
	// und aktuelle Camera zurckliefern
	return pCamera;
}

int CDirect3DRMViewport::SetBack(D3DVALUE d3dval)
{
	// existiert ein Viewport?
	if (lpD3DRMViewport)
		// ... dann setzen
		return lpD3DRMViewport->SetBack(d3dval);

	// ... ansonsten Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMViewport::SetFront(D3DVALUE d3dval)
{
	// existiert ein Viewport?
	if (lpD3DRMViewport)
		// ... dann setzen
		return lpD3DRMViewport->SetFront(d3dval);

	// ... ansonsten Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMViewport::SetPlane(D3DVALUE d3dLeft, D3DVALUE d3dRight,
								  D3DVALUE d3dBottom, D3DVALUE d3dTop)
{
	// existiert ein Viewport?
	if (lpD3DRMViewport)
		// ... dann setzen
		return lpD3DRMViewport->SetPlane(d3dLeft, d3dRight, 
			d3dBottom, d3dTop);

	// ... ansonsten Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMViewport::GetBack(D3DVALUE* d3dval)
{
	// existiert ein Viewport?
	if (lpD3DRMViewport)
	{
		// ... dann ermitteln
		*d3dval = lpD3DRMViewport->GetBack();
		// ... und keinen Fehler
		return DXDLL_OK;
	}

	// ... ansonsten Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMViewport::GetFront(D3DVALUE* d3dval)
{
	// existiert ein Viewport?
	if (lpD3DRMViewport)
	{
		// ... dann ermitteln
		*d3dval = lpD3DRMViewport->GetFront();
		// ... und keinen Fehler
		return DXDLL_OK;
	}

	// ... ansonsten Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMViewport::GetPlane(D3DVALUE* d3dLeft, D3DVALUE* d3dRight,
								  D3DVALUE* d3dBottom, D3DVALUE* d3dTop)
{
	// existiert ein Viewport?
	if (lpD3DRMViewport)
		// ... dann ermitteln
		return lpD3DRMViewport->GetPlane(d3dLeft, d3dRight, 
			d3dBottom, d3dTop);

	// ... ansonsten Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMViewport::GetWidth()
{
	// existiert ein Viewport?
	if (lpD3DRMViewport)
		// ... dann breite zurckliefern
		return lpD3DRMViewport->GetWidth();

	// ... ansonsten Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMViewport::GetHeight()
{
	// existiert ein Viewport?
	if (lpD3DRMViewport)
		// ... dann breite zurckliefern
		return lpD3DRMViewport->GetHeight();

	// ... ansonsten Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMViewport::SetField(D3DVALUE d3dval)
{
	// existiert ein Viewport?
	if (lpD3DRMViewport)
		// ... dann setzen
		return lpD3DRMViewport->SetField(d3dval);

	// ... ansonsten Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMViewport::GetField(D3DVALUE* d3dval)
{
	// existiert ein Viewport?
	if (lpD3DRMViewport)
	{
		// ... dann ermitteln
		*d3dval = lpD3DRMViewport->GetField();
		// ... und keinen Fehler
		return DXDLL_OK;
	}

	// ... ansonsten Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMViewport::Clear()
{
	// lschen des Viewports auslsen
	return lpD3DRMViewport->Clear(D3DRMCLEAR_ALL);
}

int CDirect3DRMViewport::Clear(DWORD dwFlags)
{
	// lschen des Viewports auslsen
	return lpD3DRMViewport->Clear(dwFlags);
}

int CDirect3DRMViewport::Configure(LONG lx, LONG ly,
								   DWORD dwWidth, DWORD dwHeight)
{
	// existiert ein Viewport?
	if (lpD3DRMViewport)
		// ... und neu setzen
		return lpD3DRMViewport->Configure(lx, ly, dwWidth, dwHeight);

	// ... ansonsten Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMViewport::ForceUpdate(DWORD dwx1, DWORD dwy1, 
									 DWORD dwx2, DWORD dwy2)
{
	// existiert ein Viewport?
	if (lpD3DRMViewport)
		// ... und neu setzen
		return lpD3DRMViewport->ForceUpdate(dwx1, dwy1, dwx2, dwy2);

	// ... ansonsten Fehler
	return DXDLL_INTERNALERROR;
}

CDirect3DRMDevice* CDirect3DRMViewport::GetDevice()
{
	// und den Device zurckliefern
	return pDirect3DRMDev;
}

int CDirect3DRMViewport::Render(CDirect3DRMFrame* pFrame)
{
	// existiert eigentlich dieser Frame und das Viewport?
	if (pFrame && lpD3DRMViewport)
		// ... und auch die Schnittstelle?
		if (pFrame->lpD3DRMFrame)
			// ... dann rendern
			return lpD3DRMViewport->Render(pFrame->lpD3DRMFrame);

	// ... ansonsten eben interner Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMViewport::GetX()
{
	// existiert Viewport?
	if (lpD3DRMViewport)
		// ... dann Position zurckliefern
		return lpD3DRMViewport->GetX();

	// ... ansonsten Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMViewport::GetY()
{
	// existiert Viewport?
	if (lpD3DRMViewport)
		// ... dann Position zurckliefern
		return lpD3DRMViewport->GetY();

	// ... ansonsten Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMViewport::SetProjection(D3DRMPROJECTIONTYPE d3dproj)
{
	// existiert Viewport?
	if (lpD3DRMViewport)
		// ... dann Position zurckliefern
		return lpD3DRMViewport->SetProjection(d3dproj);

	// ... ansonsten Fehler
	return DXDLL_INTERNALERROR;
}

int CDirect3DRMViewport::GetProjection(D3DRMPROJECTIONTYPE* d3dproj)
{
	// existiert Viewport?
	if (lpD3DRMViewport)
	{
		// ... dann Position zurckliefern
		*d3dproj = lpD3DRMViewport->GetProjection();
		// jetzt nichts aufgetreten
		return DXDLL_OK;
	}

	// ... ansonsten Fehler
	return DXDLL_INTERNALERROR;
}

