// DirectDraw.cpp: Implementierung der Klasse CDirectDraw.
//
//////////////////////////////////////////////////////////////////////

#include "DirectDraw.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CDirectDraw::CDirectDraw()
{
	// erst DirectDraw-Objekte initialisieren
	lpDD = NULL;
	hWindow = NULL;

	// Standardwerte setzen
	iBSWidth = 0;
	iBSHeight = 0;
}

CDirectDraw::CDirectDraw(HWND hMainWindow)
{
	// erst DirectDraw-Objekte initialisieren
	lpDD = NULL;
	hWindow = hMainWindow;

	// Standardwerte setzen
	iBSWidth = 0;
	iBSHeight = 0;
}

CDirectDraw::CDirectDraw(HWND hMainWindow, DWORD dwFlags)
{
	// erst DirectDraw-Objekte initialisieren
	lpDD = NULL;
	hWindow = hMainWindow;

	// DirectDraw erzeugen
	if (FAILED(DirectDrawCreate()))
		// bei Fehler auf NULL setzen
		lpDD = NULL;
	// noch Coop-Ebene setzen
	if (FAILED(SetCooperativeLevel(dwFlags)))
	{
		// resetten
		lpDD->Release();
		// bei Fehler auf NULL setzen
		lpDD = NULL;
	}
}

CDirectDraw::CDirectDraw(HWND hMainWindow, GUID* lpGUID, DWORD dwFlags)
{
	// erst DirectDraw-Objekte initialisieren
	lpDD = NULL;
	hWindow = hMainWindow;

	// DirectDraw erzeugen
	if (FAILED(DirectDrawCreate(lpGUID)))
		// bei Fehler auf NULL setzen
		lpDD = NULL;
	// noch Coop-Ebene setzen
	if (FAILED(SetCooperativeLevel(dwFlags)))
	{
		// resetten
		lpDD->Release();
		// bei Fehler auf NULL setzen
		lpDD = NULL;
	}
}

CDirectDraw::~CDirectDraw()
{
	// und Objekt zerstren
	DirectDrawDestroy();
}

int CDirectDraw::DirectDrawCreate()
{
	// ... und mit primren BS-Treiber erzeugen
	return DirectDrawCreate(NULL);
}

int CDirectDraw::DirectDrawCreate(GUID* lpGUID)
{
	HRESULT hres = 0;
	LPDIRECTDRAW lpDDTemp;

	// altes Objekt komplett lschen
	DirectDrawDestroy();

	// DirectDraw-Objekt erzeugen
	if (FAILED(::DirectDrawCreate(lpGUID, &lpDDTemp, NULL)))
		// ebenfalls raus, falls Fehler
		return Fail(hWindow, DXDLL_MSGERROR_DDRAWCREATE);

	// DirectDraw-Objekt erzeugen
	if (FAILED(lpDDTemp->QueryInterface(DXDLL_GUID_DIRECTDRAW, (LPVOID*)&lpDD)))
		// ebenfalls raus, falls Fehler
		hres = Fail(hWindow, DXDLL_MSGERROR_DDRAW4CREATE);
	lpDDTemp->Release();

	return hres;
}

void CDirectDraw::DirectDrawDestroy()
{
	// Objekt vorhanden?
	if (lpDD)
	{
		// ... und bei Bedarf freigeben
		lpDD->Release();
		lpDD = NULL;
	}
}

HWND CDirectDraw::SetMainWindow(HWND hMainWindow)
{
	// altes Handle speichern
	HWND hAlt = hWindow;
	// neu belegen
	hWindow = hMainWindow;
	// und altes zurck
	return hAlt;
}

int CDirectDraw::SetCooperativeLevel(DWORD dwFlags)
{
	// Kooperationsebene setzen
	if (FAILED(lpDD->SetCooperativeLevel(hWindow, dwFlags)))
		// verlief alles OK? Ansonsten Fehlermeldung
		return Fail(hWindow, DXDLL_MSGERROR_DDRAWCOOPERATIVELEVEL);

	return DXDLL_OK;
}

int CDirectDraw::SetDisplayMode(int iWidth, int iHeight, int iBits, 
								int iRefresh, DWORD dwFlags)
{
	// Displaymode setzen
	if (FAILED(lpDD->SetDisplayMode(iWidth, iHeight, iBits, 
		iRefresh, dwFlags)))
		// auch eine Fehlermeldung bei Fehler
		return Fail(hWindow, DXDLL_MSGERROR_DDRAWDISPLAYMODE);

	// aktuelle Auflsung speichern
	iBSWidth = iWidth;
	iBSHeight = iHeight;

	return DXDLL_OK;
}
