// DirectDraw.h: Schnittstelle fr alle Klassen von DirectDraw.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DIRECTDRAW_H__57C7A900_8182_11D2_B0DF_40584EC1BE27__INCLUDED_)
#define AFX_DIRECTDRAW_H__57C7A900_8182_11D2_B0DF_40584EC1BE27__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// alle mglichen Defines
#include "DXDefines.h"
// alle mglichen Tools
#include "Tools.h"
// DirectDraw-Verwaltung
#include "ddraw.h"


// -------------------
// Typendeklarationen 
// ---------.---------
typedef struct
{
	// das gefundene Pixelformat
	DXDLL_DDPIXELFORMAT ddpfFoundFormat;
	// Bittiefe, die gesucht wird (0=erste)
	DWORD dwFindBits;
} DXDLL_FINDZBUFFER;

// -------------------------------------------
// Forwarddeklarationen der einzelnen Klassen
// -------------------------------------------
class CDirectDraw;
class CDirectDrawFont;
class CDirectDrawSurface;
class CDirectDrawClipper;


class CDirectXDriverInfo;

// ----------------------------------------------------
// richtige komplette Definionen der einzelnen Klassen
// ----------------------------------------------------
class __declspec(dllexport) CDirectDraw
{
public:
	CDirectDraw();
	CDirectDraw(HWND hMainWindow);
	CDirectDraw(HWND hMainWindow, DWORD dwFlags);
	CDirectDraw(HWND hMainWindow, GUID* lpGUID, DWORD dwFlags);
	// Destructor gibt alle Objekte frei
	virtual ~CDirectDraw();

	HWND SetMainWindow(HWND hMainWindow);
	// erzeugt ein DirectDraw-Objekt fr primres Display
	int DirectDrawCreate();
	// erzeugt ein DirectDraw-Objekt fr beliebiges Display
	int DirectDrawCreate(GUID* lpGUID);
	// zerstrt aktuelles DirectDraw-Objekt
	void DirectDrawDestroy();

	// setzt Kooperationsebene mit Win9x
	int SetCooperativeLevel(DWORD dwFlags);
	// setzt Bildschirmauflsung
	int SetDisplayMode(int iWidth, int iHeight, int iBits, 
		int iRefresh, DWORD dwFlags);

	// aktuelle Auflsungsdaten
	int iBSWidth;
	int iBSHeight;

	// DirectDraw5-Objekt
	DXDLL_LPDIRECTDRAW lpDD;
	// Hauptfenster, zu dem DD-Objekt gehrt
	HWND hWindow;
};

class __declspec(dllexport) CDirectDrawFont  
{
public:
	int CheckSurfaceFont();
	CDirectDrawFont();
	CDirectDrawFont(CDirectDraw* pDD);
	// zerstrt diesen Font
	virtual ~CDirectDrawFont();

	// erzeugt einen normalen Font (BMP mit 320x200)
	int CreateNormalFont(int iFnt, int iType);
	// erzeugt einen erweiterten Font (BMP mit Yx320)
	int CreateExtendedFont(int iFnt, int iType);
	// ldt Bitmap auf erzeugten Font
	int LoadFont(HINSTANCE hInstance, LPCSTR pFilename,
		LPCSTR pResName);
	// zerstrt alle Angaben des Fonts
	void DestroyFont();

	// ist bereits ein Font erzeugt?
	bool IsCreated();
	// handelt es sich um einen erweiterten Font?
	bool IsExtendedFont();

	int WriteChar(CDirectDrawSurface* pDestDDS, int ix, 
		int iy, char ch);
	void WriteString(CDirectDrawSurface* pDestDDS, int ix, 
		int iy, LPCSTR lpStr);

	int WriteStretchedChar(CDirectDrawSurface* pDestDDS,
		int ix, int iy, int iWidth,	int iHeight, char ch);
	void WriteStretchedString(CDirectDrawSurface *pDestDDS, int ix,
		int iy, int iWidth, int iHeight, LPCSTR lpStr);

	int WriteSinChar(CDirectDrawSurface* pDestDDS, int ix, 
		int iy, char ch);
	void WriteSinString(CDirectDrawSurface* pDestDDS, int ix, 
		int iy, LPCSTR lpStr);

	int SetTransparentColor(char ch, int ix, int iy);

	void CreateSinWave(double dAmp, double dStep);

	CDirectDraw* pDirectDraw;
	CDirectDrawSurface* pDirectDrawFont;

	HBITMAP hFont;
	int iFont;
	int iFontType;
	int iFontPositions[FNT_MAX_CHARS];
	int iFontXDimension;
	int iFontYDimension;
	int iFontLetters;
	int iAnzSin;
	int iLastSin;
	int iSinWave[359];
	bool bIsExtended;
};

class __declspec(dllexport) CDirectDrawSurface  
{
public:
	CDirectDrawSurface();
	CDirectDrawSurface(CDirectDraw* pDD);
	virtual ~CDirectDrawSurface();

	// lscht nur den DirectDraw-Surface inkl. Daten
	void DestroySurface();
	// lscht kompletten Surface inkl. aller Daten aus Klasse
	void DestroyCompleteSurface();

	int CreateFadeSurface(int iMaxWidth, int iMaxHeight);
	void DestroyFadeSurface();

	// fgt einen weiteren Surface zu Surface hinzu
	int AddAttachedSurface(CDirectDrawSurface* pSurface);

	// setzt Einstellungen des Surfaces fr Erzeugung
	void SetSurfaceDescData(DWORD dwFlags, DWORD dwBackBuffer,
		DWORD dwCaps);
	void SetSurfaceDescData(DXDLL_DDSURFACEDESC* lpddsd);
	// erzeugt einen Primarysurface mit Daten aus Descriptor
	int CreatePrimarySurface();
	// erzeugt normalen Surface aus angegebenen Descriptordaten
	int CreateSurface(DXDLL_DDSURFACEDESC* lpCaps);
	// erzeugt normalen Offscreen-Surface aus Descriptordaten mit Hhe/Breite
	int CreateOffscreenSurface(DWORD dwWidth, DWORD dwHeight);
	// erzeugt einen ZBuffer aus Descriptordaten
	int CreateZBuffer(DWORD dwWidth, DWORD dwHeight, 
		DWORD dwZBufferBitDepth, CDirectXDriverInfo* pD3DInfo);

	// ldt ein Bitmap auf Surface (altes wird vorher zerstrt)
	// wenn dwWidth, dwHeight == 0: Surface ist genauso gro, wie BMP
	// ansonsten: BMP wird gestreched auf Surface gezeichnet
	// bDestroyBMP: Soll Handle zu Win-BMP gelscht werden?
	HBITMAP LoadBitmap(DWORD dwWidth, DWORD dwHeight, 
		LPCSTR pFilename, LPCSTR pResName, HINSTANCE Instance,
		bool bDestroyBMP);

	// ldt ein Bitmap auf Surface, ohne aktuelles Surface zu zerstren
	// wenn dwWidth, dwHeight == 0: Bitmap wird auf Surfacegre 
	//								gestretched
	// ansonsten: Teil des BMPs wird auf Teil des Surfaces gezeichnet
	// bDestroyBMP: Soll Handle zu Win-BMP gelscht werden?
	HBITMAP CopyBitmap(DWORD dwWidth, DWORD dwHeight,
		LPCSTR pFilename, LPCSTR pResName, HINSTANCE Instance,
		bool bDestroyBMP);
	// kopiert ein als Handle vorhandenes Bitmap auf Surface, ohne 
	// aktuelles Surface zu zerstren
	// wenn dwWidth, dwHeight == 0: Bitmap wird auf Surfacegre 
	//								gestretched
	// ansonsten: Teil des BMPs wird auf Teil des Surfaces gezeichnet
	int CopyBitmap(DWORD dwWidth, DWORD dwHeight, HBITMAP hbm);

	// erstellt neues Windowskompatibles Bitmap und kopiert den Inhalt
	// des aktuellen Surfaces auf dieses Bitmap
	// wenn dwWidth, dwHeight == 0: Bitmap ist genauso gro, wie Surface 
	// ansonsten: Teil des Surfaces wird auf BMP gezeichnet
	HBITMAP CopySurface(DWORD dwWidth, DWORD dwHeight);

	// ermittelt Surface mit angegebenen Einstellungen
	CDirectDrawSurface* GetAttachedSurface(DXDLL_DDSCAPS dsCaps);

	// wechselt auf angegebene Seite
	int Flip(CDirectDrawSurface* pDDSurfaceTargetOverride, DWORD dwFlags);
	// wechselt auf nchste Seite mit Flags-Angabe
	int Flip(DWORD dwFlags);
	// normales wechseln zur nchsten Seite
	int Flip();

	// sperrt Rechteck auf Surface fr Win-Zugriffe
	int Lock(LPRECT lpDestRect, DXDLL_DDSURFACEDESC* lpDDSurfaceDesc, 
		DWORD dwDescFlags, DWORD dwFlags);
	// sperrt gesamten Surface fr Win-Zugriffe
	int Lock(DXDLL_DDSURFACEDESC* lpDDSurfaceDesc, DWORD dwDescFlags,
		DWORD dwFlags);
	// entsperrt angegebenes Rechteck wieder
	int Unlock(LPVOID lpSurfaceData);
	// entsperrt gesamten Surface
	int Unlock();

	// verbindet DirectDraw-Surface-Objekt mit einem Clipper
	int SetClipper(CDirectDrawClipper* pClipper);

	int BltFast(DWORD dwx, DWORD dwy, CDirectDrawSurface* pSurface, 
		LPRECT lpRect, DWORD dwTrans);
	int BltFast(CDirectDrawSurface* pSurface, LPRECT lpRect, DWORD dwTrans);
	int BltFast(CDirectDrawSurface* pSurface, DWORD dwTrans);
	int BltFast(DWORD dwx, DWORD dwy, CDirectDrawSurface* pSurface, 
		DWORD dwTrans);

	int Blt(DWORD dwx, DWORD dwy, CDirectDrawSurface* pDDSurface, 
		LPRECT lpSrcRect, DWORD dwFlags);
	int Blt(LPRECT lpDestRect, CDirectDrawSurface* pDDSurface, 
		LPRECT lpSrcRect, DWORD dwFlags);
	int Blt(CDirectDrawSurface* pDDSurface, LPRECT lpSrcRect, 
		DWORD dwFlags);
	int Blt(CDirectDrawSurface* pDDSurface,	DWORD dwFlags);

	// lscht dieses Surface mit angegebener Farbe
	int ClearSurface(DWORD dwColor);

	// noch nicht integriert!
	int BltFade(CDirectDrawSurface *pDDSurface, DWORD dwFlags,
		int iFadeRed, int iFadeGreen, int iFadeBlue);
	// noch nicht integriert!
	int BltFade(CDirectDrawSurface *pDDSurface, LPRECT lpSrcRect, 
		DWORD dwFlags, int iFadeRed, int iFadeGreen, int iFadeBlue);
	// noch nicht integriert!
	int BltFade(DWORD dwx, DWORD dwy, CDirectDrawSurface *pDDSurface,
		LPRECT lpSrcRect, DWORD dwFlags, int iFadeRed, int iFadeGreen, 
		int iFadeBlue);
	// noch nicht integriert!
	int BltFade(LPRECT lpDestRect, CDirectDrawSurface* pDDSurface, 
		LPRECT lpSrcRect, DWORD dwFlags, int iFadeRed, int iFadeGreen,
		int iFadeBlue);

	// stellt Oberflche ohne Inhalt wieder her
	int Restore();
	// stellt Oberflche wieder her und restauriert den Inhalt aus
	// der angegebenen Ressource oder Datei
	// wenn dwWidth, dwHeight == 0: Bitmap wird auf Surfacegre 
	//								gestretched
	// ansonsten: Teil des BMPs wird auf Teil des Surfaces gezeichnet
	// bDestroyBMP: Soll Handle zu Win-BMP gelscht werden?
	HBITMAP Restore(DWORD dwWidth, DWORD dwHeight, LPCSTR pFilename,
		LPCSTR pResName, HINSTANCE Instance, bool bDestroyBMP);
	// stellt Oberflche wieder her und restauriert den Inhalt aus
	// dem Bitmap, welches durch das Handle angegeben ist
	// wenn dwWidth, dwHeight == 0: Bitmap wird auf Surfacegre 
	//								gestretched
	// ansonsten: Teil des BMPs wird auf Teil des Surfaces gezeichnet
	int Restore(DWORD dwWidth, DWORD dwHeight, HBITMAP hbm);

	// berprft, ob Oberflche noch vorhanden ist
	int IsLost();

	// setzt einen Colorkey
	int SetColorKey(DWORD dwLowValue, DWORD dwHighValue, DWORD dwFlags);
	// ermittelt aktuellen Colorkey
	int GetColorKey(DWORD* dwLowValue, DWORD* dwHighValue, DWORD dwFlags);

	// Hhe des Surfaces
	int iSurfaceHeight;
	// Breite des Surfaces
	int iSurfaceWidth;

	// Pointer auf DirectDraw 5.0-Hauptsurface
	DXDLL_LPDIRECTDRAWSURFACE lpSurface;
//	LPDIRECTDRAWCLIPPER lpClipper;
	CDirectDrawSurface* pDDFadeSurface;
	// Descriptor mit Einstellungen zum Surface
	DXDLL_DDSURFACEDESC ddsdesc;
	// dazugehriges DirectDraw 5.0-Objekt
	CDirectDraw* pDirectDraw;
};

class __declspec(dllexport) CDirectDrawClipper  
{
public:
	// ACHTUNG:
	// Konstruktor erzeugt automatisch ein unabhngiges
	// Clipper-Objekt!!
	CDirectDrawClipper();
	// Konstruktor erzeugt automatisch ein vom DirectDraw-
	// Objekt abhngiges Clipper-Objekt!!
	CDirectDrawClipper(CDirectDraw* pDD);
	virtual ~CDirectDrawClipper();

	// erzeugt ein Clipperobjekt
	// ACHTUNG:
	// wenn an keine DD-Klasse gebunden, dann wird ein
	// unabhngiger Clipper erzeugt, ansonsten ein
	// Clipper, welcher zu diesem DD-Objekt gehrt!
	int DirectDrawClipperCreate();
	// zerstrt aktuellen Clipper wieder
	void DirectDrawClipperDestroy();

	// bindet Clipper an Fensterhandle, welches im
	// DirectDraw-Objekt angegeben ist
	int SetHWnd();
	// bindet Clipper an angegebenes Fensterhandle
	int SetHWnd(HWND hWnd);
	// liefert Fensterhandle, welches an Clipper gebunden ist
	int GetHWnd(HWND* hWnd);

	// wurden Clipboarddaten verndert?
	// ACHTUNG:
	// 0   ... wurde nicht verndert
	// > 0 ... wurde verndert
	// < 0 ... Fehler bei Abfrage
	int IsClipListChanged();

	// lscht aktuelle Clip-Liste
	int DeleteClipList();
	// setzt die aktuelle Clip-Liste
	int SetClipList(LPRGNDATA lpRgn);

	// Pointer auf Interface des Clippers
	DXDLL_LPDIRECTDRAWCLIPPER lpDDClipper;
	// noch ein Pointer auf DirectDraw-Objekt
	CDirectDraw* pDirectDraw;
};

#endif // !defined(AFX_DIRECTDRAW_H__57C7A900_8182_11D2_B0DF_40584EC1BE27__INCLUDED_)
