// DirectDrawClipper.cpp: Implementierung der Klasse CDirectDrawClipper.
//
//////////////////////////////////////////////////////////////////////

#include "DirectDraw.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CDirectDrawClipper::CDirectDrawClipper()
{
	// alles mgliche resetten
	lpDDClipper = NULL;
	pDirectDraw = NULL;

	// ... jetzt den Clipper erzeugen
	DirectDrawClipperCreate();
}

CDirectDrawClipper::CDirectDrawClipper(CDirectDraw* pDD)
{
	// ... resetten
	lpDDClipper = NULL;
	// ... und den Rest bernehmen
	pDirectDraw = pDD;

	// ... jetzt den Clipper erzeugen
	DirectDrawClipperCreate();
}

CDirectDrawClipper::~CDirectDrawClipper()
{
	// Clipper zerstren
	DirectDrawClipperDestroy();
}

int CDirectDrawClipper::DirectDrawClipperCreate()
{
	// altes Clipper-Objekt lschen
	DirectDrawClipperDestroy();

	// ist das DD-Objekt belegt?
	if (pDirectDraw && pDirectDraw->lpDD)
		// dann den Clipper erzeugen
		return pDirectDraw->lpDD->CreateClipper(0, &lpDDClipper, NULL);
	else
		// ... ansonsten einen unabhngigen Clipper erzeugen
		return DirectDrawCreateClipper(0, &lpDDClipper, NULL);

	// ... ein Fehlerchen aufgetreten
	return DXDLL_INTERNALERROR;
}

void CDirectDrawClipper::DirectDrawClipperDestroy()
{
	// existiert Clipperobjekt
	if (lpDDClipper)
	{
		// ... dann dieses lschen
		lpDDClipper->Release();
		lpDDClipper = NULL;
	}
}

int CDirectDrawClipper::SetHWnd()
{
	// existiert ein DirectDraw-Objekt?
	if (pDirectDraw && lpDDClipper)
		// ... dann an Fenster binden
		return lpDDClipper->SetHWnd(0, pDirectDraw->hWindow);

	// ... ansonsten Fehlerhaft
	return DXDLL_INTERNALERROR;
}

int CDirectDrawClipper::SetHWnd(HWND hWnd)
{
	// existiert ein DirectDraw-Objekt?
	if (lpDDClipper)
		// ... dann an Fenster binden
		return lpDDClipper->SetHWnd(0, hWnd);

	// ... ansonsten Fehlerhaft
	return DXDLL_INTERNALERROR;
}

int CDirectDrawClipper::GetHWnd(HWND* hWnd)
{
	// existiert Objekt?
	if (lpDDClipper)
		// ... und aktuelles Handle ermitteln
		return lpDDClipper->GetHWnd(hWnd);

	return DXDLL_INTERNALERROR;
}

int CDirectDrawClipper::IsClipListChanged()
{
	HRESULT hres = DXDLL_INTERNALERROR;

	// existiert Objekt?
	if (lpDDClipper)
	{
		int iChanged;

		// ... und aktuelles Handle ermitteln
		if (SUCCEEDED((hres = lpDDClipper->IsClipListChanged(&iChanged))))
			// Ergebnis zurckliefern
			return iChanged;
	}

	return hres;
}

int CDirectDrawClipper::DeleteClipList()
{
	HRESULT hres = DXDLL_INTERNALERROR;

	// existiert Objekt?
	if (lpDDClipper)
		// ... dann alle Eintrge lschen
		hres = lpDDClipper->SetClipList(NULL, 0);

	// ... und letztes Ergebnis zurckliefern
	return hres;
}

int CDirectDrawClipper::SetClipList(LPRGNDATA lpRgn)
{
	HRESULT hres = DXDLL_INTERNALERROR;

	// existiert Objekt?
	if (lpDDClipper)
		// ... dann alle Eintrge lschen
		hres = lpDDClipper->SetClipList(lpRgn, 0);

	// ... und letztes Ergebnis zurckliefern
	return hres;
}


