// DirectDrawFont.cpp: Implementierung der Klasse CDirectDrawFont.
//
//////////////////////////////////////////////////////////////////////

#include "DirectDraw.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CDirectDrawFont::CDirectDrawFont()
{
	// Font normal erzeugen
	pDirectDrawFont = new CDirectDrawSurface();
	// Einstellungen belegen
	pDirectDrawFont->SetSurfaceDescData(DDSD_CAPS, 0, 0);

	// ... und auch das DirectDraw-Objekt
	pDirectDraw = NULL;

	// Standardwerte setzen
	hFont = NULL;
	iFont = FNT_WHOLECHARS;
	iFontType = FNT_8X8;
	iFontXDimension = 8;
	iFontYDimension = 8;
	iFontLetters = 0;
	iAnzSin = 0;
	iLastSin = 0;
	bIsExtended = false;
	for (int i=0; i<FNT_MAX_CHARS; i++)
		iFontPositions[i] = CHR_NOTPRESENT;
}

CDirectDrawFont::CDirectDrawFont(CDirectDraw* pDD)
{
	// Font normal erzeugen
	pDirectDrawFont = new CDirectDrawSurface(pDD);
	// Einstellungen belegen
	pDirectDrawFont->SetSurfaceDescData(DDSD_CAPS, 0, 0);

	// ... und auch das DirectDraw-Objekt
	pDirectDraw = pDD;

	// Standardwerte setzen
	hFont = NULL;
	iFont = FNT_WHOLECHARS;
	iFontType = FNT_8X8;
	iFontXDimension = 8;
	iFontYDimension = 8;
	iFontLetters = 0;
	iAnzSin = 0;
	iLastSin = 0;
	bIsExtended = false;
	for (int i=0; i<FNT_MAX_CHARS; i++)
		iFontPositions[i] = CHR_NOTPRESENT;
}

CDirectDrawFont::~CDirectDrawFont()
{
	// ein Font vorhanden?
	if (pDirectDrawFont)
		// ... dann zerstren
		delete pDirectDrawFont;
}

bool CDirectDrawFont::IsExtendedFont()
{
	return bIsExtended;
}

bool CDirectDrawFont::IsCreated()
{
	// mindestens ein Buchstabe vorhanden?
	if (iFontLetters == 0 || pDirectDrawFont == NULL)
		return false;
	// bei mindestens einem -> Font erzeugt
	return true;
}

void CDirectDrawFont::DestroyFont()
{
	// ein Font erzeugt?
	if (IsCreated() == true)
	{
		// ... dann alten zerstren
		delete pDirectDrawFont;
		// Handle fr Bitmap belegt?
		if (hFont != NULL)
			// ... und noch das Bitmap lschen
			DeleteObject(hFont);
		// Font erneut erzeugen
		pDirectDrawFont = new CDirectDrawSurface();
		// Einstellungen neu belegen
		pDirectDrawFont->SetSurfaceDescData(DDSD_CAPS, 0, 0);

		// Standardwerte setzen
		iFont = FNT_WHOLECHARS;
		iFontType = FNT_8X8;
		iFontXDimension = 8;
		iFontYDimension = 8;
		iFontLetters = 0;
		bIsExtended = false;
		for (int i=0; i<FNT_MAX_CHARS; i++)
			iFontPositions[i] = CHR_NOTPRESENT;
	}
}

int CDirectDrawFont::CreateNormalFont(int iFnt, int iType)
{
	int iFontDim;
	int iCurrRAM;
	int i;
	int iRes;

	// alten Font zerstren
	DestroyFont();

	// nun Eigenschaften angeben
	iFont = iFnt;
	iFontType = iType;
	iFontXDimension = 8+ iType* 4;
	iFontYDimension = iFontXDimension;
	bIsExtended = false;

	// Flche eines Zeichens berechnen
	iFontDim = iFontXDimension* iFontYDimension;
	iCurrRAM = 0;

	// nun alle Positionen vorberechnen
	switch (iFont)
	{
		// kompletter Zeichensatz von 256 Zeichen
		case FNT_WHOLECHARTABLE:
			for (i=0; i<=FNT_MAX_CHARS; i++)
			{
				// aktuelle Position im virtuellen RAM berechnen
				iCurrRAM = i* iFontDim;
				// schon ber Auflsung 320x200 drberweg?
				if (iCurrRAM < MEM_320X200)
				{
					// ... dann Position berechnen
					iFontPositions[i] = i* iFontXDimension;
					// ... und ein weiterer Buchstabe dazu
					iFontLetters++;
				}
			}
			break;
		// alle Zeichen, auer Zeichen von 0 bis 31
		case FNT_WHOLECHARS:
			for (i=32; i<=FNT_MAX_CHARS; i++)
			{
				// aktuelle Position im virtuellen RAM berechnen
				iCurrRAM = (i- 32)* iFontDim;
				// schon ber Auflsung 320x200 drberweg?
				if (iCurrRAM < MEM_320X200)
				{
					// ... dann Position berechnen
					iFontPositions[i] = (i- 32)* iFontXDimension;
					// ... und ein weiterer Buchstabe dazu
					iFontLetters++;
				}
			}
			break;
		// nur Buchstaben/Zahlen
		case FNT_ONLYLETTERS:
			// Zahlen berechnen
			for (i=48; i<=57; i++)
				// ... dann Position berechnen
				iFontPositions[i] = (i- 48)* iFontXDimension;
			// Grobuchstaben berechnen
			for (i=65; i<=90; i++)
				// ... dann Position berechnen
				iFontPositions[i] = (i- 55)* iFontXDimension;
			// Kleinbuchstaben berechnen
			for (i=97; i<=122; i++)
				// ... dann Position berechnen
				iFontPositions[i] = (i- 61)* iFontXDimension;
			// Anzahl Buchstaben immer gleich
			if (iFnt != FNT_32X32)
				// ... immer 62 Buchstaben
				iFontLetters = 62;
			else
			{
				// ... nur bei 32x32 gibt es maximal 61
				iFontLetters = 61;
				iFontPositions[122] = CHR_NOTPRESENT;
			}
			break;
	}

	// noch Backgroundsurface fr Font erzeugen
	iRes = pDirectDrawFont->CreateOffscreenSurface(
		iFontLetters* iFontXDimension, iFontYDimension);
	// lief alles ok?
	if (FAILED(iRes))
		// ... bei Fehler komplette Berechnung wieder lschen
		DestroyFont();
	else
		// jetzt noch den Fadesurface erzeugen
		if (FAILED(pDirectDrawFont->CreateFadeSurface(iFontXDimension, 
						iFontYDimension)))
			// ... falls Fehler eben Fehlermeldung
			Fail(pDirectDraw->hWindow, DXDLL_MSGERROR_FADESURFACECREATE);

	// ... und Fehlercode zurckliefern
	return iRes;
}

int CDirectDrawFont::CreateExtendedFont(int iFnt, int iType)
{
	int i;
	int iRes;

	// alten Font zerstren
	DestroyFont();

	// nun Eigenschaften angeben
	iFont = iFnt;
	iFontType = iType;
	iFontXDimension = 8+ iFnt* 4;
	iFontYDimension = 8+ iFnt* 4;
	bIsExtended = true;

	// nun alle Positionen vorberechnen
	switch (iFont)
	{
		// kompletter Zeichensatz von 256 Zeichen
		case FNT_WHOLECHARTABLE:
			for (i=0; i<=FNT_MAX_CHARS; i++)
				// ... dann Position berechnen
				iFontPositions[i] = i* iFontXDimension;
			iFontLetters = FNT_MAX_CHARS;
			break;
		// alle Zeichen, auer Zeichen von 0 bis 31
		case FNT_WHOLECHARS:
			for (i=32; i<=FNT_MAX_CHARS; i++)
				// ... dann Position berechnen
				iFontPositions[i] = (i- 32)* iFontXDimension;
			iFontLetters = FNT_MAX_CHARS- 32;
			break;
		// nur Buchstaben/Zahlen
		case FNT_ONLYLETTERS:
			// Zahlen berechnen
			for (i=48; i<=57; i++)
				// ... dann Position berechnen
				iFontPositions[i] = (i- 48)* iFontXDimension;
			// Grobuchstaben berechnen
			for (i=65; i<=90; i++)
				// ... dann Position berechnen
				iFontPositions[i] = (i- 55)* iFontXDimension;
			// Kleinbuchstaben berechnen
			for (i=97; i<=122; i++)
				// ... dann Position berechnen
				iFontPositions[i] = (i- 61)* iFontXDimension;
			// ... immer 62 Buchstaben
			iFontLetters = 62;
			break;
	}

	// noch Backgroundsurface fr Font erzeugen
	iRes = pDirectDrawFont->CreateOffscreenSurface(
		iFontLetters* iFontXDimension, iFontYDimension);
	// lief alles ok?
	if (FAILED(iRes))
		// ... bei Fehler komplette Berechnung wieder lschen
		DestroyFont();
	else
		// jetzt noch den Fadesurface erzeugen
		if (FAILED(pDirectDrawFont->CreateFadeSurface(iFontXDimension, 
						iFontYDimension)))
			// ... falls Fehler eben Fehlermeldung
			Fail(pDirectDraw->hWindow, DXDLL_MSGERROR_FADESURFACECREATE);

	// ... und Fehlercode zurckliefern
	return iRes;
}


int CDirectDrawFont::LoadFont(HINSTANCE hInstance, LPCSTR pFilename,
							  LPCSTR pResName)
{
	int xPos = 0;
	int yPos = 0;
	RECT Rect;
	int i;
	int hres = 0;

	// mindestens ein Font mit Daten gefllt?
	if (IsCreated())
	{
		// ist Bitmaphandle schon belegt?
		if (hFont != NULL)
			// ... dann vorher lschen
			DeleteObject(hFont);

		// neuen temporren Surface erzeugen
		CDirectDrawSurface* pFont = new CDirectDrawSurface(pDirectDraw);

		if (pFont)
		{
			// darauf nun das Bitmap laden
			if ((hFont = (HBITMAP)pFont->LoadBitmap(0, 0, pFilename, 
				pResName, hInstance, false)) != NULL)
			{
				// jetzt gehen wir jeden einzelnen Buchstaben kopieren
				for (i=0; i<=FNT_MAX_CHARS; i++)
				{
					// Zeichen im Zeichensatz vorhanden?
					if (iFontPositions[i] != CHR_NOTPRESENT)
					{
						// Quellrechteck belegen
						Rect.top = yPos;
						Rect.left = xPos;
						Rect.bottom = yPos+ iFontYDimension;
						Rect.right = xPos+ iFontXDimension;
						// ... ein einziges Zeichen kopieren
						pDirectDrawFont->BltFast(iFontPositions[i], 0, 
							pFont, &Rect, DDBLTFAST_NOCOLORKEY);

						// ... x-Position um Auflsung erhhen
						xPos+= iFontXDimension;
						// ... am Ende der Zeile angekommen?
						if (xPos >= PIXEL_WIDTH)
						{
							// ... dann nchste Zeile und x-Pos resetten
							yPos+= iFontYDimension;
							xPos = 0;
						}
					}
					// Bitmap fr Restoreoperationen kopieren
					hFont = pDirectDrawFont->CopySurface(0, 0);
					// ging alles glatt?
					if (!hFont)
					{
						// ... Nein, dann Fehlerchen
						Fail(pDirectDraw->hWindow, 
							DXDLL_MSGERROR_FONTCOMPATIBLEBMPCREATE);
						// noch Fehler markieren
						hres = DXDLL_INTERNALERROR;
					}
				}

				// temporren Surface zerstren
				delete pFont;
				// ... und ohne Fehler raus
				return hres;
			}

			// temporren Surface zerstren
			delete pFont;
			// ... und mit Fehlermeldung raus
			return Fail(pDirectDraw->hWindow, DXDLL_MSGERROR_FONTBITMAPLOAD);
		}

		// ... gab es eben einen Fehler beim erzeugen
		return Fail(pDirectDraw->hWindow, DXDLL_MSGERROR_FONTSURFACECREATE);
	}

	// ... eben Fehler, da noch nichts erzeugt!
	return Fail(pDirectDraw->hWindow, DXDLL_MSGERROR_FONTNODATE);
}

int CDirectDrawFont::SetTransparentColor(char ch, int ix, int iy)
{
	// berhaupt ein Font vorhanden?
	if (IsCreated())
	{
		DXDLL_DDSURFACEDESC ddsdesc;
		RECT Rect;
		DWORD dwCol;
		BYTE bCh = ch;

		// existiert dieses Zeichen?
		if (iFontPositions[bCh] != CHR_NOTPRESENT)
		{
			// Bereich angeben
			Rect.top = iy;
			Rect.bottom = iy;
			Rect.left = ix+ iFontPositions[bCh];
			Rect.right = Rect.left;
			// Locking der Oberflche
			if (SUCCEEDED(pDirectDrawFont->Lock(&Rect, &ddsdesc, DDSD_CAPS,
							DDLOCK_WAIT | DDLOCK_READONLY)))
			{
				// Farbe holen
				dwCol = *((DWORD*)ddsdesc.lpSurface);
				// ... Surface wieder freigeben
				if (SUCCEEDED(pDirectDrawFont->Unlock(&Rect)))
				{
					// jetzt haben wir die Farbe ... also setzen
					if (FAILED(pDirectDrawFont->SetColorKey(dwCol, dwCol, 
							DDCKEY_SRCBLT)))
						// alles fehlerhaft!
						Fail(pDirectDraw->hWindow, 
							DXDLL_MSGERROR_FONTCOLORKEYSET);

					return DXDLL_OK;
				}
				// alles fehlerhaft!
				return Fail(pDirectDraw->hWindow, DXDLL_MSGERROR_FONTUNLOCK);
			}
			// alles fehlerhaft!
			return Fail(pDirectDraw->hWindow, DXDLL_MSGERROR_FONTLOCK);
		}
		// alles fehlerhaft!
		return Fail(pDirectDraw->hWindow, DXDLL_MSGERROR_FONTCHARNOTAVAILABLE);
	}
	// alles fehlerhaft!
	return Fail(pDirectDraw->hWindow, DXDLL_MSGERROR_FONTCOLORKEYSET);
}

int CDirectDrawFont::WriteChar(CDirectDrawSurface* pDestDDS, int ix, 
							   int iy, char ch)
{
	// Nummer im Asciizeichensatz ermitteln
	BYTE bCh = ch;

	// existiert Zeichen berhaupt?
	if (iFontPositions[bCh] != CHR_NOTPRESENT || 
		ix >= pDestDDS->iSurfaceWidth || ix < -iFontXDimension ||
		iy >= pDestDDS->iSurfaceHeight || iy < -iFontYDimension)
	{
		RECT Rect;

		// Quellposition berechnen
		// ... zuerst die y-Position
		if (iy < 0)
		{
			Rect.top = -iy;
			Rect.bottom = iFontYDimension+ iy;
			iy = 0;
		}
		else
			if (iy > pDestDDS->iSurfaceHeight- iFontYDimension)
			{
				Rect.top = 0;
				Rect.bottom = pDestDDS->iSurfaceHeight- iy;
			}
			else
			{
				Rect.top = 0;
				Rect.bottom = iFontYDimension;
			}

		// ... dann die x-Position
		if (ix < 0)
		{
			Rect.left = iFontPositions[bCh]- ix;
			Rect.right = Rect.left+ iFontXDimension+ ix;
			ix = 0;
		}
		else
			if (ix > pDestDDS->iSurfaceWidth- iFontXDimension)
			{
				Rect.left = iFontPositions[bCh];
				Rect.right = Rect.left+ pDestDDS->iSurfaceWidth- ix;
			}
			else
			{
				Rect.left = iFontPositions[bCh];
				Rect.right = Rect.left+ iFontXDimension;
			}

		// Zeichen anzeigen und Routine beenden
//		return pDestDDS->BltFast(ix, iy, pDirectDrawFont, &Rect, 
//			DDBLTFAST_SRCCOLORKEY);
		return pDestDDS->Blt(ix, iy, pDirectDrawFont, &Rect, 
			DDBLT_KEYSRC);
//		return pDestDDS->BltFade(ix, iy, pDirectDrawFont, &Rect, 
//			DDBLT_KEYSRC, 10, 10, 10);
	}

	// Fehler, da Zeichen nicht gezeichnet wurde
	return DXDLL_INTERNALERROR;
}

int CDirectDrawFont::WriteSinChar(CDirectDrawSurface* pDestDDS, int ix,
								  int iy, char ch)
{
	// Nummer im Asciizeichensatz ermitteln
	BYTE bCh = ch;
	int i;
	int ixNew = ix;
	int iyNew;

	// existiert Zeichen berhaupt?
	if (iFontPositions[bCh] != CHR_NOTPRESENT || 
		ix >= pDestDDS->iSurfaceWidth || ix < -iFontXDimension ||
		iy >= pDestDDS->iSurfaceHeight || iy < -iFontYDimension)
	{
		RECT Rect;

		for (i=iFontPositions[bCh]; i<iFontPositions[bCh]+ 
			iFontXDimension; i++)
		{
			// innerhalb Fenster?
			if (ixNew >= 0 && ixNew <= pDestDDS->iSurfaceWidth)
			{
				// obere Kante berechnen
				iyNew = iy- iSinWave[iLastSin];

				// linke und rechte Kante setzen
				Rect.left = i;
				Rect.right = i+ 1;

				// liegt obere Ecke ber BS-Rand?
				if (iyNew < 0)
				{
					// ... dann diese korrigieren
					Rect.top = -iyNew;
					Rect.bottom = iFontYDimension;
					// und noch Startposition
					iyNew = 0;
				}
				else
					// vielleicht untere Kante ber BS?
					if (iyNew+ iFontYDimension > pDestDDS->iSurfaceHeight- 1)
					{
						// ... dann auch korrigieren
						Rect.top = 0;
						Rect.bottom = iFontYDimension+ iyNew-
							pDestDDS->iSurfaceHeight;
					}
					else
					{
						// ... ansonsten nur bernehmen
						Rect.top = 0;
						Rect.bottom = iFontYDimension;
					}
				// Zeichen anzeigen und Routine beenden
				pDestDDS->BltFast(ixNew, iyNew, pDirectDrawFont, 
					&Rect, DDBLTFAST_SRCCOLORKEY);
			}
			// nchste Spalte
			ixNew++;
			// nchster Sinus-Idx
			iLastSin = (++iLastSin) % iAnzSin;
		}
	}

	// Fehler, da Zeichen nicht gezeichnet wurde
	return DXDLL_OK;
}

int CDirectDrawFont::WriteStretchedChar(CDirectDrawSurface* pDestDDS,
										int ix, int iy, int iWidth,
										int iHeight, char ch)
{
	// Nummer im Asciizeichensatz ermitteln
	BYTE bCh = ch;

	// existiert Zeichen berhaupt?
	if (iFontPositions[bCh] != CHR_NOTPRESENT || 
		ix >= pDestDDS->iSurfaceWidth || ix < -iWidth ||
		iy >= pDestDDS->iSurfaceHeight || iy < -iHeight)
	{
		RECT DestRect;
		RECT SrcRect;
		// Stretchfaktor der Korrektur wegen vorberechnen
		double dsx = 1;//iFontXDimension/ abs(iWidth);
		double dsy = iFontYDimension/ abs(iHeight);

		// Quellposition berechnen
		// ... zuerst die y-Position
		if (iy < 0)
		{
			// Quellrechteck festlegen
			SrcRect.top = -int(iy* dsy);
			SrcRect.bottom = int(iFontYDimension+ (iy* dsy));
			// Zielrechteck festlegen
			DestRect.top = 0;
			DestRect.bottom = iy+ iHeight;
		}
		else
			if (iy > pDestDDS->iSurfaceHeight- iHeight)
			{
				// Quellrechtreck komplett neu berechnen
				SrcRect.top = 0;
				SrcRect.bottom = int(iFontYDimension- 
					(dsy* (iHeight+ iy- pDestDDS->iSurfaceHeight)));
				// Zielrechteck ist immer dasselbe
				DestRect.top = iy;
				DestRect.bottom = pDestDDS->iSurfaceHeight- 1;
			}
			else
			{
				// Quellrechteck festlegen
				SrcRect.top = 0;
				SrcRect.bottom = iFontYDimension;
				// Zielrechteck festlegen
				DestRect.top = iy;
				DestRect.bottom = iy+ iHeight;
			}

		// ... dann die x-Position
		if (ix < 0)
		{
			// Quellrechteck festlegen
			SrcRect.left = -int(ix* dsx)+ iFontPositions[bCh];
			SrcRect.right = SrcRect.left+ int(iFontXDimension+ (ix* dsx));
			// Zielrechteck festlegen
			DestRect.left = 0;
			DestRect.right = ix+ iWidth;
		}
		else
			if (ix > pDestDDS->iSurfaceWidth- iWidth)
			{
				// Quellrechtreck komplett neu berechnen
				SrcRect.left = iFontPositions[bCh];
				SrcRect.right = int(iFontPositions[bCh]+ 
					iFontXDimension- (dsx* (iWidth+ ix- 
					pDestDDS->iSurfaceWidth)));
				// Zielrechteck ist immer dasselbe
				DestRect.left = ix;
				DestRect.right = pDestDDS->iSurfaceWidth- 1;
			}
			else
			{
				// hier ist alles innerhalb BS!
				SrcRect.left = iFontPositions[bCh];
				SrcRect.right = iFontPositions[bCh]+ 
					iFontXDimension;
				// ... also eben alles so lassen!
				DestRect.left = ix;
				DestRect.right = ix+ iWidth;
			}

		// Zeichen anzeigen und Routine beenden
		return pDestDDS->Blt(&DestRect, pDirectDrawFont, &SrcRect, 
			DDBLT_KEYSRC);
	}

	// Fehler, da Zeichen nicht gezeichnet wurde
	return DXDLL_INTERNALERROR;
}

void CDirectDrawFont::WriteString(CDirectDrawSurface *pDestDDS, int ix, 
								  int iy, LPCSTR lpStr)
{
	// aktuelle Position
	int ixNow = ix;
	int i = 0;

	// jedes einzelne Zeichen ausgeben
	while (lpStr[i] != '\0')
	{
		// aktuelles Zeichen anzeigen
		if (FAILED(WriteChar(pDestDDS, ixNow, iy, lpStr[i])))
			// ... bei Fehler Ausgabe beenden
			break;
		// ... ansonsten nchste Position
		ixNow+= iFontXDimension;
		i++;
	}
}

void CDirectDrawFont::WriteStretchedString(CDirectDrawSurface *pDestDDS, int ix, 
										   int iy, int iWidth, int iHeight,
										   LPCSTR lpStr)
{
	// aktuelle Position
	int ixNow = ix;
	int i = 0;

	// jedes einzelne Zeichen ausgeben
	while (lpStr[i] != '\0')
	{
		// aktuelles Zeichen anzeigen
		if (FAILED(WriteStretchedChar(pDestDDS, ixNow, iy, 
			iWidth, iHeight, lpStr[i])))
			// ... bei Fehler Ausgabe beenden
			break;
		// ... ansonsten nchste Position
		ixNow+= iWidth;
		i++;
	}
}

void CDirectDrawFont::WriteSinString(CDirectDrawSurface *pDestDDS, int ix, 
									 int iy, LPCSTR lpStr)
{
	// aktuelle Position
	int ixNow = ix;
	int i = 0;

	// jedes einzelne Zeichen ausgeben
	while (lpStr[i] != '\0')
	{
		// aktuelles Zeichen anzeigen
		if (FAILED(WriteSinChar(pDestDDS, ixNow, iy, lpStr[i])))
			// ... bei Fehler Ausgabe beenden
			break;
		// ... ansonsten nchste Position
		ixNow+= iFontXDimension;
		i++;
	}
}

void CDirectDrawFont::CreateSinWave(double dAmp, double dStep)
{
	double dWaveIdx = 0.00;
	int iArrayIdx = 0;

	// solange laufen, bis Array voll
	while (dWaveIdx < 360)
	{
		// Wert berechnen
		iSinWave[iArrayIdx] = (int)round(sin(PI/ 180* dWaveIdx)* dAmp, 0);
		// nchster Index und Grad
		iArrayIdx++;
		dWaveIdx+= dStep;
	}
	// Anzahl Werte speichern
	iAnzSin = --iArrayIdx;
	iLastSin = 0;
}

int CDirectDrawFont::CheckSurfaceFont()
{
	// Font noch vorhanden?
	if (pDirectDrawFont->IsLost())
		// ... dann mu dieser restauriert werden
		return pDirectDrawFont->Restore(0, 0, hFont);

	return DXDLL_OK;
}
