// DirectInfo.h: Schnittstelle fr alle Klassen, mit denen Infos ber
//				 DirectX-Gerte ermittelt werden knnen
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DIRECTINFO_H__57C7A900_8182_11D2_B0DF_40584EC1BE27__INCLUDED_)
#define AFX_DIRECTINFO_H__57C7A900_8182_11D2_B0DF_40584EC1BE27__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// alle mglichen Defines
#include "DXDefines.h"
// alle mglichen Tools
#include "Tools.h"
// DirectDraw-Verwaltung
#include "ddraw.h"
// DirectSound-Verwaltung
#include "dsound.h"
// DirectInput-Verwaltung
#include "dinput.h"
// Direct3D-Verwaltung
#include "d3d.h"
#include "d3dcaps.h"
#include "d3drm.h"
#include "d3drmdef.h"
#include "d3drmobj.h"
#include "d3drmwin.h"
#include "d3dtypes.h"


// -------------------------------------------
// Forwarddeklarationen der einzelnen Klassen
// -------------------------------------------
class CDirectXInfo;
class CDirectXDriverInfo;


// ----------------------------------------------------
// richtige komplette Definionen der einzelnen Klassen
// ----------------------------------------------------
class __declspec(dllexport) CDirectXInfo
{
public:
	// -------------------------------------------
	// untersttzte Treibertypen:
	// -------------------------------------------
	// - DXI_DIRECTDRAW		... DirectDraw-Gert
	// - DXI_DIRECT3D		... Direct3D-Gert
	// - DXI_DIRECTSOUND	... DirectSound-Gert
	// - DXI_DIRECTINPUT	... DirectInput-Gert
	// -------------------------------------------

	CDirectXInfo();
	virtual ~CDirectXInfo();

	// Zhlt im System vorhandene Gerte des Typs ab
	// ACHTUNG:
	// die beiden Zusatzparameter sind zur Zeit noch
	// unbelegt, auer, es sollen alle DirectInput-Gerte
	// abgezhlt werden, dann entspricht lpLParam der
	// Programminstance
	// Wenn D3D-Gerte gezhlt werden sollen, so ist HParam
	// mit einem Pointer auf eine DirectDraw-Gerteinfo zu
	// belegen
	int EnumerateDirectXDrivers(int iInterface, LPVOID lpLParam,
		LPVOID lpHParam);

	// fgt neues Gert des angegebenen Typs zur Liste hinzu
	int AddDirectXDriver(int iInterface, GUID* lpGUID, 
		LPSTR lpDesc, LPSTR lpName, LPVOID lpDescHEL,
		LPVOID lpDescHAL);

	// zerstrt alle gesammelten Informationen zur Hardware
	void DestroyDirectXInfos();

	// holt sich Informationen zum angegebenen Typ
	CDirectXDriverInfo* GetDirectXDriverInfo(int iInterface,
		int iDriver);
	// gibt Anzahl gefundener Gerte des Typs zurck
	int GetDirectXDriversCount(int iInterface);

	// gibt GUID des Typs mit der Treibernummer x zurck
	GUID* GetDirectXDriverGUID(int iInterface, int iDriver);
	// gibt Beschreibung des Typs mit der Treibernummer x zurck
	LPSTR GetDirectXDriverDescription(int iInterface, int iDriver);
	// gibt Name des Typs mit der Treibernummer x zurck
	LPSTR GetDirectXDriverName(int iInterface, int iDriver);
	// gibt Infos zur HEL des Typs mit der Treibernummer x zurck
	LPVOID GetDirectXDriverHEL(int iInterface, int iDriver);
	// gibt Infos zur HAL des Typs mit der Treibernummer x zurck
	LPVOID GetDirectXDriverHAL(int iInterface, int iDriver);
	
	// ermittelt aktuelle DX-Version und speichert diese in
	// Klasse ab.
	int FindDirectXVersion();

	// gibt aktuelle Versionsnummer von DirectX zurck
	// (nicht bentigte Werte mit NULL kennzeichnen!)
	// VORSICHT:
	// Bei Verwendung am Anfang des Programmes FindDirectXVersion
	// aufrufen und die DLL's "DSETUP.DLL", "DSETUP16.DLL" und
	// "DSETUP32.DLL" fr DLL auffindbar haben!
	void GetDirectXVersion(int* lpiMainVersion, int* lpiSecondVersion,
		int* lpiMainRevision, int* lpiSecondRevision);

	// berprft aktuelle Versionsnummer, ob mit DLL kompatibel
	// return ... DXDLL_OK - Versionsnummer kompatibel
	//			  DXDLL_INTERNALERROR - keine korrekte DX-Version
	// VORSICHT:
	// Bei Verwendung am Anfang des Programmes FindDirectXVersion
	// aufrufen und die DLL's "DSETUP.DLL", "DSETUP16.DLL" und
	// "DSETUP32.DLL" fr DLL auffindbar haben!
	int CheckCurrentVersion();

private:
	// Informationen zu den DirectDraw-Gerten
	CDirectXDriverInfo* pDirectDrawInfo[MAX_DRIVERS];
	// Informationen zu den DirectSound-Gerten
	CDirectXDriverInfo* pDirectSoundInfo[MAX_DRIVERS];
	// Informationen zu den DirectInput-Gerten
	CDirectXDriverInfo* pDirectInputInfo[MAX_DRIVERS];

	// Anzahl der gefundenen DirectDraw-Gerte
	int iDirectDrawDriversCount;
	// Anzahl der gefundenen DirectSound-Gerte
	int iDirectSoundDriversCount;
	// Anzahl der gefundenen DirectInput-Gerte
	int iDirectInputDriversCount;

	// Haupptnummer der Version (bei 6.0 -> 4)
	int iMainVersion;
	// Unternummer der Version (bei 6.0 -> 6)
	int iSecondVersion;

	// Hauptnummer der Revision
	int iMainRevision;
	// Unternummer der Revision
	int iSecondRevision;
};

class __declspec(dllexport) CDirectXDriverInfo
{
public:
	CDirectXDriverInfo();
	CDirectXDriverInfo(GUID *lpGUID, LPSTR lpDescription,
		LPSTR lpName, int iDXDeviceType, LPVOID lpDescHEL,
		LPVOID lpDescHAL);
	virtual ~CDirectXDriverInfo();

	// fgt einen Direct3D-Treiber zum DirectDraw-Objekt hinzu
	int AddDirect3DDriver(GUID *lpGUID, LPSTR lpDesc, 
		LPSTR lpName, LPVOID lpDescHEL, LPVOID lpDescHAL);
	// fgt verfgbare Auflsung des DirectDraw-Gertes hinzu
	int AddDirectDrawResolution(DXDLL_DDSURFACEDESC* lpDesc);

	// holt sich den Descriptor fr entsprechende Auflsung
	DXDLL_DDSURFACEDESC* GetResolutionDesc(int iRes);
	// holt sich die Auflsungsdaten fr entsprechende Auflsung
	int GetResolution(int iRes, int* iWidth, int* iHeight,
		int* iBits, int* iRefresh);
	// holt sich die Anzahl verfgbarer Auflsungen
	int GetResolutionsCount();

	// setzt Treiberinformationen fr diese Klasse
	int SetDriverInfo(GUID* lpGUID, LPSTR lpDescription, 
		LPSTR lpName, int iDXDeviceType, LPVOID lpDescHEL,
		LPVOID lpDescHAL);
	// zerstrt alle Treiberinformationen
	void DestroyDriverInfo();

	// holt GUID des Treibers
	GUID* GetDriverGUID();
	// holt Beschreibung des Treibers
	LPSTR GetDriverDescription();
	// holt Namen des Treibers
	LPSTR GetDriverName();
	// holt DirectX-Gertetyp des Treibers
	int GetDriverDeviceType();
	// holt Informationen ber HEL zu diesem Gert
	LPVOID GetDriverDescInfoHEL();
	// holt Informationen ber HAL zu diesem Gert
	LPVOID GetDriverDescInfoHAL();

	// holt sich Informationen zum angegebenen Typ
	CDirectXDriverInfo* GetD3DDriverInfo(int iDriver);
	// gibt Anzahl gefundener Gerte des Typs zurck
	int GetD3DDriversCount();

	// gibt GUID des Typs mit der Treibernummer x zurck
	GUID* GetD3DDriverGUID(int iDriver);
	// gibt Beschreibung des Typs mit der Treibernummer x zurck
	LPSTR GetD3DDriverDescription(int iDriver);
	// gibt Name des Typs mit der Treibernummer x zurck
	LPSTR GetD3DDriverName(int iDriver);
	// gibt Infos zur HEL des Typs mit der Treibernummer x zurck
	LPVOID GetD3DDriverHEL(int iDriver);
	// gibt Infos zur HAL des Typs mit der Treibernummer x zurck
	LPVOID GetD3DDriverHAL(int iDriver);

private:
	GUID* lpDriverGUID;
	char lpDriverDescription[MAX_DRIVERDESC_SIZE];
	char lpDriverName[MAX_DRIVERNAME_SIZE];
	// -------------------------------------------
	// untersttzte Treibertypen:
	// -------------------------------------------
	// - DXI_DIRECTDRAW		... DirectDraw-Gert
	// - DXI_DIRECT3D		... Direct3D-Gert
	// - DXI_DIRECTSOUND	... DirectSound-Gert
	// -------------------------------------------
	int iDirectXDeviceType;
	// sonstige Informationen zu Device (HAL)
	LPVOID lpDescInfoHAL;
	// sonstige Informationen zu Device (HEL)
	LPVOID lpDescInfoHEL;

	// Anzahl der Direct3D-Gerte
	int iDirect3DDriversCount;
	// Anzahl der verfgbaren Auflsungen
	int iDirectDrawResolutionsCount;
	// noch die einzelnen Direct3D-Gerte, falls ein DirectDraw-Objekt
	CDirectXDriverInfo* pD3DInfo[MAX_DRIVERS];
	// die verfgbaren Auflsungen, des DirectDraw-Gertes
	DXDLL_DDSURFACEDESC* lpResolutions[MAX_RESOLUTIONS];
};

#endif // !defined(AFX_DIRECTINFO_H__57C7A900_8182_11D2_B0DF_40584EC1BE27__INCLUDED_)
