// DirectInput.h: Schnittstelle fr die Implementation der Komponenten
//				  von DirectInput
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DIRECTINPUT_H__42CC6500_BDFE_11D2_AD5A_DAE683505A4B__INCLUDED_)
#define AFX_DIRECTINPUT_H__42CC6500_BDFE_11D2_AD5A_DAE683505A4B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// alle mglichen Defines
#include "DXDefines.h"
// alle mglichen Tools
#include "Tools.h"
// DirectInput-Verwaltung
#include "dinput.h"


// -------------------------------------------
// einige Defines, die das Leben vereinfachen
// -------------------------------------------
#define BUTTONDOWN(b) (b & 0x80)


// -------------------------------------------
// Typendeklarationen der einzelnen Klassen
// -------------------------------------------
typedef BYTE DIKEYBOARDDATA[256];


// -------------------------------------------
// Forwarddeklarationen der einzelnen Klassen
// -------------------------------------------
class CDirectInput;
class CDirectInputDevice;


// ----------------------------------------------------
// richtige komplette Definionen der einzelnen Klassen
// ----------------------------------------------------
class __declspec(dllexport) CDirectInput  
{
public:
	CDirectInput();
	CDirectInput(HINSTANCE hInst, HWND hWnd);
	virtual ~CDirectInput();

	// erzeugt ein DirectInput-Objekt
	int DirectInputCreate();
	// zerstrt komplettes DirectInput-Objekt
	DirectInputDestroy();

	// holt den Gertestatus eines Gertes
	int GetDeviceStatus(GUID* lpGUID);
	// startet das ControlPanel von Win9x
	int RunControlPanel();

	// Pointer auf Interface von DirectInput
	DXDLL_LPDIRECTINPUT lpDI;
	// Daten des Fensters/der Anwendung, zu dem Objekt gehrt
	HINSTANCE hInstance;
	HWND hWindow;
};

class __declspec(dllexport) CDirectInputDevice
{
public:
	CDirectInputDevice();
	CDirectInputDevice(CDirectInput* pDI);
	virtual ~CDirectInputDevice();

	// erzeugt ein beliebiges Device
	// ACHTUNG:
	// nach Aufruf mu noch Cooperative-Level gesetzt werden!!
	int CreateDevice(REFGUID refGUID);

	// erzeugt ein Keyboard-Device (Systemkeyboard)
	int CreateKeyboardDevice(DWORD dwCooperativeFlags);
	// erzeugt ein gepuffertes Keyboard-Device (Systemkeyboard)
	int CreateBufferedKeyboardDevice(DWORD dwBufferSize,
		DWORD dwCooperativeFlags);
	// erzeugt ein Mouse-Device (Systemmouse)
	int CreateMouseDevice(DWORD dwCooperativeFlags);
	// erzeugt ein gepuffertes Mouse-Device (Systemmouse)
	int CreateBufferedMouseDevice(DWORD dwBufferSize,
		DWORD dwCooperativeFlags);

	// zerstrt aktuelles Device
	void DestroyDevice();

	// ermittelt aktuelle Devicetyp
	// DIDEVTYPE_KEYBOARD	... Tastatur
	// DIDEVTYPE_MOUSE		... Maus
	// DIDEVTYPE_JOYSTICK	... Joystick
	int GetCurrentDeviceType();
	// setzt Eigenschaften des Devices, welches Verhalten
	// beeinflusst (siehe Hilfe fr mehr Informationen)
	int SetProperty(REFGUID rguidProp, LPCDIPROPHEADER pdiph);

	// holt Informationen zu jeder einzelnen Taste
	// ACHTUNG:
	// Nur verwenden, wenn Keyboard-Device erzeugt
	// Keine Checks, ob Device bereits erzeugt
	int GetKeyboardData(DIKEYBOARDDATA chBuffer);
	// holt Informationen zu jeder gepufferten Taste
	// ACHTUNG:
	// Nur verwenden, wenn gepuffertes Keyboard-Device erzeugt
	// Keine Checks, ob Device bereits erzeugt
	int GetBufferedKeyboardData(DIDEVICEOBJECTDATA* didod);

	// holt Informationen zur Maus (rel. Position, ...)
	// ACHTUNG:
	// Nur verwenden, wenn Mouse-Device erzeugt
	// Keine Checks, ob Device bereits erzeugt
	int GetMouseData(DIMOUSESTATE* dims);
	// holt Informationen zu jeder gepufferten Mausaktion
	// ACHTUNG:
	// Nur verwenden, wenn gepuffertes Mouse-Device erzeugt
	// Keine Checks, ob Device bereits erzeugt
	int GetBufferedMouseData(DIDEVICEOBJECTDATA* didod);

	// holt Informationen zum Joystick (rel. Position, ...)
	// ACHTUNG:
	// Nur verwenden, wenn Joystick-Device erzeugt
	// Keine Checks, ob Device bereits erzeugt
	int GetJoystickData(DIJOYSTATE2* dijs);
	// holt Informationen zu jeder gepufferten Joystickaktion
	// ACHTUNG:
	// Nur verwenden, wenn gepuffertes Joystick-Device erzeugt
	// Keine Checks, ob Device bereits erzeugt
	int GetBufferedJoystickData(DIDEVICEOBJECTDATA* didod);

	// holt Daten des aktuellen Devices (gepufferte, ...)
	// ACHTUNG:
	// Keine Checks, ob Device bereits erzeugt
	int GetCurrentData(LPVOID lpdid);

	// ruft ControlPanel von Win9x des aktuellen Devices auf
	int RunControlPanel();

	int Acquire();
	int Unacquire();

	// der Pointer auf Schnittstelle von Device
	DXDLL_LPDIRECTINPUTDEVICE lpDIDevice;
	// Pointer auf DirectInput-Klasse
	CDirectInput* pDirectInput;
	// aktueller DeviceType (siehe GetCurrentDeviceType())
	int iCurrentDevType;
	// sollen Daten gepuffert werden?
	unsigned long iBuffered;
};

#endif // !defined(AFX_DIRECTINPUT_H__42CC6500_BDFE_11D2_AD5A_DAE683505A4B__INCLUDED_)
