// DirectInputDevice.cpp: Implementierung der Klasse CDirectInputDevice.
//
//////////////////////////////////////////////////////////////////////

#include "DirectInput.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CDirectInputDevice::CDirectInputDevice()
{
	// alles mgliche resetten
	lpDIDevice = NULL;
	pDirectInput = NULL;

	iBuffered = 0;
	iCurrentDevType = 0;
}

CDirectInputDevice::CDirectInputDevice(CDirectInput* pDI)
{
	// alles mgliche resetten
	lpDIDevice = NULL;

	iBuffered = 0;
	iCurrentDevType = 0;

	// und das bernehmen
	pDirectInput = pDI;
}

CDirectInputDevice::~CDirectInputDevice()
{
	// noch alles auslschen
	DestroyDevice();
}

int CDirectInputDevice::CreateDevice(REFGUID refGUID)
{
	// sind wir mit einem DirectInput-Klasse verbunden?
	if (pDirectInput)
	{
		// auf ein DirectInput-Objekt erzeugt?
		if (pDirectInput->lpDI)
		{
			LPDIRECTINPUTDEVICE lpDID;
			DIDEVICEINSTANCE diDevInfo;
			HRESULT hres = DI_OK;

			// vorher noch altes zerstren
			DestroyDevice();

			// ... dann geht's hier weiter
			hres = pDirectInput->lpDI->CreateDevice(
				refGUID, &lpDID, NULL);

			// wurde alles ordnungsgem erzeugt?
			if (SUCCEEDED(hres))
			{
				// per Query Schnittstelle auf DirectInputDevice2 holen
				hres = lpDID->QueryInterface(DXDLL_GUID_DIRECTINPUTDEVICE, 
					(LPVOID*)&lpDIDevice);

				// war auch das in Ordnung
				if (SUCCEEDED(hres))
				{
					// jetzt den Typ holen
					diDevInfo.dwSize = sizeof(DIDEVICEINSTANCE);
					// Informationen zu Device holen
					hres = lpDIDevice->GetDeviceInfo(&diDevInfo);
					if (SUCCEEDED(hres))
					{
						// jetzt auf alle bekannten Typen testen
						switch (GET_DIDEVICE_TYPE(diDevInfo.dwDevType))
						{
							case DIDEVTYPE_KEYBOARD:
								// noch das Ausgabeformat auf Keyboard setzen
								hres = lpDIDevice->SetDataFormat(&c_dfDIKeyboard);
								iCurrentDevType = DIDEVTYPE_KEYBOARD;
								break;
							case DIDEVTYPE_MOUSE:
								// noch das Ausgabeformat auf Mouse setzen
								hres = lpDIDevice->SetDataFormat(&c_dfDIMouse);
								iCurrentDevType = DIDEVTYPE_MOUSE;
								break;
							case DIDEVTYPE_JOYSTICK:
								// noch das Ausgabeformat auf Joystick setzen
								hres = lpDIDevice->SetDataFormat(&c_dfDIJoystick2);
								iCurrentDevType = DIDEVTYPE_JOYSTICK;
								break;
							default:
								hres = DXDLL_INTERNALERROR;
						}
						// lief alles ok?
						if (FAILED(hres))
							// Nein! Fehlermeldung!
							Fail(pDirectInput->hWindow, 
								DXDLL_MSGERROR_DINPUTDEVDATAFORMATSET);
					}
					else
						// ... dann mit Fehlermeldung raus
						return Fail(pDirectInput->hWindow, 
							DXDLL_MSGERROR_DINPUTDEVTYPEGET);
				}
				else
				{
					// ... bei Fehler nur Fehlermeldung ausgeben
					Fail(pDirectInput->hWindow, 
						DXDLL_MSGERROR_DINPUTDEV2CREATE);
					lpDIDevice = NULL;
				}

				// uraltes Objekt wieder freigeben
				lpDID->Release();
			}
			else
				// ... dann mit Fehlermeldung raus
				return Fail(pDirectInput->hWindow, 
					DXDLL_MSGERROR_DINPUTDEVCREATE);
			return hres;
		}
	}

	return DXDLL_INTERNALERROR;
}

int CDirectInputDevice::CreateKeyboardDevice(DWORD dwCooperativeFlags)
{
	HRESULT hres = DI_OK;

	// Device erzeugen
	hres = CreateDevice(GUID_SysKeyboard);
	// Fehler?
	if (SUCCEEDED(hres))
	{
		// als letztes das Cooperative-Level setzen
		hres = lpDIDevice->SetCooperativeLevel(
			pDirectInput->hWindow, dwCooperativeFlags);
		// war hier ein Fehler?
		if (FAILED(hres))
			// ... Fehlermeldung
			Fail(pDirectInput->hWindow, DXDLL_MSGERROR_DINPUTDEVCOOPLEVEL);
	}

	return hres;
}

int CDirectInputDevice::CreateBufferedKeyboardDevice(DWORD dwBufferSize,
													 DWORD dwCooperativeFlags)
{
	HRESULT hres = DI_OK;

	// Device erzeugen
	hres = CreateDevice(GUID_SysKeyboard);
	// Fehler?
	if (SUCCEEDED(hres))
	{
		// als letztes das Cooperative-Level setzen
		hres = lpDIDevice->SetCooperativeLevel(
			pDirectInput->hWindow, dwCooperativeFlags);
		// war hier ein Fehler?
		if (SUCCEEDED(hres))
		{
			// nun noch die Anweisung fr buffered geben
			DIPROPDWORD diprop =
			{
				{
					sizeof(DIPROPDWORD),		// diph.dwSize
					sizeof(DIPROPHEADER),		// diph.dwHeaderSize
					0,							// diph.dwObj
					DIPH_DEVICE,				// diph.dwHow
				},
				dwBufferSize,					// dwData
			};
			// das waren die Werte, welche immer gleich sind!
			// --> jetzt diese setzen
			hres = SetProperty(DIPROP_BUFFERSIZE, &diprop.diph);
			// ging alles klar?
			if (FAILED(hres))
				// ... nur Fehler ausgeben
				Fail(pDirectInput->hWindow, DXDLL_MSGERROR_DINPUTDEVPROPFORMATSET);
		}
		else
			// ... Fehlermeldung
			Fail(pDirectInput->hWindow, DXDLL_MSGERROR_DINPUTDEVCOOPLEVEL);
	}

	return hres;
}

int CDirectInputDevice::CreateMouseDevice(DWORD dwCooperativeFlags)
{
	HRESULT hres = DI_OK;

	// Device erzeugen
	hres = CreateDevice(GUID_SysMouse);
	// Fehler?
	if (SUCCEEDED(hres))
	{
		// als letztes das Cooperative-Level setzen
		hres = lpDIDevice->SetCooperativeLevel(
			pDirectInput->hWindow, dwCooperativeFlags);
		// war hier ein Fehler?
		if (FAILED(hres))
			// ... Fehlermeldung
			Fail(pDirectInput->hWindow, DXDLL_MSGERROR_DINPUTDEVCOOPLEVEL);
	}

	return hres;
}

int CDirectInputDevice::CreateBufferedMouseDevice(DWORD dwBufferSize,
												  DWORD dwCooperativeFlags)
{
	HRESULT hres = DI_OK;

	// Device erzeugen
	hres = CreateDevice(GUID_SysMouse);
	// Fehler?
	if (SUCCEEDED(hres))
	{
		// als letztes das Cooperative-Level setzen
		hres = lpDIDevice->SetCooperativeLevel(
			pDirectInput->hWindow, dwCooperativeFlags);
		// war hier ein Fehler?
		if (SUCCEEDED(hres))
		{
			// nun noch die Anweisung fr buffered geben
			DIPROPDWORD diprop =
			{
				{
					sizeof(DIPROPDWORD),		// diph.dwSize
					sizeof(DIPROPHEADER),		// diph.dwHeaderSize
					0,							// diph.dwObj
					DIPH_DEVICE,				// diph.dwHow
				},
				dwBufferSize,					// dwData
			};
			// das waren die Werte, welche immer gleich sind!
			// --> jetzt diese setzen
			hres = SetProperty(DIPROP_BUFFERSIZE, &diprop.diph);
			// ging alles klar?
			if (FAILED(hres))
				// ... nur Fehler ausgeben
				Fail(pDirectInput->hWindow, DXDLL_MSGERROR_DINPUTDEVDATAFORMATSET);
		}
		else
			// ... Fehlermeldung
			Fail(pDirectInput->hWindow, DXDLL_MSGERROR_DINPUTDEVCOOPLEVEL);
	}

	return hres;
}

void CDirectInputDevice::DestroyDevice()
{
	// ist eins da?
	if (lpDIDevice)
	{
		HRESULT hres;

		// erstmal Verbindung lsen
		hres = Unacquire();
		// ... dann auslschen
		lpDIDevice->Release();
		lpDIDevice = NULL;
	}

	// den Rest auch noch resetten
	iBuffered = 0;
	iCurrentDevType = 0;
}

int CDirectInputDevice::GetCurrentDeviceType()
{
	// ist Device belegt?
	if (lpDIDevice)
	{
		DIDEVICEINSTANCE diDevInfo;
		HRESULT hres;

		// Informationen einholen
		hres = lpDIDevice->GetDeviceInfo(&diDevInfo);
		// bei Fehlerchen raus
		if (FAILED(hres))
			return DXDLL_INTERNALERROR;

		// und den Typen zurckliefern und gleich mit speichern
		return (iCurrentDevType = GET_DIDEVICE_TYPE(diDevInfo.dwDevType));
	}

	return DXDLL_INTERNALERROR;
}

int CDirectInputDevice::SetProperty(REFGUID rguidProp,
									LPCDIPROPHEADER pdiph)
{
	// existiert das DirectInputDevice-Objekt?
	if (lpDIDevice)
	{
		// soll die Buffergre gesetzt werden?
		if (rguidProp == DIPROP_BUFFERSIZE)
		{
			HRESULT hres;

			// ausfhren
			hres = SetProperty(rguidProp, pdiph);
			// gab's Probleme
			if (FAILED(hres))
				iBuffered = 0;
			else
				// ... ansonsten Gre speichern
				iBuffered = DWORD(pdiph+ pdiph->dwHeaderSize);
		}
		else
			// ansonsten ganz normale Routine aufrufen
			return lpDIDevice->SetProperty(rguidProp, pdiph);
	}

	return DXDLL_INTERNALERROR;
}

int CDirectInputDevice::GetKeyboardData(DIKEYBOARDDATA chBuffer)
{
	HRESULT hres = DI_OK;

	// vorher mal Poll auslsen
	lpDIDevice->Poll();

	// erstes mal Informationen holen
	hres = lpDIDevice->GetDeviceState(sizeof(DIKEYBOARDDATA), chBuffer);

	while (hres == DIERR_INPUTLOST);
	{
		// da ist irgendwas beim Datentransfer schiefgegangen
		// mal versuchen, das Device wieder zu acquiren
		hres = Acquire();
		// wenn kein Fehlerchen dabei auftrat -> nochmal
		if (SUCCEEDED(hres))
			// ... Hallle!
			hres = lpDIDevice->GetDeviceState(sizeof(DIKEYBOARDDATA), 
						chBuffer);
	}

	return hres;
}

int CDirectInputDevice::GetBufferedKeyboardData(DIDEVICEOBJECTDATA* didod)
{
	HRESULT hres = DI_OK;

	// vorher mal Poll auslsen
	lpDIDevice->Poll();

	// erstes mal Informationen holen
	hres = lpDIDevice->GetDeviceData(sizeof(DIDEVICEOBJECTDATA),
		didod, &iBuffered, 0);

	while (hres == DIERR_INPUTLOST);
	{
		// da ist irgendwas beim Datentransfer schiefgegangen
		// mal versuchen, das Device wieder zu acquiren
		hres = Acquire();
		// wenn kein Fehlerchen dabei auftrat -> nochmal
		if (SUCCEEDED(hres))
			// ... Hallle!
			hres = lpDIDevice->GetDeviceData(sizeof(DIDEVICEOBJECTDATA),
				didod, &iBuffered, 0);
	}

	return hres;
}

int CDirectInputDevice::GetMouseData(DIMOUSESTATE* dims)
{
	HRESULT hres = DI_OK;

	// vorher mal Poll auslsen
	lpDIDevice->Poll();

	// erstes mal Informationen holen
	hres = lpDIDevice->GetDeviceState(sizeof(DIMOUSESTATE),	dims);

	while (hres == DIERR_INPUTLOST);
	{
		// da ist irgendwas beim Datentransfer schiefgegangen
		// mal versuchen, das Device wieder zu acquiren
		hres = Acquire();
		// wenn kein Fehlerchen dabei auftrat -> nochmal
		if (SUCCEEDED(hres))
			// ... Hallle!
			hres = lpDIDevice->GetDeviceState(sizeof(DIMOUSESTATE),	dims);
	}

	return hres;
}

int CDirectInputDevice::GetBufferedMouseData(DIDEVICEOBJECTDATA* didod)
{
	HRESULT hres = DI_OK;

	// vorher mal Poll auslsen
	lpDIDevice->Poll();

	// erstes mal Informationen holen
	hres = lpDIDevice->GetDeviceData(sizeof(DIDEVICEOBJECTDATA),
		didod, &iBuffered, 0);

	while (hres == DIERR_INPUTLOST);
	{
		// da ist irgendwas beim Datentransfer schiefgegangen
		// mal versuchen, das Device wieder zu acquiren
		hres = Acquire();
		// wenn kein Fehlerchen dabei auftrat -> nochmal
		if (SUCCEEDED(hres))
			// ... Hallle!
			hres = lpDIDevice->GetDeviceData(sizeof(DIDEVICEOBJECTDATA),
				didod, &iBuffered, 0);
	}

	return hres;
}

int CDirectInputDevice::GetJoystickData(DIJOYSTATE2* dijs)
{
	HRESULT hres = DI_OK;

	// vorher mal Poll auslsen
	lpDIDevice->Poll();

	// erstes mal Informationen holen
	hres = lpDIDevice->GetDeviceState(sizeof(DIJOYSTATE2), dijs);

	while (hres == DIERR_INPUTLOST);
	{
		// da ist irgendwas beim Datentransfer schiefgegangen
		// mal versuchen, das Device wieder zu acquiren
		hres = Acquire();
		// wenn kein Fehlerchen dabei auftrat -> nochmal
		if (SUCCEEDED(hres))
			// ... Hallle!
			hres = lpDIDevice->GetDeviceState(sizeof(DIJOYSTATE2), dijs);
	}

	return hres;
}

int CDirectInputDevice::GetBufferedJoystickData(DIDEVICEOBJECTDATA* didod)
{
	HRESULT hres = DI_OK;

	// vorher mal Poll auslsen
	lpDIDevice->Poll();

	// erstes mal Informationen holen
	hres = lpDIDevice->GetDeviceData(sizeof(DIDEVICEOBJECTDATA),
		didod, &iBuffered, 0);

	while (hres == DIERR_INPUTLOST);
	{
		// da ist irgendwas beim Datentransfer schiefgegangen
		// mal versuchen, das Device wieder zu acquiren
		hres = Acquire();
		// wenn kein Fehlerchen dabei auftrat -> nochmal
		if (SUCCEEDED(hres))
			// ... Hallle!
			hres = lpDIDevice->GetDeviceData(sizeof(DIDEVICEOBJECTDATA),
				didod, &iBuffered, 0);
	}

	return hres;
}

int CDirectInputDevice::GetCurrentData(LPVOID lpdid)
{
	// mal sehen, was fr ein Device wir gerade haben
	switch (iCurrentDevType)
	{
		case DIDEVTYPE_KEYBOARD:
			// gepufferte Eingabe?
			if (iBuffered)
				// ... dann diese holen
				return GetBufferedKeyboardData((DIDEVICEOBJECTDATA*)lpdid);
			else
				// ... sonst alles
				return GetKeyboardData((BYTE*)lpdid);
			break;
		case DIDEVTYPE_MOUSE:
			// gepufferte Eingabe?
			if (iBuffered)
				// ... dann diese holen
				return GetBufferedMouseData((DIDEVICEOBJECTDATA*)lpdid);
			else
				// ... sonst alles
				return GetMouseData((DIMOUSESTATE*)lpdid);
			break;
		case DIDEVTYPE_JOYSTICK:
			// gepufferte Eingabe?
			if (iBuffered)
				// ... dann diese holen
				return GetBufferedMouseData((DIDEVICEOBJECTDATA*)lpdid);
			else
				// Joystickdaten holen
				return GetJoystickData((DIJOYSTATE2*)lpdid);
		default:
			// hier gab's einen Fehler
			return DXDLL_INTERNALERROR;
	}
}

int CDirectInputDevice::RunControlPanel()
{
	// existiert ein Device?
	if (lpDIDevice)
		// ... dann anzeigen
		return lpDIDevice->RunControlPanel(pDirectInput->hWindow, 0);

	return DXDLL_INTERNALERROR;
}

int CDirectInputDevice::Acquire()
{
	return lpDIDevice->Acquire();
}

int CDirectInputDevice::Unacquire()
{
	return lpDIDevice->Unacquire();
}

