// DirectSound.cpp: Implementierung der Klasse CDirectSound.
//
//////////////////////////////////////////////////////////////////////


#include "DirectSound.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CDirectSound::CDirectSound()
{
	// alle Pointer resetten
	lpDS = NULL;
	pDSBPrimary = NULL;
	// und Hauptfenster resetten
	hWindow = NULL;
}

CDirectSound::CDirectSound(HWND hMainWindow)
{
	// alle Pointer resetten
	lpDS = NULL;
	pDSBPrimary = NULL;
	// und Hauptfenster speichern
	hWindow = hMainWindow;
}

CDirectSound::CDirectSound(HWND hMainWindow, DWORD dwDSFlags, DWORD dwDSBFlags)
{
	// alle Pointer resetten
	lpDS = NULL;
	pDSBPrimary = NULL;
	// und Hauptfenster speichern
	hWindow = hMainWindow;

	// ... jetzt DirectSound-Objekt erzeugen
	DirectSoundCreate();
	// ... und Kooperationsebene setzen
	SetCooperativeLevel(dwDSFlags);

	// ... als letztes den Primarybuffer erzeugen
	if ((pDSBPrimary = new CDirectSoundBuffer(this)))
		pDSBPrimary->CreatePrimaryDirectSoundBuffer(dwDSBFlags);
}

CDirectSound::CDirectSound(HWND hMainWindow, GUID* lpGUID, DWORD dwDSFlags, 
						   DWORD dwDSBFlags)
{
	// alle Pointer resetten
	lpDS = NULL;
	pDSBPrimary = NULL;
	// und Hauptfenster speichern
	hWindow = hMainWindow;

	// ... jetzt DirectSound-Objekt erzeugen
	DirectSoundCreate(lpGUID);
	// ... und Kooperationsebene setzen
	SetCooperativeLevel(dwDSFlags);

	// ... als letztes den Primarybuffer erzeugen
	if ((pDSBPrimary = new CDirectSoundBuffer(this)))
		pDSBPrimary->CreatePrimaryDirectSoundBuffer(dwDSBFlags);
}

CDirectSound::~CDirectSound()
{
	// ... jetzt alle Objekte lschen
	DirectSoundDestroy();
}

void CDirectSound::DirectSoundDestroy()
{
	// existiert das DS-Objekt?
	if (lpDS)
	{
		// ... dann alles lschen und resetten
		lpDS->Release();
		lpDS = NULL;
		// existiert Primarybuffer berhaupt?
		if (pDSBPrimary)
		{
			pDSBPrimary->lpDSB = NULL;
			delete pDSBPrimary;
			pDSBPrimary = NULL;
		}
	}
}

HWND CDirectSound::SetMainWindow(HWND hMainWindow)
{
	// altes Handle speichern
	HWND hAlt = hWindow;
	// neu belegen
	hWindow = hMainWindow;
	// und altes zurck
	return hAlt;
}


int CDirectSound::DirectSoundCreate()
{
	// erzeugen und raus aus Routine
	return DirectSoundCreate(NULL);
}

int CDirectSound::DirectSoundCreate(GUID* lpGUID)
{
	// altes vorher lschen
	DirectSoundDestroy();

	// DirectSound-Objekt erzeugen
	if (FAILED(::DirectSoundCreate(lpGUID, &lpDS, NULL)))
		// ebenfalls raus, falls Fehler
		return Fail(hWindow, DXDLL_MSGERROR_DSOUNDCREATE);

	return DXDLL_OK;
}

int CDirectSound::SetCooperativeLevel(DWORD dwFlags)
{
	// Kooperationsebene setzen
	if (FAILED(lpDS->SetCooperativeLevel(hWindow, dwFlags)))
		// verlief alles OK? Ansonsten Fehlermeldung
		return Fail(hWindow, DXDLL_MSGERROR_DSOUNDCOOPERATIVELEVEL);

	return DXDLL_OK;
}

CDirectSoundBuffer* CDirectSound::SetPrimarySoundBuffer(CDirectSoundBuffer *pDSB)
{
	// alten merken
	CDirectSoundBuffer* pAlt = pDSBPrimary;

	// ... neuen setzen
	pDSBPrimary = pDSB;
	// alten zurckliefern
	return pAlt;
}

CDirectSoundBuffer* CDirectSound::GetPrimarySoundBuffer()
{
	// zurckliefern
	return pDSBPrimary;
}
