// DirectSoundBuffer.cpp: Implementierung der Klasse CDirectSoundBuffer.
//
//////////////////////////////////////////////////////////////////////

#include "DirectSound.h"
#include "dsutil.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CDirectSoundBuffer::CDirectSoundBuffer()
{
	// alle Pointer resetten
	pDirectSound = NULL;
	lpDSB = NULL;
}

CDirectSoundBuffer::CDirectSoundBuffer(CDirectSound* pDS)
{
	// alle Pointer resetten
	lpDSB = NULL;
	// ... und DirectSound-Objekt bernehmen
	pDirectSound = pDS;
}

CDirectSoundBuffer::~CDirectSoundBuffer()
{
	// ... jetzt alle Objekte lschen
	DirectSoundBufferDestroy();
}

int CDirectSoundBuffer::CreateDirectSoundBuffer(DWORD dwSize)
{
	// ... jetzt alle Objekte lschen
	DirectSoundBufferDestroy();

	// DirectSound-Buffer erzeugen
	if (FAILED(pDirectSound->lpDS->CreateSoundBuffer(&dsbdesc, 
				&lpDSB, NULL)))
		return Fail(pDirectSound->hWindow, DXDLL_MSGERROR_DSOUNDBUFFERCREATE);

	// ... und keinen Fehler zurckliefern
	return DXDLL_OK;
}

int CDirectSoundBuffer::CreatePrimaryDirectSoundBuffer(DWORD dwFlags)
{
	// wenigstens mit DirectSound-Objekt verbunden?
	if (pDirectSound)
	{
		// vorher alten Buffer zerstren
		DirectSoundBufferDestroy();

		// Primary-Buffer fr DirectSound erzeugen
		SetBufferDescData(DSBCAPS_PRIMARYBUFFER | dwFlags, NULL);
		// Buffer erzeugen
		if (FAILED(pDirectSound->lpDS->CreateSoundBuffer(&dsbdesc, 
						&lpDSB, NULL)))
			// ... falls Fehler bei Erzeugung eben Meldung
			return Fail(pDirectSound->hWindow, 
				DXDLL_MSGERROR_DSOUNDPRIMBUFFERCREATE);

		return DXDLL_OK;
	}

	return DXDLL_INTERNALERROR;
}

void CDirectSoundBuffer::DirectSoundBufferDestroy()
{
	// existiert ein DS-Buffer?
	if (lpDSB)
	{
		// ... dann diesen zerstren
		lpDSB->Release();
		lpDSB = NULL;
	}
}

void CDirectSoundBuffer::SetBufferDescData(DWORD dwFlags, 
										   LPWAVEFORMATEX lpFormat)
{
	// ... erstmal alles leeren
	ZeroMemory(&dsbdesc, sizeof(DSBUFFERDESC));
	// ... und Eintragungen belegen
	dsbdesc.dwSize = sizeof(DSBUFFERDESC);
	dsbdesc.dwFlags = dwFlags;
	dsbdesc.lpwfxFormat = lpFormat;
}

void CDirectSoundBuffer::SetBufferDescData(DSBUFFERDESC *lpBufDesc)
{
	// ... und Inhalte bertragen
	CopyMemory(&dsbdesc, lpBufDesc, sizeof(DSBUFFERDESC));
}

int CDirectSoundBuffer::Play(DWORD dwFlags)
{
	// existiert Objekt?
	if (lpDSB)
		// ... und Buffer abspielen
		return lpDSB->Play(0, 0, dwFlags);
	else
		// ... ansonsten wegen Fehler abbrechen
		return DXDLL_INTERNALERROR;
}

int CDirectSoundBuffer::Stop()
{
	// existiert Objekt?
	if (lpDSB)
		// ... und Buffer abspielen
		return lpDSB->Stop();
	else
		// ... ansonsten wegen Fehler abbrechen
		return DXDLL_INTERNALERROR;
}

int CDirectSoundBuffer::LoadWAV(LPCTSTR lpName, LPCSTR lpWAVSection,
								DWORD dwFlags)
{
	// alten Buffer lschen
	DirectSoundBufferDestroy();

	// ... als nchstes ber Routine aus DSUTIL.C WAV-Datei laden
	if ((lpDSB = DSLoadSoundBuffer(NULL, pDirectSound->lpDS, lpName, 
									lpWAVSection, dwFlags)))
		// ... bei gltiger Adresse keinen Fehler zurckliefern
		return DXDLL_OK;
	else
		// ... ansonsten war irgend etwas falsch
		return DXDLL_INTERNALERROR;
}

int CDirectSoundBuffer::LoadWAV(HMODULE hModule, LPCTSTR lpName,
								LPCSTR lpWAVSection, DWORD dwFlags)
{
	// alten Buffer lschen
	DirectSoundBufferDestroy();

	// ... als nchstes ber Routine aus DSUTIL.C WAV-Datei laden
	if ((lpDSB = DSLoadSoundBuffer(hModule, pDirectSound->lpDS, lpName, 
									lpWAVSection, dwFlags)))
		// ... bei gltiger Adresse keinen Fehler zurckliefern
		return DXDLL_OK;
	else
		// ... ansonsten war irgend etwas falsch
		return DXDLL_INTERNALERROR;
}

