// DirectXDriverInfo.cpp: Implementierung der Klasse CDirectXDriverInfo.
//
//////////////////////////////////////////////////////////////////////


#include "DirectInfo.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CDirectXDriverInfo::CDirectXDriverInfo()
{
	// ... alles resetten
	lpDriverGUID = NULL;
	lpDescInfoHEL = NULL;
	lpDescInfoHAL = NULL;
	iDirectXDeviceType = DXI_UNKNOWN;
	iDirect3DDriversCount = 0;
	iDirectDrawResolutionsCount = 0;
}

CDirectXDriverInfo::CDirectXDriverInfo(GUID *lpGUID, LPSTR lpDescription,
									   LPSTR lpName, int iDXDeviceType,
									   LPVOID lpDescHEL, LPVOID lpDescHAL)
{
	// ... und nun vorgegebene Werte fr diesen Treiber setzen
	SetDriverInfo(lpGUID, lpDescription, lpName, iDXDeviceType,
		lpDescHEL, lpDescHAL);
}

CDirectXDriverInfo::~CDirectXDriverInfo()
{
	// bevor gelscht wird -> noch alle Infos lschen
	DestroyDriverInfo();
}

int CDirectXDriverInfo::SetDriverInfo(GUID *lpGUID, LPSTR lpDescription, 
									  LPSTR lpName, int iDXDeviceType,
									  LPVOID lpDescHEL, LPVOID lpDescHAL)
{
	// noch alte Beschreibung lschen
	DestroyDriverInfo();

	// ... und GUID neu belegen
	if (lpGUID)
	{
		// neuen Speicherbereich fr GUID erzeugen
		lpDriverGUID = (GUID*)malloc(sizeof(GUID));
		// und Daten aus bergebenen kopieren
		CopyMemory(lpDriverGUID, lpGUID, sizeof(GUID));
	}
	else
		// ... wenn primres Gert, dann NULL
		lpDriverGUID = NULL;

	// ... Info-Block fr HEL neu belegen
	lpDescInfoHEL = lpDescHEL;
	// ... Info-Block fr HAL neu belegen
	lpDescInfoHAL = lpDescHAL;

	// ... als nchstes Treiberbeschreibung kopieren
	if (lpDescription)
		// ... und kopieren
		lstrcpy(&lpDriverDescription[0], lpDescription);

	// ... zum Schlu noch Treibernamen kopieren
	if (lpName)
		// ... und kopieren
		lstrcpy(&lpDriverName[0], lpName);

	// ... als letztes noch Devicetype angeben
	iDirectXDeviceType = iDXDeviceType;

	return DXDLL_OK;
}

GUID* CDirectXDriverInfo::GetDriverGUID()
{
	return lpDriverGUID;
}

LPSTR CDirectXDriverInfo::GetDriverDescription()
{
	return lpDriverDescription;
}

LPSTR CDirectXDriverInfo::GetDriverName()
{
	return lpDriverName;
}

int CDirectXDriverInfo::GetDriverDeviceType()
{
	return iDirectXDeviceType;
}

LPVOID CDirectXDriverInfo::GetDriverDescInfoHEL()
{
	return lpDescInfoHEL;
}

LPVOID CDirectXDriverInfo::GetDriverDescInfoHAL()
{
	return lpDescInfoHAL;
}

void CDirectXDriverInfo::DestroyDriverInfo()
{
	// ... ist GUID mit Pointer belegt?
	if (lpDriverGUID)
	{
		// ... dann lschen
		delete lpDriverGUID;
		lpDriverGUID = NULL;
	}
	// ... ist Infoblock 1 mit Pointer belegt?
	if (lpDescInfoHEL)
	{
		// ... dann lschen
		delete lpDescInfoHEL;
		lpDescInfoHEL = NULL;
	}
	// ... ist Infoblock 2 mit Pointer belegt?
	if (lpDescInfoHAL)
	{
		// ... dann lschen
		delete lpDescInfoHAL;
		lpDescInfoHAL = NULL;
	}
	// sind Direct3D-Gerte vorhanden?
	if (GetDriverDeviceType() == DXI_DIRECTDRAW &&
		iDirect3DDriversCount)
	{
		// ... dann diese auch freigeben
		for (int i=1; i<=iDirect3DDriversCount; i++)
			// ... belegt?
			if (pD3DInfo[i])
			{
				// ... dann lschen
				delete pD3DInfo[i];
				pD3DInfo[i] = NULL;
			}
	}
	// sind verfgbare Auflsungen vorhanden?
	if (GetDriverDeviceType() == DXI_DIRECTDRAW &&
		iDirectDrawResolutionsCount)
	{
		// ... dann diese auch freigeben
		for (int i=1; i<=iDirectDrawResolutionsCount; i++)
			// ... belegt?
			if (lpResolutions[i])
			{
				// ... dann lschen
				delete lpResolutions[i];
				lpResolutions[i] = NULL;
			}
	}

	iDirect3DDriversCount = 0;
	iDirectDrawResolutionsCount = 0;
	// ... und nicht belegt angeben
	iDirectXDeviceType = DXI_UNKNOWN;
}

int CDirectXDriverInfo::AddDirect3DDriver(GUID *lpGUID, LPSTR lpDesc, LPSTR lpName,
										  LPVOID lpDescHEL, LPVOID lpDescHAL)
{
	// handlet es sich um ein DD-Gert
	if (GetDriverDeviceType() == DXI_DIRECTDRAW)
	{
		if (iDirect3DDriversCount < MAX_DRIVERS)
			// ... und diese Daten speichern
			pD3DInfo[++iDirect3DDriversCount] =
				new CDirectXDriverInfo(lpGUID, lpDesc, lpName,
						DXI_DIRECT3D, lpDescHEL, lpDescHAL);
		else
			// ... ansonsten mit Fehler raus
			return DXDLL_INTERNALERROR;
	}

	// ... kein Fehlerchen aufgetreten!
	return DXDLL_INTERNALERROR;
}

int CDirectXDriverInfo::AddDirectDrawResolution(DXDLL_DDSURFACEDESC* lpDesc)
{
	// handlet es sich um ein DD-Gert
	if (GetDriverDeviceType() == DXI_DIRECTDRAW)
	{
		if (iDirectDrawResolutionsCount < MAX_DRIVERS)
		{
			// ... und diese Daten speichern
			lpResolutions[++iDirectDrawResolutionsCount] =
				(DXDLL_DDSURFACEDESC*)malloc(sizeof(DXDLL_DDSURFACEDESC));
			memcpy(lpResolutions[iDirectDrawResolutionsCount],
				lpDesc, sizeof(DXDLL_DDSURFACEDESC));
		}
		else
			// ... ansonsten mit Fehler raus
			return DXDLL_INTERNALERROR;
	}

	// ... kein Fehlerchen aufgetreten!
	return DXDLL_INTERNALERROR;
}

CDirectXDriverInfo* CDirectXDriverInfo::GetD3DDriverInfo(int iDriver)
{
	if (GetDriverDeviceType() == DXI_DIRECTDRAW)
	{
		// gltiger Treiber selektiert?
		if (iDriver <= iDirect3DDriversCount && iDriver >= 1)
			// ... dann auch diesen zurckliefern
			return pD3DInfo[iDriver];
		else
			// ... ansonsten Fehler
			return NULL;
	}

	return NULL;
}

int CDirectXDriverInfo::GetD3DDriversCount()
{
	if (GetDriverDeviceType() == DXI_DIRECTDRAW)
		// ... gltigen zurckliefern
		return iDirect3DDriversCount;;

	return 0;
}

GUID* CDirectXDriverInfo::GetD3DDriverGUID(int iDriver)
{
	if (GetDriverDeviceType() == DXI_DIRECTDRAW)
	{
		// gltiger Treiber selektiert?
		if (iDriver <= iDirect3DDriversCount && iDriver >= 1)
			// ... dann auch diesen zurckliefern
			return pD3DInfo[iDriver]->GetDriverGUID();
		else
			// ... ansonsten Fehler
			return NULL;
	}

	return NULL;
}

LPSTR CDirectXDriverInfo::GetD3DDriverDescription(int iDriver)
{
	if (GetDriverDeviceType() == DXI_DIRECTDRAW)
	{
		// gltiger Treiber selektiert?
		if (iDriver <= iDirect3DDriversCount && iDriver >= 1)
			// ... dann auch diesen zurckliefern
			return pD3DInfo[iDriver]->GetDriverDescription();
		else
			// ... ansonsten Fehler
			return NULL;
	}

	return NULL;
}

LPSTR CDirectXDriverInfo::GetD3DDriverName(int iDriver)
{
	if (GetDriverDeviceType() == DXI_DIRECTDRAW)
	{
		// gltiger Treiber selektiert?
		if (iDriver <= iDirect3DDriversCount && iDriver >= 1)
			// ... dann auch diesen zurckliefern
			return pD3DInfo[iDriver]->GetDriverName();
		else
			// ... ansonsten Fehler
			return NULL;
	}

	return NULL;
}

LPVOID CDirectXDriverInfo::GetD3DDriverHEL(int iDriver)
{
	if (GetDriverDeviceType() == DXI_DIRECTDRAW)
	{
		// gltiger Treiber selektiert?
		if (iDriver <= iDirect3DDriversCount && iDriver >= 1)
			// ... dann auch diesen zurckliefern
			return pD3DInfo[iDriver]->GetDriverDescInfoHEL();
		else
			// ... ansonsten Fehler
			return NULL;
	}

	return NULL;
}

LPVOID CDirectXDriverInfo::GetD3DDriverHAL(int iDriver)
{
	if (GetDriverDeviceType() == DXI_DIRECTDRAW)
	{
		// gltiger Treiber selektiert?
		if (iDriver <= iDirect3DDriversCount && iDriver >= 1)
			// ... dann auch diesen zurckliefern
			return pD3DInfo[iDriver]->GetDriverDescInfoHAL();
		else
			// ... ansonsten Fehler
			return NULL;
	}

	return NULL;
}

DXDLL_DDSURFACEDESC* CDirectXDriverInfo::GetResolutionDesc(int iRes)
{
	if (GetDriverDeviceType() == DXI_DIRECTDRAW)
	{
		// gltige Auflsung selektiert?
		if (iRes <= iDirectDrawResolutionsCount && iRes >= 1)
			// ... dann auch diesen zurckliefern
			return lpResolutions[iRes];
		else
			// ... ansonsten Fehler
			return NULL;
	}

	return NULL;
}

int CDirectXDriverInfo::GetResolution(int iRes, int* iWidth,
									  int* iHeight, int* iBits, int* iRefresh)
{
	if (GetDriverDeviceType() == DXI_DIRECTDRAW)
	{
		// gltige Auflsung selektiert?
		if (iRes <= iDirectDrawResolutionsCount && iRes >= 1)
		{
			// Daten speichern
			if (iWidth)
				*iWidth = lpResolutions[iRes]->dwWidth;
			if (iHeight)
				*iHeight = lpResolutions[iRes]->dwHeight;
			if (iBits)
				*iBits = lpResolutions[iRes]->ddpfPixelFormat.dwRGBBitCount;
			if (iRefresh)
				*iRefresh = lpResolutions[iRes]->dwRefreshRate;

			// ... dann auch diesen zurckliefern
			return DXDLL_OK;
		}
		else
			// ... ansonsten Fehler
			return DXDLL_INTERNALERROR;
	}

	return DXDLL_INTERNALERROR;
}

int CDirectXDriverInfo::GetResolutionsCount()
{
	if (GetDriverDeviceType() == DXI_DIRECTDRAW)
		return iDirectDrawResolutionsCount;

	return DXDLL_INTERNALERROR;
}

