// DirectXInfo.cpp: Implementierung der Klasse CDirectXInfo.
//
//////////////////////////////////////////////////////////////////////
// nur vorhanden, um COM-Interfaces zu nutzen
#define INITGUID

#include "DirectInfo.h"
#include "dsetup.h"


BOOL WINAPI EnumDDrawDevice(GUID FAR* lpGUID, LPSTR lpDriverDescription,
							LPSTR lpDriverName, LPVOID lpContext)
{
	// dazugehrige Klasse holen
	CDirectXInfo* pInfo = (CDirectXInfo*)lpContext;

	// ist Infoklasse gltig?
	if (pInfo)
	{
		LPDIRECTDRAW lpDD = NULL;
		LPDDCAPS lpHEL = NULL;
		LPDDCAPS lpHAL = NULL;

		// Objekt zum Test einmal erzeugen
		if (SUCCEEDED(DirectDrawCreate(lpGUID, &lpDD, NULL)))
		{
			// Speicher fr beide Descriptoren holen und Teil belegen
			lpHAL = (LPDDCAPS)malloc(sizeof(DDCAPS));
			ZeroMemory(lpHAL, sizeof(DDCAPS));
			lpHAL->dwSize = sizeof(DDCAPS);

			lpHEL = (LPDDCAPS)malloc(sizeof(DDCAPS));
			ZeroMemory(lpHEL, sizeof(DDCAPS));
			lpHEL->dwSize = sizeof(DDCAPS);

			// jetzt Informationen dazu einholen
			if (SUCCEEDED(lpDD->GetCaps(lpHAL, lpHEL)))
				// ... Gert zu Liste hinzufgen
				pInfo->AddDirectXDriver(DXI_DIRECTDRAW, lpGUID, 
					lpDriverDescription, lpDriverName, (LPVOID)lpHAL, 
					(LPVOID)lpHEL);
			else
			{
				// wieder den Rest freigeben
				if (lpHAL)
					delete lpHAL;
				if (lpHEL)
					delete lpHEL;
			}

			// Gert wieder lschen
			lpDD->Release();
		}
	}

	// weiteres Device auflisten lassen
	return DDENUMRET_OK;
}

BOOL WINAPI EnumDDrawModes(LPDDSURFACEDESC2 lpDesc, LPVOID lpContext)
{
	// dazugehrige Klasse holen
	CDirectXDriverInfo* pInfo = (CDirectXDriverInfo*)lpContext;

	// ist Infoklasse gltig?
	if (pInfo)
		// Modus hinzufgen
		pInfo->AddDirectDrawResolution(lpDesc);

	// weiteres Device auflisten lassen
	return DDENUMRET_OK;
}

BOOL WINAPI EnumDSoundDevice(GUID FAR* lpGUID, LPCSTR lpcDriverDescription,
							 LPCSTR lpcDriverName, LPVOID lpContext)
{
	// dazugehrige Klasse holen
	CDirectXInfo* pInfo = (CDirectXInfo*)lpContext;
	LPDIRECTSOUND lpDS = NULL;
	LPDSCAPS lpDesc = NULL;

	// ist Infoklasse gltig?
	if (pInfo)
	{
		// Objekt zum Test einmal erzeugen
		if (SUCCEEDED(DirectSoundCreate(lpGUID, &lpDS, NULL)))
		{
			// Speicher fr beide Descriptoren holen und Teil belegen
			lpDesc = (LPDSCAPS)malloc(sizeof(DSCAPS));
			ZeroMemory(lpDesc, sizeof(DSCAPS));
			lpDesc->dwSize = sizeof(DSCAPS);

			// jetzt Informationen dazu einholen
			if (SUCCEEDED(lpDS->GetCaps(lpDesc)))
				// ... Gert zu Liste hinzufgen
				pInfo->AddDirectXDriver(DXI_DIRECTSOUND, lpGUID, 
					(LPSTR)lpcDriverDescription, (LPSTR)lpcDriverName,
					(LPVOID)lpDesc, NULL);

			// Objekt wiedern auslschen
			lpDS->Release();
		}
	}


	// weiteres Device auflisten lassen
	return DDENUMRET_OK;
}

BOOL WINAPI EnumD3DDevice(GUID FAR* lpGUID, LPSTR lpDriverDescription,
						  LPSTR lpDriverName, LPD3DDEVICEDESC lpHWDesc,
						  LPD3DDEVICEDESC lpHELDesc, LPVOID lpContext)
{
	// dazugehrige Klasse holen
	CDirectXDriverInfo* pInfo = (CDirectXDriverInfo*)lpContext;
	LPD3DDEVICEDESC lpHEL = NULL;
	LPD3DDEVICEDESC lpHAL = NULL;

	// ist Infoklasse gltig?
	if (pInfo)
	{
		// jetzt beide Descriptorblcke kopieren
		if (lpHELDesc)
		{
			lpHEL = (LPD3DDEVICEDESC)malloc(sizeof(D3DDEVICEDESC));
			memcpy(lpHEL, lpHELDesc, sizeof(D3DDEVICEDESC));
		}
		if (lpHWDesc)
		{
			lpHAL = (LPD3DDEVICEDESC)malloc(sizeof(D3DDEVICEDESC));
			memcpy(lpHAL, lpHWDesc, sizeof(D3DDEVICEDESC));
		}

		// ... Gert zu Liste hinzufgen
		pInfo->AddDirect3DDriver(lpGUID, lpDriverDescription, 
			lpDriverName, (LPVOID)lpHEL, (LPVOID)lpHAL);
	}

	// weiteres Device auflisten lassen
	return D3DENUMRET_OK;
}

BOOL WINAPI EnumDIDevice(LPCDIDEVICEINSTANCE lpddi, LPVOID lpContext)
{
	// dazugehrige Klasse holen
	CDirectXInfo* pInfo = (CDirectXInfo*)lpContext;
	LPDIDEVICEINSTANCE lpDescDI;

	// ist Infoklasse gltig?
	if (pInfo)
	{
		// Descriptorinstance gltig?
		if (lpddi)
		{
			// GUID ermittelt
			GUID guid = lpddi->guidInstance;

			// ... und Descriptorblock kopieren
			lpDescDI = (LPDIDEVICEINSTANCE)malloc(sizeof(DIDEVICEINSTANCE));
			memcpy(lpDescDI, lpddi, sizeof(DIDEVICEINSTANCE));

			// ... Gert zu Liste hinzufgen
			pInfo->AddDirectXDriver(DXI_DIRECTINPUT, &guid, 
				(LPSTR)lpddi->tszProductName, (LPSTR)lpddi->tszInstanceName,
				(LPVOID)lpDescDI, NULL);
		}
	}

	// weiteres Device auflisten lassen
	return DIENUM_CONTINUE;
}

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CDirectXInfo::CDirectXInfo()
{
	// kein einziger Treiber abgezhlt
	iDirectDrawDriversCount = 0;
	iDirectSoundDriversCount = 0;
	iDirectInputDriversCount = 0;

	// alle sonstigen auch resetten
	iMainVersion = 0;
	iSecondVersion = 0;
	iMainRevision = 0;
	iSecondRevision = 0;
}

CDirectXInfo::~CDirectXInfo()
{
	// alle mglichen Einstellungen lschen
	DestroyDirectXInfos();
}

int CDirectXInfo::EnumerateDirectXDrivers(int iInterface, LPVOID lpLParam,
										  LPVOID lpHParam)
{
	HRESULT hres = DXDLL_INTERNALERROR;
	// Gerteinfo holen
	CDirectXDriverInfo* pInfo = (CDirectXDriverInfo*)lpHParam;
	LPDIRECTDRAW lpDD;
	DXDLL_LPDIRECTDRAW lpDDNeu;
	LPGUID lpGUID;

	switch (iInterface)
	{
		// alle DirectDraw-Gerte aufzhlen
		case DXI_DIRECTDRAW:
			// Gerte auflisten lassen
			hres = DirectDrawEnumerate((DXDLL_LPDDENUMCALLBACK)EnumDDrawDevice, 
				(LPVOID)this);
			// bei Erfolg noch die Auflsungen auflisten
			if (SUCCEEDED(hres) && iDirectDrawDriversCount)
			{
				for (int i=1; i<=iDirectDrawDriversCount; i++)
				{
					// Treiber gltig?
					if (pDirectDrawInfo[i])
					{
						// GUID holen
						lpGUID = pDirectDrawInfo[i]->GetDriverGUID();
						// ... jetzt DD-Objekt erzeugen
						if (SUCCEEDED(DirectDrawCreate(lpGUID, &lpDD, NULL)))
						{
							// noch neuestes Objekt erzeugen
							if (SUCCEEDED(lpDD->QueryInterface(
									DXDLL_GUID_DIRECTDRAW, (LPVOID*)&lpDDNeu)))
							{
								// ... und auflisten
								hres = lpDDNeu->EnumDisplayModes(0, NULL,
									(void*)pDirectDrawInfo[i], 
									(DXDLL_LPDDENUMMODESCALLBACK)EnumDDrawModes);
								// das neue zerstren
								lpDDNeu->Release();
							}
							// Objekt wieder zerstren
							lpDD->Release();
						}
					}
				}
			}

			return hres;
			break;
		// alle DirectSound-Gerte aufzhlen
		case DXI_DIRECTSOUND:
			// Gerte auflisten lassen
			return DirectSoundEnumerate((DXDLL_LPDSENUMCALLBACK)EnumDSoundDevice, 
				(LPVOID)this);
			break;
		// alle Direct3D-Gerte aufzhlen
		case DXI_DIRECT3D:
			LPDIRECT3D lpD3D;

			// Gerteinfo belegt?
			if (pInfo)
			{
				// auch ein DD-Gert?
				if (pInfo->GetDriverDeviceType() == DXI_DIRECTDRAW)
					// ... dann diesen GUID holen
					lpGUID = pInfo->GetDriverGUID();
				else
					// sonst mit Fehler raus
					return DXDLL_INTERNALERROR;
			}
			else
				// eben auch raus
				return DXDLL_INTERNALERROR;

			// ... also mal das bentigte Handle fr DD erstellen
			if (SUCCEEDED(hres = DirectDrawCreate(lpGUID, &lpDD, NULL)))
			{
				// ... jetzt ein Interface auf D3D holen
				if (SUCCEEDED(hres = lpDD->QueryInterface(IID_IDirect3D,
						(void**) &lpD3D)))
				{
					// endlich kommen wir mal zum aufzhlen!
					hres = lpD3D->EnumDevices((DXDLL_LPD3DENUMDEVICESCALLBACK)
						EnumD3DDevice, (LPVOID)pInfo);
					// ... danach wieder alles freigeben
					lpD3D->Release();
				}
				// DD-Objekt wieder freigeben
				lpDD->Release();
			}

			// ... und nun wieder verlassen
			return hres;
			break;
		// alle DirectInput-Gerte aufzeichnen
		case DXI_DIRECTINPUT:
			// zum auflisten aller DirectInput-Gerte wird vorher
			// ein ganz normales DirectInput-Objekt bentigt, da
			// das aufzhlen nur ber dieses luft
			LPDIRECTINPUT lpDI;
			
			// als erstes das Objekt erzeugen
			if (SUCCEEDED(DirectInputCreate((HINSTANCE)lpLParam, 
							DIRECTINPUT_VERSION, &lpDI, NULL)))
			{
				// jetzt alles abzhlen
				hres = lpDI->EnumDevices(0, (DXDLL_LPDIENUMDEVICESCALLBACK)
					EnumDIDevice, (LPVOID)this, DIEDFL_ALLDEVICES);
				// und DirectInput wieder lschen
				lpDI->Release();
			}

			// wieder verlassen
			return hres;
			break;
		// alle Gerte aufzhlen
		case DXI_ALLDEVICES:
			// luft alles ok?
			if (SUCCEEDED(EnumerateDirectXDrivers(DXI_DIRECTDRAW, 
							NULL, NULL)) &&
				SUCCEEDED(EnumerateDirectXDrivers(DXI_DIRECTSOUND, 
							NULL, NULL)) &&
				SUCCEEDED(EnumerateDirectXDrivers(DXI_DIRECTINPUT, 
							lpLParam, NULL)))
			{
				// jetzt zu allen DirectDraw-Gerten die D3D-Gerte auflisten
				if (GetDirectXDriversCount(DXI_DIRECTDRAW))
					// ... und Beginn ...
					for (int i=1; i<=GetDirectXDriversCount(DXI_DIRECTDRAW); i++)
						// ... der Auflistung
						if (FAILED(EnumerateDirectXDrivers(DXI_DIRECT3D,
								NULL, (void*)GetDirectXDriverInfo(
								DXI_DIRECTDRAW, i))))
								// bei Fehler raus
								return DXDLL_INTERNALERROR;

				return DXDLL_OK;
			}
			break;
	}

	// ... ansonsten mit Fehler raus
	return DXDLL_INTERNALERROR;
}

void CDirectXInfo::DestroyDirectXInfos()
{
	int i;

	// ... zuerst alle DirectDraw-Gerte lschen
	if (iDirectDrawDriversCount > 0)
		for (i=1; i<iDirectDrawDriversCount; i++)
			// aktueller Pointer gltig?
			if (pDirectDrawInfo[i])
			{
				// ... dann diesen lschen
				delete pDirectDrawInfo[i];
				pDirectDrawInfo[i] = NULL;
			}


	// ... jetzt alle DirectSound-Gerte lschen
	if (iDirectSoundDriversCount > 0)
		for (i=1; i<iDirectSoundDriversCount; i++)
			// aktueller Pointer gltig?
			if (pDirectSoundInfo[i])
			{
				// ... dann diesen lschen
				delete pDirectSoundInfo[i];
				pDirectSoundInfo[i] = NULL;
			}

	// ... auch noch alle DirectInput-Gerte lschen
	if (iDirectInputDriversCount > 0)
		for (i=1; i<iDirectInputDriversCount; i++)
			// aktueller Pointer gltig?
			if (pDirectInputInfo[i])
			{
				// ... dann diesen lschen
				delete pDirectInputInfo[i];
				pDirectInputInfo[i] = NULL;
			}

	// noch die Anzahl komplett resetten
	iDirectDrawDriversCount = 0;
	iDirectSoundDriversCount = 0;
	iDirectInputDriversCount = 0;
}


int CDirectXInfo::AddDirectXDriver(int iInterface, GUID *lpGUID, 
								   LPSTR lpDesc, LPSTR lpName,
								   LPVOID lpDescHEL, LPVOID lpDescHAL)
{
	switch (iInterface)
	{
		case DXI_DIRECTDRAW:
			if (iDirectDrawDriversCount < MAX_DRIVERS)
				// ... und diese Daten speichern
				pDirectDrawInfo[++iDirectDrawDriversCount] =
					new CDirectXDriverInfo(lpGUID, lpDesc, lpName,
							DXI_DIRECTDRAW, lpDescHEL, lpDescHAL);
			else
				// ... ansonsten mit Fehler raus
				return DXDLL_INTERNALERROR;
			break;
		case DXI_DIRECTSOUND:
			if (iDirectSoundDriversCount < MAX_DRIVERS)
				// ... und diese Daten speichern
				pDirectSoundInfo[++iDirectSoundDriversCount] =
					new CDirectXDriverInfo(lpGUID, lpDesc, lpName,
							DXI_DIRECTSOUND, lpDescHEL, lpDescHAL);
			else
				// ... ansonsten mit Fehler raus
				return DXDLL_INTERNALERROR;
			break;
		case DXI_DIRECTINPUT:
			if (iDirectInputDriversCount < MAX_DRIVERS)
				// ... und diese Daten speichern
				pDirectInputInfo[++iDirectInputDriversCount] =
					new CDirectXDriverInfo(lpGUID, lpDesc, lpName,
							DXI_DIRECTINPUT, lpDescHEL, lpDescHAL);
			else
				// ... ansonsten mit Fehler raus
				return DXDLL_INTERNALERROR;
			break;
	}

	// ... kein Fehlerchen aufgetreten!
	return DXDLL_INTERNALERROR;
}

CDirectXDriverInfo* CDirectXInfo::GetDirectXDriverInfo(int iInterface,
													   int iDriver)
{
	switch (iInterface)
	{
		case DXI_DIRECTDRAW:
			// gltiger Treiber selektiert?
			if (iDriver <= iDirectDrawDriversCount && iDriver >= 1)
				// ... dann auch diesen zurckliefern
				return pDirectDrawInfo[iDriver];
			else
				// ... ansonsten Fehler
				return NULL;
			break;
		case DXI_DIRECTSOUND:
			// gltiger Treiber selektiert?
			if (iDriver <= iDirectSoundDriversCount && iDriver >= 1)
				// ... dann auch diesen zurckliefern
				return pDirectSoundInfo[iDriver];
			else
				// ... ansonsten Fehler
				return NULL;
			break;
		case DXI_DIRECTINPUT:
			// gltiger Treiber selektiert?
			if (iDriver <= iDirectInputDriversCount && iDriver >= 1)
				// ... dann auch diesen zurckliefern
				return pDirectInputInfo[iDriver];
			else
				// ... ansonsten Fehler
				return NULL;
			break;
	}

	// ... ansonsten eben ungltig!
	return NULL;
}

int CDirectXInfo::GetDirectXDriversCount(int iInterface)
{
	switch (iInterface)
	{
		case DXI_DIRECTDRAW:
			return iDirectDrawDriversCount;
			break;
		case DXI_DIRECTSOUND:
			return iDirectSoundDriversCount;
			break;
		case DXI_DIRECTINPUT:
			return iDirectInputDriversCount;
			break;
	}

	// ... da ungltig -> keine!
	return 0;
}

GUID* CDirectXInfo::GetDirectXDriverGUID(int iInterface, int iDriver)
{
	switch (iInterface)
	{
		case DXI_DIRECTDRAW:
			// gltiger Treiber selektiert?
			if (iDriver <= iDirectDrawDriversCount && iDriver >= 1)
				// ... dann auch diesen zurckliefern
				return pDirectDrawInfo[iDriver]->GetDriverGUID();
			else
				// ... ansonsten Fehler
				return NULL;
			break;
		case DXI_DIRECTSOUND:
			// gltiger Treiber selektiert?
			if (iDriver <= iDirectSoundDriversCount && iDriver >= 1)
				// ... dann auch diesen zurckliefern
				return pDirectSoundInfo[iDriver]->GetDriverGUID();
			else
				// ... ansonsten Fehler
				return NULL;
			break;
		case DXI_DIRECTINPUT:
			// gltiger Treiber selektiert?
			if (iDriver <= iDirectInputDriversCount && iDriver >= 1)
				// ... dann auch diesen zurckliefern
				return pDirectInputInfo[iDriver]->GetDriverGUID();
			else
				// ... ansonsten Fehler
				return NULL;
			break;
	}

	// ... ungltig, da unbekannter Interface
	return NULL;
}

LPSTR CDirectXInfo::GetDirectXDriverDescription(int iInterface, int iDriver)
{
	switch (iInterface)
	{
		case DXI_DIRECTDRAW:
			// gltiger Treiber selektiert?
			if (iDriver <= iDirectDrawDriversCount && iDriver >= 1)
				// ... dann auch diesen zurckliefern
				return pDirectDrawInfo[iDriver]->GetDriverDescription();
			else
				// ... ansonsten Fehler
				return NULL;
			break;
		case DXI_DIRECTSOUND:
			// gltiger Treiber selektiert?
			if (iDriver <= iDirectSoundDriversCount && iDriver >= 1)
				// ... dann auch diesen zurckliefern
				return pDirectSoundInfo[iDriver]->GetDriverDescription();
			else
				// ... ansonsten Fehler
				return NULL;
			break;
		case DXI_DIRECTINPUT:
			// gltiger Treiber selektiert?
			if (iDriver <= iDirectInputDriversCount && iDriver >= 1)
				// ... dann auch diesen zurckliefern
				return pDirectInputInfo[iDriver]->GetDriverDescription();
			else
				// ... ansonsten Fehler
				return NULL;
			break;
	}

	// ... ungltig, da unbekannter Interface
	return NULL;
}

LPSTR CDirectXInfo::GetDirectXDriverName(int iInterface, int iDriver)
{
	switch (iInterface)
	{
		case DXI_DIRECTDRAW:
			// gltiger Treiber selektiert?
			if (iDriver <= iDirectDrawDriversCount && iDriver >= 1)
				// ... dann auch diesen zurckliefern
				return pDirectDrawInfo[iDriver]->GetDriverName();
			else
				// ... ansonsten Fehler
				return NULL;
			break;
		case DXI_DIRECTSOUND:
			// gltiger Treiber selektiert?
			if (iDriver <= iDirectSoundDriversCount && iDriver >= 1)
				// ... dann auch diesen zurckliefern
				return pDirectSoundInfo[iDriver]->GetDriverName();
			else
				// ... ansonsten Fehler
				return NULL;
			break;
		case DXI_DIRECTINPUT:
			// gltiger Treiber selektiert?
			if (iDriver <= iDirectInputDriversCount && iDriver >= 1)
				// ... dann auch diesen zurckliefern
				return pDirectInputInfo[iDriver]->GetDriverName();
			else
				// ... ansonsten Fehler
				return NULL;
			break;
	}

	// ... ungltig, da unbekannter Interface
	return NULL;
}

LPVOID CDirectXInfo::GetDirectXDriverHEL(int iInterface, int iDriver)
{
	switch (iInterface)
	{
		case DXI_DIRECTDRAW:
			// gltiger Treiber selektiert?
			if (iDriver <= iDirectDrawDriversCount && iDriver >= 1)
				// ... dann auch diesen zurckliefern
				return pDirectDrawInfo[iDriver]->GetDriverDescInfoHEL();
			else
				// ... ansonsten Fehler
				return NULL;
			break;
		case DXI_DIRECTSOUND:
			// gltiger Treiber selektiert?
			if (iDriver <= iDirectSoundDriversCount && iDriver >= 1)
				// ... dann auch diesen zurckliefern
				return pDirectSoundInfo[iDriver]->GetDriverDescInfoHEL();
			else
				// ... ansonsten Fehler
				return NULL;
			break;
		case DXI_DIRECTINPUT:
			// gltiger Treiber selektiert?
			if (iDriver <= iDirectInputDriversCount && iDriver >= 1)
				// ... dann auch diesen zurckliefern
				return pDirectInputInfo[iDriver]->GetDriverDescInfoHEL();
			else
				// ... ansonsten Fehler
				return NULL;
			break;
	}

	// ... ungltig, da unbekannter Interface
	return NULL;
}

LPVOID CDirectXInfo::GetDirectXDriverHAL(int iInterface, int iDriver)
{
	switch (iInterface)
	{
		case DXI_DIRECTDRAW:
			// gltiger Treiber selektiert?
			if (iDriver <= iDirectDrawDriversCount && iDriver >= 1)
				// ... dann auch diesen zurckliefern
				return pDirectDrawInfo[iDriver]->GetDriverDescInfoHAL();
			else
				// ... ansonsten Fehler
				return NULL;
			break;
		case DXI_DIRECTSOUND:
			// gltiger Treiber selektiert?
			if (iDriver <= iDirectSoundDriversCount && iDriver >= 1)
				// ... dann auch diesen zurckliefern
				return pDirectSoundInfo[iDriver]->GetDriverDescInfoHAL();
			else
				// ... ansonsten Fehler
				return NULL;
			break;
		case DXI_DIRECTINPUT:
			// gltiger Treiber selektiert?
			if (iDriver <= iDirectInputDriversCount && iDriver >= 1)
				// ... dann auch diesen zurckliefern
				return pDirectInputInfo[iDriver]->GetDriverDescInfoHAL();
			else
				// ... ansonsten Fehler
				return NULL;
			break;
	}

	// ... ungltig, da unbekannter Interface
	return NULL;
}

int CDirectXInfo::FindDirectXVersion()
{
	DWORD dwVersion;
	DWORD dwRevision;
	int ires;

	// jetzt Versionsnummer holen
	if (ires = DirectXSetupGetVersion(&dwVersion, &dwRevision))
	{
		// und jetzt die Nummern holen
		iMainVersion = HIWORD(dwVersion);
		iSecondVersion = LOWORD(dwVersion);
		iMainRevision = HIWORD(dwRevision);
		iSecondRevision = LOWORD(dwRevision);
	}
	else
	{
		// bei Fehler alle Versionsnummern resetten
		iMainVersion = 0;
		iSecondVersion = 0;
		iMainRevision = 0;
		iSecondRevision = 0;
	}

	// Fehlercode zurckliefern
	return ires;
}

void CDirectXInfo::GetDirectXVersion(int* lpiMainVersion,
									 int* lpiSecondVersion,
									 int* lpiMainRevision,
									 int* lpiSecondRevision)
{
	// soll die Hauptversionsnummer geholt werden?
	if (lpiMainVersion)
		// ... dann holen
		*lpiMainVersion = iMainVersion;
	// soll die Unterversionsnummer geholt werden?
	if (lpiSecondVersion)
		// ... dann holen
		*lpiSecondVersion = iSecondVersion;
	// soll die Hauptrevisionsnummer geholt werden?
	if (lpiMainRevision)
		// ... dann holen
		*lpiMainRevision = iMainRevision;
	// soll die Unterrevisionsnummer geholt werden?
	if (lpiSecondRevision)
		// ... dann holen
		*lpiSecondRevision = iSecondRevision;
}

int CDirectXInfo::CheckCurrentVersion()
{
	// mal schauen, ob die Hauptversionsnummer min. der DLL entspricht
	if (iSecondVersion >= DXDLL_DIRECTX_VERSION)
		// ... alles klar!
		return DXDLL_OK;

	// ... ansonsten Fehler ausgeben
	Fail(NULL, DXDLL_MSGERROR_DXVERSIONINCOMPATIBLE);
	// und Fehler zurckliefern
	return DXDLL_INTERNALERROR;
}

