#define WIN32_LEAN_AND_MEAN

#include "Tools.h"
#include "DXDefines.h"


// Handle zur LOG-Datei
HFILE __declspec(dllexport) hLOGFile = NULL;

int Fail(HWND hwnd, char* Msg)
{
	// Messagebox mit Fehlermeldung anzeigen
	MessageBox(hwnd, Msg, "Error", MB_OK);
	// Fenster wieder zerstren
	DestroyWindow(hwnd);
	// und einen Fehlercode zurckliefern
	return DXDLL_INTERNALERROR;
}

bool FileExists(LPCSTR pFilename)
{
	int fh;

	// Datei ffnen
	fh = _open(pFilename, _O_RDONLY);
	// beim ffnen ein Fehler aufgetreten?
	if (fh == -1)
		// ... nicht vorhanden oder kein Zugriff
		return false;
	// ... ansonsten normal schlieen
	_close(fh);
	// und beenden
	return true;
}

double Potenz(double dZahl, int iExponent)
{
	double dNew = dZahl;
	int i;

	for (i=1; i<abs(iExponent); i++)
		dNew = dNew* dZahl;

	if (iExponent < 0)
		return 1/ dNew;
	else
		if (iExponent > 0)
			return dNew;
		else
			return 1;

	return 0;
}

double round(double dZahl, int iStellen)
{
	double dZehn = Potenz(10, iStellen);
	double dNew = int(dZahl* dZehn);
	double dOld = dZahl* dZehn;
	double dRest;
	int dKorr = 1;

	if (dZahl < 0)
		dKorr = -1;

	dRest = fabs(dOld- dNew);

	if (dRest >= 0.5)
		return (dNew+ dKorr)/ dZehn;
	else
		return dNew/ dZehn;

	return 0;
}

int CreateLOGFile()
{
	// existiert LOG-Datei?
	if (hLOGFile)
		// ... dann schlieen
		hLOGFile = _lclose(hLOGFile);

	// Datei ffnen
	hLOGFile = _lcreat("c:\\directx.log", 0);

	// ist LOG-File gltig?
	if (hLOGFile != HFILE_ERROR)
		// ... kein Fehler
		return DXDLL_OK;

	// noch leeren und dann raus
	hLOGFile = NULL;
	return Fail(NULL, DXDLL_MSGERROR_LOGFILECREATE);
}

int CreateLOGFile(LPCSTR lpcFilename)
{
	// existiert LOG-Datei?
	if (hLOGFile)
		// ... dann schlieen
		hLOGFile = _lclose(hLOGFile);

	// Datei ffnen
	hLOGFile = _lcreat(lpcFilename, 0);

	// ist LOG-File gltig?
	if (hLOGFile != HFILE_ERROR)
		// ... kein Fehler
		return DXDLL_OK;

	LPSTR lpStr = "Couldn't create user defined LOG file (";

	// die Fehlermeldung zusammenbasteln
	strcat(lpStr, lpcFilename);
	strcat(lpStr, ")!");

	// noch leeren und dann raus
	hLOGFile = NULL;
	return Fail(NULL, lpStr);
}

int CloseLOGFile()
{
	// existiert LOG-Datei?
	if (hLOGFile)
	{
		// ... dann schlieen
		hLOGFile = _lclose(hLOGFile);
		// war alles ok?
		if (!hLOGFile)
			// ... keine Probleme
			return DXDLL_OK;
	}

	// raus aus Routine
	return Fail(NULL, DXDLL_MSGERROR_LOGFILECLOSE);
}

int WriteLOGLine(LPCSTR lpcStr)
{
	// ist ein String angelegt?
	if (lpcStr)
	{
		// existiert LOG-Handle?
		if (hLOGFile)
			// ... dann schreiben
			return _lwrite(hLOGFile, lpcStr, strlen(lpcStr));
			
	}

	// ... ansonsten immer Fehler
	return DXDLL_INTERNALERROR;
}

int WriteLOGHResult(HRESULT hres)
{
	char hresult[256];

	// ... umwandeln
	wsprintf(hresult, "%lu", hres);
	// und ausgeben
	return WriteLOGLine(hresult);
}

int WriteLOGEndSection()
{
	// eine Linie mit anderer Routine zeichnen
	return WriteLOGLine("==========================================="
				"===================================\n");
}

int WriteLOGSoftEndSection()
{
	// eine Linie mit anderer Routine zeichnen
	return WriteLOGLine("-------------------------------------------"
				"-----------------------------------\n");
}

